--

creditsScreen = {}
creditsScreen_mt = { __index = creditsScreen }

function creditsScreen.create()
	local instance = {}
	setmetatable( instance, creditsScreen_mt)
	instance.scene = director:createScene()
	instance.scene.name = "Scene: creditsScreen"
	instance.scene:addEventListener({ "setUp" , "tearDown"}, instance)
	return instance
end

function creditsScreen:backKey(event)
	dbg.print("creditsScreen:backKey")
	dbg.printTable(event)
	system:sendEvent("onBackToMainMenu")
end

function creditsScreen:setUp(event)
    dbg.print("creditsScreen:setUp")
	dialogManager:setUp()
    -- Create a scene node for easy cleanup
	self.node = director:createNode()
	self.scene:addChild(self.node)

    -- Create the background
	self.bg = director:createSprite({ 
        source = "Assets/UI/level_select_screen.jpg",
        x = director.displayCenterX,
        y = director.displayCenterY,
        xAnchor = 0.5,
        yAnchor = 0.5,
        xScale = UI_SCALE,
        yScale = UI_SCALE
    })
--	self.bg:addEventListener("touch", self)
	self.node:addChild(self.bg)

    -- Create the back button
	self.backButton = director:createSprite({
        source = "Assets/UI/back_arrow.png",
        x = 16 * UI_SCALE,
        y = director.displayHeight - (16 * UI_SCALE),
        xAnchor = 0,
        yAnchor = 1,
        xScale = UI_SCALE,
        yScale = UI_SCALE,
        zOrder = 10
    })
	SpriteButton.addAnimatedButtonToSprite(self.backButton, "Assets/Audio/SFX_Back.raw")
	self.node:addChild(self.backButton)

    -- Create the title label
	local title = director:createLabel({
        x = 0, y = self.backButton.y,
        w = director.displayWidth, h = self.backButton.h * UI_SCALE,
        textXScale = UI_SCALE * 0.7, textYScale = UI_SCALE * 0.7,
        xAnchor = 0, yAnchor = 1,
        text = getString(S_MENU_CREDITS_TITLE),
        font = GAME_FONT_TITLE,
        color = GAME_FONT_COLOR,
        hAlignment="centre", vAlignment="middle",
    })
	self.node:addChild(title)

    -- Create a node to attach all the credits to so we can simply scroll them all up the screen
	self.scrollNode = director:createNode()
	self.scene:addChild(self.scrollNode)

	self.scrollNode.x = director.displayCenterX
	self.scrollNode.y = 0
	self.scrollNode.zOrder = 10

	self.scrollNode.xScale = UI_SCALE
	self.scrollNode.yScale = UI_SCALE
	
	self.creditsText = {}
	
	local stringID = 1
	
	while S_MENU_CREDITS[stringID] ~= nil do
	
		self.creditsText[stringID] = director:createLabel({
			x = 0, y = -28 * stringID,
            w = director.displayWidth/self.scrollNode.xScale, h=28,
			textXScale = 0.4, textYScale = 0.4,
            xAnchor=0.5, yAnchor=0.5,
			alpha = 0,
			text = getString(S_MENU_CREDITS[stringID]),
			font = GAME_FONT,
			color = GAME_FONT_COLOR,
			hAlignment="centre", vAlignment="middle"
		})

		self.scrollNode:addChild(self.creditsText[stringID])
		stringID = stringID + 1
	end
	
	-- How high should we scroll the node?
	self.scrollNode.scrollHeight = (28 * stringID * UI_SCALE) + director.displayHeight + 64
	self.scrollNode.scrollTime = stringID
	
	tween:to(self.scrollNode, { y = self.scrollNode.scrollHeight, time = self.scrollNode.scrollTime, delay = 0.5, onComplete=creditsScreen.resetCredits })
	
	system:addEventListener({"update", "backKey"}, self)
	system:addEventListener("buttonActionPerformed", self)
end

function creditsScreen:update(event)

	for i = 1, #self.creditsText do
	
		if self.creditsText[i].y * UI_SCALE + self.scrollNode.y > director.displayHeight - (192 * UI_SCALE) then
		
			if self.creditsText[i].alpha == 1 then
			
				tween:to(self.creditsText[i], { alpha = 0, time = 1 })
			end
	
		elseif self.creditsText[i].y * UI_SCALE + self.scrollNode.y > 32 * UI_SCALE then

			if self.creditsText[i].alpha == 0 then
			
				tween:to(self.creditsText[i], { alpha = 1, time = 1 })
			end
		end
	end
end

function creditsScreen.resetCredits(target)
	target.y = 0
	tween:to(target, { y = target.scrollHeight, time = target.scrollTime, onComplete=creditsScreen.resetCredits })
end

function creditsScreen:tearDown(event)
    dbg.print("creditsScreen:tearDown")
	dialogManager:tearDown()
	system:removeEventListener({"update", "backKey"}, self)
	system:removeEventListener("buttonActionPerformed", self)
	SpriteButton.tearDown()

    self.bg = nil
    self.backButton = nil
	self.scrollNode:removeFromParent()
	self.scrollNode = nil
	self.node:removeFromParent()
    self.node = nil
end

function creditsScreen:buttonActionPerformed(event)

	if event.sprite == self.backButton then
		dbg.print("Back")
		system:sendEvent("onBackToMainMenu")
		return true
	end
end
