--

hud = {}
hud_mt = { __index = hud }

function hud.init()
	local instance = {}
	setmetatable( instance, hud_mt)
	return instance
end

function hud:create(posX, posY, parentScene)
	self.scene = parentScene
	self.node = director:createNode()
	self.node.y = posY
	self.node.x = posX
	
	self.playButton = director:createSprite(40, 80 ,"Assets/sprNext_0.png")
	self.playButton.xAnchor = 0.5
	self.playButton.yAnchor = 0.5
	self.playButton.xScale = 0.5
	self.playButton.yScale = 0.5
	self.playButton:addEventListener("touch", self)
	self.node:addChild(self.playButton)
	
	self.gridButton = director:createSprite(40, 0 ,"Assets/sprNext_0.png")
	self.gridButton.xAnchor = 0.5
	self.gridButton.yAnchor = 0.5
	self.gridButton.xScale = 0.5
	self.gridButton.yScale = 0.5
	self.gridButton:addEventListener("touch", self)
	self.node:addChild(self.gridButton)
	
	self.menuButton = director:createSprite(120, 80 ,"Assets/sprMainMenu_0.png")
	self.menuButton.xAnchor = 0.5
	self.menuButton.yAnchor = 0.5
	self.menuButton.xScale = 0.5
	self.menuButton.yScale = 0.5
	self.menuButton:addEventListener("touch", self)
	self.node:addChild(self.menuButton)
	
	self.saveButton = director:createSprite(120, 0 ,"Assets/sprMainMenu_0.png")
	self.saveButton.xAnchor = 0.5
	self.saveButton.yAnchor = 0.5
	self.saveButton.xScale = 0.5
	self.saveButton.yScale = 0.5
	self.saveButton:addEventListener("touch", self)
	self.node:addChild(self.saveButton)
	
	self.numMovesLabel = director:createLabel(4, -80, "Moves : 0")
	self.node:addChild(self.numMovesLabel)
	
	self.levelCompleteLabel = director:createLabel(director.displayWidth / 2, director.displayHeight / 2 - posY, "LEVEL COMPLETE!!!")
	self.levelCompleteLabel.isVisible = false
	self.node:addChild(self.levelCompleteLabel)
	
	self.scene:addChild(self.node)
	
	self.isEditing = true;
	self.gridSizeIndex = 1
	self.gridSizes = {4,6,8}
	self.buttons = {}
	
	local blockDefault = director:createSprite(160, 20 ,"Assets/sprBlock_0.png")
	self.buttons[BLOCK_TYPE_DEFAULT] = blockDefault
	
	local blockWall = director:createSprite(240, 20 ,"Assets/sprWall_0.png")
	self.buttons[BLOCK_TYPE_WALL] = blockWall
	
	local blockArrow = director:createSprite(320, 20 ,"Assets/sprArrowBlock_0.png")
	self.buttons[BLOCK_TYPE_ARROW] = blockArrow
	
	local blockNum = director:createSprite(400, 20 ,"Assets/sprNumber1_0.png")
	self.buttons[BLOCK_TYPE_NUM] = blockNum
	
	local blockGhost = director:createSprite(480, 20 ,"Assets/ghost.png")
	self.buttons[BLOCK_TYPE_GHOST] = blockGhost
	
	local blockFlip = director:createSprite(560, 20 ,"Assets/flip.png")
	self.buttons[BLOCK_TYPE_FLIP] = blockFlip
	
	for i = 1, table.getn(self.buttons) do
		self.node:addChild(self.buttons[i])
		self.buttons[i]:addEventListener("touch", self)
	end

	self.selectedBlockType = BLOCK_TYPE_DEFAULT
	system:addEventListener( { "onCursorPositionChanged", "onLevelComplete"}, self)
	
	self:reset()
end

function hud:destroy()

	self.playButton = nil
	self.gridButton = nil
	self.menuButton = nil
	self.saveButton = nil
	self.numMovesLabel = nil
	self.levelCompleteLabel = nil

    self.node:removeFromParent()
	self.node = nil

	self.buttons = {}
	
	system:removeEventListener("onCursorPositionChanged", self)
	system:removeEventListener("onLevelComplete", self)
end

function hud:reset()
	self.levelCompleteLabel.isVisible = false
	self.numMovesLabel.text = "Moves : 0"
	self.isLevelComplete = false
end

function hud:onCursorPositionChanged(event)
	self.numMovesLabel.text = "Moves : " .. event.numMoves
end

function hud:onLevelComplete(event)
	self.levelCompleteLabel.isVisible = true
	self.isLevelComplete = true
end

function hud:touch(event)
    local node = event.target
	if self.isLevelComplete == true then
		return
	end
	
	if event.phase == "began" then
		if node == self.playButton then
			if self.isEditing then
				self.isEditing = false
			else
				self:reset()
				self.isEditing = true
			end
			system:sendEvent("onEditModeChanged", {isEditing = self.isEditing})
		elseif node == self.menuButton then
			system:sendEvent("onBackToMainMenu", {})
		elseif self.isEditing then
			for i = 1, table.getn(self.buttons) + 1 do
				if node == self.buttons[i] then
					self.selectedBlockType = i
					break
				end
			end
			if node == self.gridButton then
				self.gridSizeIndex = self.gridSizeIndex + 1
				if self.gridSizeIndex > table.getn(self.gridSizes) then
					self.gridSizeIndex = 1
				end
				system:sendEvent("onGridSizeChanged", {gridSize = self.gridSizes[self.gridSizeIndex]})
			elseif node == self.saveButton then
				system:sendEvent("onSaveLevel")
			end
		end
	end
end



