--

gHud = {}
gHud_mt = { __index = gHud }

function gHud.init()
	local instance = {}
	setmetatable( instance, gHud_mt)
	instance.time = 0
	return instance
end

function gHud:backKey(event)
	if(self.locked == false) then
		system:sendEvent("backToLevelSelection")
	end
end

function gHud:createButton(x, y, textureName)
	local atlas =  director:createAtlas( {
	width = 128,
	height = 128,
	numFrames = 2,
	textureName = textureName
    } )
	
	local anim = director:createAnimation( {
	start = 1,
	count = 2,
	atlas = atlas
    } )
	
	local newButton = director:createUIButton({x=x,y=y , xScale=UI_SCALE, yScale=UI_SCALE, source=anim})
	if(self.buttonsTable == nil) then
		self.buttonsTable = {}
	end
	self.buttonsTable[newButton.name] = newButton
	--dbg.print("** ** ** ** HUD button created")
	--dbg.printTable(self.buttonsTable)
	return newButton
end


function gHud:updateHintsCount(hintsCount)

	if hintsCount == "0" then
		self.hintLabel.text = getString(S_HUD_HINTS_BUY)
	else
		self.hintLabel.text = hintsCount
	end
end

function gHud:create(posX, posY, parentScene, showHint, worldId, levelId, hintsCount)

	self.scene = parentScene
	self.node = director:createNode()
	self.node.zOrder = 45

	self.restartButton = self:createButton(0, -32 * UI_SCALE, "Assets/HUD/ButtonRestartLevel.png")
	self.restartButton:addEventListener("activated", self)
	self.node:addChild(self.restartButton)
	
	self.menuButton = self:createButton(0, director.displayHeight - (144 * UI_SCALE), "Assets/HUD/ButtonBack.png")
	self.menuButton.zOrder = 1
	self.menuButton:addEventListener("activated", self)
	
	self.node:addChild(self.menuButton)

	self.hintNode = director:createNode()
	self.hintNode.x = director.displayWidth - (144 * UI_SCALE)
	self.hintNode.y = -32 * UI_SCALE
	self.hintButton = self:createButton(0,0, "Assets/HUD/ButtonHints.png")

	self.hintPanel = director:createSprite({xAnchor = 0.5, yAnchor = -0.8, xScale = UI_SCALE, yScale = UI_SCALE, source="Assets/HUD/NumberofHintsPanel.png"})
	self.hintLabel = director:createLabel({
        x=self.hintPanel.w/2, y=self.hintPanel.h/2,
        xAnchor=0.5, yAnchor=0.5,
        textXScale=0.45, textYScale=0.45,
        rotation=350,
        text="", font=GAME_FONT_TITLE,
        hAlignment="center", vAlignment="middle",
        textBorderLeft = 10, textBorderRight = 10, textBorderTop = 16, textBorderBottom = 8
    })
	self.hintPanel:addChild(self.hintLabel)
	self.hintNode:addChild(self.hintPanel)
	self.hintNode:addChild(self.hintButton)
	self.node:addChild(self.hintNode)

	self.skipButton = self:createButton(director.displayWidth, director.displayHeight - (144 * UI_SCALE), "Assets/HUD/ButtonBack.png")
	self.skipButton:addEventListener("activated", self)
	self.skipButton.xFlip = true
	self.skipButton.isVisible = false
	self.skipButton.xAnchor = 1
	self.node:addChild(self.skipButton)

	self.skipPanel = director:createSprite({x=self.skipButton.w/2, y=self.skipButton.h/2 - 24, xAnchor = 0.5, yAnchor = 0.5, source="Assets/HUD/NumberofHintsPanel.png"})
	local skipLabel = director:createLabel({
        x=self.skipPanel.w/2, y=self.skipPanel.h/2,
        xAnchor=0.5, yAnchor=0.5,
        textXScale=0.5, textYScale=0.5,
        rotation=350,
        text=getString(S_HUD_SKIP), font=GAME_FONT_TITLE,
        hAlignment="centre", vAlignment="middle",
        textBorderLeft = 10, textBorderRight = 10, textBorderTop = 16, textBorderBottom = 8
    })
	self.skipButton:addChild(self.skipPanel)
	self.skipPanel:addChild( skipLabel)
	self.skipPanel.isVisible = false

	self.title = director:createNode()
	self.title.x = director.displayWidth/2
	self.title.y = 48 * UI_SCALE
	self.titlePanel = director:createSprite({xAnchor = 0.5, yAnchor = 0.5, xScale = UI_SCALE, yScale = UI_SCALE, source="Assets/HUD/worldPanel.png"})
	self.titleLabel = director:createLabel( {
        textXScale = 0.4, textYScale = 0.4,
        text=getString(S_HUD_CHAPTER) .. " " .. worldId .. "\n" .. getString(S_HUD_LEVEL) .. " " .. levelId,
        font=GAME_FONT_TITLE,
        hAlignment="centre", vAlignment="middle",
        textBorderLeft = 60, textBorderRight = 60, textBorderTop = 40, textBorderBottom = 20
    } )
	self.titlePanel:addChild(self.titleLabel)
	self.title:addChild(self.titlePanel)
	self.node:addChild(self.title)

	self.scene:addChild(self.node)
	self:showHintButton(showHint)
	
	self.showHint = showHint
	
	system:addEventListener( { "onLevelComplete",
        "update","onNumStarsChanged", "backKey" }, self)

	self.locked = true
	self:updateHintsCount(hintsCount)
end

function gHud:show(visible)
	self.node.isVisible = visible
end

function gHud:onNumStarsChanged(event)
	if event.numStars > 0 then
		self.skipButton.isVisible = true
		system:sendEvent("skipButtonVisible", {})
--[[
		self.skipButtonAppear.isVisible = true
		self.skipButtonAppear:play({startFrame=1, loopCount=1})
]]
		self.skipPanel.xScale = 0
		self.skipPanel.yScale = 0
		self.skipPanel.rotation = 225
		self.skipPanel.isVisible = true

		tween:to(self.skipPanel, { xScale = 1, yScale = 1, rotation = 360, time = 0.5 })
	else
		self.skipPanel.isVisible = false
		self.skipButton.isVisible = false
		system:sendEvent("skipButtonInVisible", {})
--		self.skipButtonAppear.isVisible = false
	end
end

function gHud:update(event)
	--[[if self.locked then
		self.lastTime = system.gameTime
		return
	end
	self.time = self.time +  system.gameTime - self.lastTime
	self.lastTime = system.gameTime
	local s = math.floor(self.time)
	self.clockLabel.text = string.format("%02i",math.floor(s/60)) .. ":" .. string.format("%02i",s%60)
	]]
end

function gHud:updateTitle(worldId, levelId)
	self.titleLabel.text = getString(S_HUD_CHAPTER) .. " " .. worldId .. "\n" .. getString(S_HUD_LEVEL) .. " " .. levelId
end

function gHud:showHintButton(visible)
	self.hintNode.isVisible = visible

	if not visible then
		self.hintButton:removeEventListener("activated", self)
		dbg.print("gameHUD - removed listener for hint button")
	else
		self.hintButton:addEventListener("activated", self)
		dbg.print("gameHUD - added listener for hint button")
	end
end

function gHud:destroy()

    self.restartButton = nil
    self.menuButton = nil
    self.hintNode = nil
    self.hintButton = nil
    self.hintLabel = nil
    self.hintPanel = nil
    self.skipButton = nil
    self.skipPanel = nil
    self.title = nil
	self.buttonsTable = nil

    self.node:removeFromParent()
	self.node = nil	
	
	system:removeEventListener( { "onLevelComplete", "update", "backKey" }, self)
end

function gHud:onLevelComplete(event)
	self.isLevelComplete = true
end

function gHud:reset()
	self.skipButton.isVisible = false
end

function gHud:activated(event)
    local node = event.target
	
	-- If the HUD is locked or the button you pressed isn't currently visible, abort
	if self.locked or not node.isVisible then
		return false
	end
	
	if node == self.restartButton then
		audio:playSound("Assets/Audio/SFX_Confirm.raw")
		system:sendEvent("onRestart")
	elseif node == self.menuButton then
		system:sendEvent("backToLevelSelection")
		self:lock()
	elseif node == self.hintButton then
		dbg.print("gameHUD - gHud:activated - detected hint button being pressed")
		tutorialsManager:hintButtonPressed()
		audio:playSound("Assets/Audio/SFX_Confirm.raw")
		system:sendEvent("showHint")
	elseif node == self.skipButton then
		audio:playSound("Assets/Audio/SFX_Confirm.raw")
		system:sendEvent("skipCurrentLevel")
		self:lock()
	end
	return true
end

function gHud:lock()
	self.locked = true
	
	for k, v in pairs(self.buttonsTable) do
		--dbg.print("disabling button " ..v.name)
		v:disable()
	end
end

function gHud:unLock(numStars)
	self.locked = false
	--dbg.print("=======================gHud:unLock()==============================")
	for k, v in pairs(self.buttonsTable) do
		--dbg.print("enabling button " ..v.name)
		v:enable()
	end
end
