--

introCutscene = {}
introCutscene.FADE_IN_TIME_LENGTH	= 2.0
introCutscene.GLOW_FADE_OUT_TIME_LENGTH	= 0.25
introCutscene_mt = { __index = introCutscene }

function introCutscene.create()
	local instance = {}
	setmetatable( instance, introCutscene_mt)
	instance.scene = director:createScene()
	instance.scene.name = "Scene: Intro Cutscene"
	instance.scene:addEventListener( { "setUp" , "tearDown"}, instance)
	return instance
end

function introCutscene:setUp(event)

	self.node = director:createNode()
	self.scene:addChild(self.node)

	self.fadeScreen = director:createRectangle( {
        x=director.displayCenterX,
        y=director.displayCenterY,
        w=director.displayWidth,
        h=director.displayHeight,
	    xAnchor = 0.5,
        yAnchor = 0.5,
        zOrder = 10000,
        IsVisible = true,
        color={0, 0, 0},
        strokeColor={0, 0, 0},
	    strokeAlpha = 1,	
	    alpha = 0
    } )
	self.node:addChild(self.fadeScreen)

	self.bg = director:createSprite( {
        source = "Assets/UI/Intro_Bg.png",
        x = director.displayCenterX,
        y = director.displayCenterY,
        xAnchor = 0.5,
        yAnchor = 0.5,
        xScale = UI_SCALE,
        yScale = UI_SCALE,
    })
	self.node:addChild(self.bg)
	
	self.pic1 = director:createSprite( {
        source = "Assets/UI/Intro_pic_1.png",
        x = 300,
        y = self.bg.h - 306,
		xScale = 1.0,
		yScale = 1.0,
        xAnchor = 0.5,
        yAnchor = 0.5,
		alpha = 0,
    })
	--introCutscene:addGlowingSprite(self.pic1, "Assets/UI/Intro_pic_1.png")

	self.bg:addChild(self.pic1)
	
	self.pic2 = director:createSprite( {
        source = "Assets/UI/Intro_pic_2.png",
        x = 551,
        y = self.bg.h - 523,
		xScale = 1.0,
		yScale = 1.0,
        xAnchor = 0.5,
        yAnchor = 0.5,
		alpha = 0,
    })
	--introCutscene:addGlowingSprite(self.pic2, "Assets/UI/Intro_pic_2.png")
	self.bg:addChild(self.pic2)
	
	self.pic3 = director:createSprite( {
        source = "Assets/UI/Intro_pic_3.png",
        x = 276,
        y = self.bg.h - 646,
		xScale = 1.0,
		yScale = 1.0,
        xAnchor = 0.5,
        yAnchor = 0.5,
		alpha = 0,
    })
	--introCutscene:addGlowingSprite(self.pic3, "Assets/UI/Intro_pic_3.png")
	self.bg:addChild(self.pic3)

	self.pic4 = director:createSprite( {
        source = "Assets/UI/Intro_pic_4.png",
        x = 546,
        y = self.bg.h - 813,
		xScale = 1.0,
		yScale = 1.0,
        xAnchor = 0.5,
        yAnchor = 0.5,
		alpha = 0,
    })
	--introCutscene:addGlowingSprite(self.pic4, "Assets/UI/Intro_pic_4.png")
	self.bg:addChild(self.pic4)

	self.pic5 = director:createSprite( {
        source = "Assets/UI/Intro_pic_5.png",
        x = 214,
        y = self.bg.h - 909,
		xScale = 1.0,
		yScale = 1.0,
        xAnchor = 0.5,
        yAnchor = 0.5,
		alpha = 0,
    })
	--introCutscene:addGlowingSprite(self.pic5, "Assets/UI/Intro_pic_5.png")
	self.bg:addChild(self.pic5)
	
	-- Work-around for tweening onComplete function being run on target object and unable to reference back to main scene
	self.pic1.backRef = self
	self.pic2.backRef = self
	self.pic3.backRef = self
	self.pic4.backRef = self
	self.pic5.backRef = self
	self.fadeScreen.backRef = self
	
	-- Count progress through cutscene
	self.stepCount = 1
	self.stepTrigger = { self.pic1, self.pic2, self.pic3, self.pic4, self.pic5, self.fadeScreen }
	
	-- Add a listener for the player tapping the screen to rush through the pics or pressing the next button
	system:addEventListener("touch", self)
	system:addEventListener("buttonActionPerformed", self)

	self.next = director:createSprite( {
        source = "Assets/UI/Popup_icon_next_button .png",
        x = director.displayWidth - 64 * UI_SCALE,
        y = 64 * UI_SCALE,
        xAnchor = 0.5,
        yAnchor = 0.5,
        xScale = UI_SCALE,
        yScale = UI_SCALE,
    })

	self.nextLabel = director:createLabel( {
        w = self.next.w * 2,
		text = getString(S_MENU_PLAY),
        xAnchor = 1.0,
		hAlignment = "right", vAlignment = "middle"
    })
	
	self.node:addChild(self.next)
	self.next:addChild(self.nextLabel)
	
	self.next.isVisible = false
	SpriteButton.addAnimatedButtonToSprite(self.next, "Assets/Audio/SFX_Confirm.raw")

	-- Start first tween
	--tween:to(self.pic1, { alpha = 1, time = introCutscene.FADE_IN_TIME_LENGTH, onComplete = introCutscene.onStepComplete })	
	self:fadePicIn(self.pic1, introCutscene.FADE_IN_TIME_LENGTH)
	dialogManager:setUp()
end

function introCutscene:addGlowingSprite(sprite, spriteSource)
	local glowingSprite = director:createSprite( {
        source = spriteSource,
        x = 0,
        y = 0,
		xScale = 1.0,
		yScale = 1.0,
        xAnchor = 0.0,
        yAnchor = 0.0,
		alpha = 0,
		zOrder = 1,
		blendMode = "add"
    })
	sprite:addChild(glowingSprite)
end

function introCutscene:fadePicIn(picSprite, timeLength)
	for i = 1,#picSprite.children do
		tween:to(picSprite.children[i], { alpha = 1, time = timeLength})
	end
	tween:to(picSprite, { alpha = 1, time = timeLength, onComplete = introCutscene.onStepComplete })
end

function introCutscene.onStepComplete(target)

	for i = 1,#target.children do
		tween:to(target.children[i], { alpha = 0, time = introCutscene.GLOW_FADE_OUT_TIME_LENGTH})
	end

	if target.backRef.stepCount < 5 and target == target.backRef.stepTrigger[target.backRef.stepCount] then
	
		target.backRef.stepCount = target.backRef.stepCount + 1
		--tween:to(target.backRef.stepTrigger[target.backRef.stepCount], { alpha = 1, time = introCutscene.FADE_IN_TIME_LENGTH, onComplete = introCutscene.onStepComplete })
		introCutscene:fadePicIn(target.backRef.stepTrigger[target.backRef.stepCount], introCutscene.FADE_IN_TIME_LENGTH)
	elseif target.backRef.stepCount == 5 and target == target.backRef.stepTrigger[5] then
	
		target.backRef.stepCount = target.backRef.stepCount + 1
		target.backRef.next.isVisible = true

	elseif target.backRef.stepCount == 6 and target == target.backRef.stepTrigger[6] then

		target.backRef.stepCount = target.backRef.stepCount + 1
		system:sendEvent("onStartFirstGame")
	end
end

function introCutscene:touch(event)

    local target = event.target

	if event.phase == "began" then
	
		if target == self.next and self.next.isVisible then
		
			-- Do nothing, this is handled in a separate event
			
		elseif self.stepCount < 6 then
			introCutscene.onStepComplete(self.stepTrigger[self.stepCount])
		end
	end
end

function introCutscene:buttonActionPerformed(event)

	if (event.sprite == self.next and self.next.isVisible) then
		tween:to(self.fadeScreen, { alpha = 1, delay = 0.5, time = 0.5, onComplete = introCutscene.onStepComplete })
		self.next.isVisible = false
	end
end

function introCutscene:tearDown(event)
	dialogManager:tearDown()
	self.bg:removeFromParent()
	self.bg = nil
	self.pic1 = nil
	self.pic2 = nil
	self.pic3 = nil
	self.pic4 = nil
	self.pic5 = nil
	self.next = nil

	self.fadeScreen:removeFromParent()
    self.fadeScreen = nil

    self.node:removeFromParent()
    self.node = nil
	
	system:removeEventListener("touch", self)
	system:removeEventListener("buttonActionPerformed", self)
	SpriteButton.tearDown()
end
