--

levelSelect = {}
levelSelect_mt = { __index = levelSelect }

levelCoordinates = 
{
{x=-152,y=-268},
{x=0,y=-268},
{x=155,y=-268},
{x=-201,y=-121},
{x=1,y=-134},
{x=204,y=-121},
{x=-132,y=-13},
{x=1,y=-1},
{x=135,y=-11},
{x=-190,y=100},
{x=2,y=161},
{x=192,y=100},
{x=-160,y=254},
{x=1,y=302},
{x=163,y=252},
}

function levelSelect.create()
	local instance = {}
	setmetatable( instance, levelSelect_mt)
	instance.scene = director:createScene()
	instance.scene.name = "Scene: LevelSelect"
	instance.dw = director.displayWidth
	instance.dh = director.displayHeight
	instance.scene:addEventListener({ "setUp" , "tearDown"}, instance)
	return instance
end

function levelSelect:backKey(event)
	dbg.print("levelSelect:backKey")
	dbg.printTable(event)
	system:sendEvent("onBackToWorldSelect")	
end

function levelSelect:LoadAssets()

    local atlas
	atlas = director:createAtlas( "Assets/UI/level_select_screen.jpg" )
	self.bgAnim = director:createAnimation( atlas )

	atlas = director:createAtlas( "Assets/UI/back_arrow.png" )
	self.backButtonAnim = director:createAnimation( atlas )

    atlas = director:createAtlas("Assets/UI/button.png")
    self.buttonAnim = director:createAnimation( atlas )

    atlas = director:createAtlas("Assets/UI/padlock_small.png")
    self.padLockAnim = director:createAnimation( atlas )

    atlas = director:createAtlas("Assets/UI/star.png")
    self.starAnim = director:createAnimation( atlas )

	atlas =  director:createAtlas( { width = 54, height = 54, numFrames = 2, textureName = "Assets/UI/dot.png" } )
	self.dotAnim = director:createAnimation( { start = 1, count = 2, atlas = atlas } )

end

function levelSelect:UnloadAssets()
    
    self.bgAnim = nil
    self.backButtonAnim = nil
    self.buttonAnim = nil
    self.padLockAnim = nil
    self.starAnim = nil
    self.dotAnim = nil

end

function levelSelect:setUp(event)
    dbg.print("levelSelect:setUp")
	local buttonWidth = 184
	dbg.print("levelSelect:setUp debug 1")
    -- Make sure the assets are loaded if we are returning from the game
    LoadFrontEndAssets()
	dbg.print("levelSelect:setUp debug 2")
    -- Create a scene node for easy cleanup
	self.node = director:createNode()

	self.scene:addChild(self.node)

    -- Create the background
	self.bg = director:createSprite({ 
        source = self.bgAnim,
        x = director.displayCenterX,
        y = director.displayCenterY,
        xAnchor = 0.5,
        yAnchor = 0.5,
        xScale = UI_SCALE,
        yScale = UI_SCALE
    })
	self.bg:addEventListener("touch", self)
	self.node:addChild(self.bg)

    -- Create the page node
	self.pageNode = director:createNode()
	self.node:addChild(self.pageNode)

    -- Create the back button
	self.backButton = director:createSprite({
        source = self.backButtonAnim,
        x = 16 * UI_SCALE, y = director.displayHeight - (16 * UI_SCALE),
        xAnchor = 0, yAnchor = 1,
        xScale = UI_SCALE, yScale = UI_SCALE,
        zOrder = 10
    })
	SpriteButton.addAnimatedButtonToSprite(self.backButton, "Assets/Audio/SFX_Back.raw")
	self.node:addChild(self.backButton)

    -- Create the title label
	local title = director:createLabel({
        x = 0, y = self.backButton.y,
        w = director.displayWidth, h = self.backButton.h * UI_SCALE,
        textXScale = UI_SCALE * 0.7, textYScale = UI_SCALE * 0.7,
        xAnchor = 0, yAnchor = 1,
        text = getString(S_MENU_LEVEL_SELECT),
        font = GAME_FONT_TITLE,
        color = GAME_FONT_COLOR,
        hAlignment="centre", vAlignment="middle",
    })
	self.node:addChild(title)

	local numHorizontalSprites = 3--math.floor(DEFAULT_UI_WIDTH / (buttonWidth + buttonWidth * 0.2))
	local numVerticalSprites = 5--math.floor((DEFAULT_UI_HEIGHT - buttonWidth) / buttonWidth)
	local offset = ((director.displayWidth - (numHorizontalSprites * buttonWidth)) / 2) * UI_SCALE
	self.numPages = math.ceil(table.getn(self.worldInfo.levels) / numHorizontalSprites /numVerticalSprites)
	self.currentPage = 1

	for i = 1, #self.worldInfo.levels do
		local y = director.displayHeight / 2 - levelCoordinates[(i-1)%15 + 1].y * UI_SCALE
		local x = director.displayWidth / 2 + levelCoordinates[(i-1)%15 + 1].x * UI_SCALE + (math.floor(((i-1)/15)) * director.displayWidth)

		local sprite = director:createSprite({
            x = x,
            y = y,
            xAnchor = 0.5,
            yAnchor = 0.5,
            xScale = UI_SCALE,
            yScale = UI_SCALE,
            source = self.buttonAnim,
			alpha = 1.0
        })
		self.pageNode:addChild(sprite)

		sprite.levelId = i
		if self.worldInfo.levels[i].isLocked then
			local padlock = director:createSprite({
                source = self.padLockAnim,
                x = sprite.w / 2,
                y = sprite.h / 2,    
                xAnchor = 0.5,
                yAnchor = 0.5
            })
			sprite:addChild(padlock)
			
			SpriteButton.addAnimatedButtonToSprite(padlock, "Assets/Audio/SFX_BlockedTile.raw")
		else
		    local label = director:createLabel({
                color = GAME_FONT_COLOR,
                hAlignment="centre", vAlignment="middle",
		        textXScale = 0.8, textYScale = 0.8,
                text = i,
                font = GAME_FONT
            })
    		sprite:addChild(label)

			for y = 1, self.worldInfo.levels[i].numStars do
				local star = director:createSprite({
                    source = self.starAnim,
                    x = 43 + (y - 1) * 42,
                    y = 50 - (y % 2) * 5,
                    xAnchor = 0.5,
                    yAnchor = 0.5,
                    isTouchable = false
                })
                sprite:addChild(star)
			end

			SpriteButton.addAnimatedButtonToSprite(sprite, "Assets/Audio/SFX_Confirm.raw")
		end

	end
	
	self.dots = {}
	for i = 1, self.numPages do
		local dot = director:createUIToggle({
            source = self.dotAnim,
            enableInputs = false,
            state = i > 1 and "off" or "on",
            x = director.displayCenterX - ((self.numPages/2) * 54 * UI_SCALE) / 2 + (i - 1) * 54 * UI_SCALE,
            xAnchor = 0.5,
            xScale = UI_SCALE,
            yScale = UI_SCALE
        })
		self.dots[i] = dot
		self.node:addChild(dot)
	end
	if(not audio:isStreamPlaying()) then
		if(audio.musicVolume > 0) then
			audio:playStream("Assets/Audio/MUS_FrontEnd.mp3", true)
		end
	end
	
	-- If we've returned to the level select screen from the game, and the last level we played wasn't on the first page...
	if self.worldInfo.lastLevelPlayed ~= nil and self.worldInfo.lastLevelPlayed > 15 then
	
	    self.dots[self.currentPage]:toggle("off")
		self.currentPage = math.ceil(self.worldInfo.lastLevelPlayed / 15)
	    self.dots[self.currentPage]:toggle("on")
		
		self.pageNode.x = -(self.currentPage - 1) * director.displayWidth
	end
	system:addEventListener({"buttonActionPerformed", "buttonActionStarted", "backKey"}, self)
	dialogManager:setUp()
end

function levelSelect:tearDown(event)
    dbg.print("levelSelect:tearDown")
	dialogManager:tearDown()
	system:removeEventListener({"buttonActionPerformed", "buttonActionStarted", "backKey" }, self)
	dbg.print("levelSelect:tearDown removed listener")
	SpriteButton.tearDown()
	dbg.print("levelSelect:tearDown SpriteButtons destroyed")
    self.bg = nil
	dbg.print("levelSelect:tearDown self.bg going to be GC")
    self.backButton = nil
	dbg.print("levelSelect:tearDown self.backButton going to be GC")
    self.dots = nil
	dbg.print("levelSelect:tearDown self.dots going to be GC")
	self.pageNode:removeFromParent()
    dbg.print("levelSelect:tearDown self.pageNode:removeFromParent()")
	self.pageNode = nil
	dbg.print("levelSelect:tearDown self.pageNode going to be GC")
	self.node:removeFromParent()
    dbg.print("levelSelect:tearDown self.node:removeFromParent()")
	self.node = nil
	dbg.print("levelSelect:tearDown self.node going to be GC")
	self.lastLevel = nil
	dbg.print("levelSelect:tearDown self.lastLevel going to be GC")

end

--[[
function levelSelect:touchEvent(event)
	if event.phase == "began" then
		self.startX = event.x
		self.startY = event.y
		self.startTime =  system.gameTime
		self.elapsedTime = 0
        --return true

	elseif event.phase == "ended" then
        -- Check for swipes
		local moveX = event.x - self.startX
		local moveY = event.y - self.startY
		local dist_sq = (moveX*moveX) + (moveY*moveY)
		if dist_sq >= (50 * 50) * UI_SCALE then
			local elapsedTime =  system.gameTime - self.startTime
			if elapsedTime > 0 then
                local dist = math.sqrt(dist_sq)
			    local velocity = dist / elapsedTime
				-- random value 1, can be changed to apply a more efficient velocity
				if velocity > 1 then
					moveX = moveX / dist
					if moveX > 0.7 then
						self:goToPrevPage()
					elseif moveX < -0.7 then
						self:goToNextPage()
					end
					--return true
				end
			end
		end
	else
		
	end
end
--]]

function levelSelect:buttonActionStarted(event)
	self.startX = event.x
	self.startY = event.y
end

function levelSelect:buttonActionPerformed(event)
	
	local target = event.sprite
	if target == self.backButton then
		dbg.print("Back")
		system:sendEvent("onBackToWorldSelect")
		return true
	elseif target ~= self.bg and not self.worldInfo.levels[target.levelId].isLocked then
		system:sendEvent("onStartGame", {levelId = target.levelId})
		return true
	else
	
	end
	
end

function levelSelect:touch(event)
	if event.phase == "began" then
		self.startX = event.x
		self.startY = event.y
		self.startTime =  system.gameTime
		self.elapsedTime = 0
        --return true

	elseif event.phase == "ended" then
        -- Check for swipes
		if(self.startX == nil)then
			self.startX = event.x
		end
		if(self.startY == nil)then
			self.startY = event.y
		end
		local moveX = event.x - self.startX
		local moveY = event.y - self.startY
		local dist_sq = (moveX*moveX) + (moveY*moveY)
		if dist_sq >= (50 * 50) * UI_SCALE then
			local elapsedTime =  system.gameTime - self.startTime
			if elapsedTime > 0 then
                local dist = math.sqrt(dist_sq)
			    local velocity = dist / elapsedTime
				-- random value 1, can be changed to apply a more efficient velocity
				if velocity > 1 then
					moveX = moveX / dist
					if moveX > 0.7 then
						self:goToPrevPage()
					elseif moveX < -0.7 then
						self:goToNextPage()
					end
					--return true
				end
			end
		end
	end

    return false
end

function levelSelect:goToNextPage()
	SpriteButton.cancelAllActions()
	if self.currentPage < self.numPages then
	    --SpriteButton.resetAll()
		
		audio:playSound("Assets/Audio/SFX_Swipe.raw")

	    self.dots[self.currentPage]:toggle("off")
	    self.currentPage = self.currentPage + 1
	    self.dots[self.currentPage]:toggle("on")

	    tween:to(self.pageNode,{x=-(self.currentPage - 1) * director.displayWidth, time = 0.5, easing=ease.elasticInOut})
    end
end

function levelSelect:goToPrevPage()
	SpriteButton.cancelAllActions()
    if self.currentPage > 1 then
		--SpriteButton.resetAll()
    	
		audio:playSound("Assets/Audio/SFX_Swipe.raw")

	    self.dots[self.currentPage]:toggle("off")
	    self.currentPage = self.currentPage - 1
	    self.dots[self.currentPage]:toggle("on")

	    tween:to(self.pageNode,{x=-(self.currentPage - 1) * director.displayWidth, time = 0.5, easing=ease.elasticInOut})
    end
end
