
local BUTTON_SETTINGS = 1
local BUTTON_MUSIC = 2
local BUTTON_SOUND = 3
local BUTTON_CREDITS = 4
local BUTTON_COUNT = 4

G_QUIT_APPLICATION_DIALOG_ID = -9977

mainMenu = {}
mainMenu_mt = { __index = mainMenu }

function mainMenu.create()
	local instance = {}
	setmetatable( instance, mainMenu_mt)
	instance.scene = director:createScene()
	instance.scene.name = "Scene: Menu"
	instance.dw = director.displayWidth
	instance.dh = director.displayHeight
	instance.scene:addEventListener({ "setUp", "tearDown", "dialog" }, instance)
	return instance
end

function mainMenu:LoadAssets()

    local atlas

	atlas = director:createAtlas( "Assets/UI/bkg.jpg" )
	self.bgAnim = director:createAnimation( atlas )

	atlas = director:createAtlas( "Assets/UI/Cloud1.png" )
	self.cloud1Anim = director:createAnimation( atlas )

	atlas = director:createAtlas( "Assets/UI/Cloud2.png" )
	self.cloud2Anim = director:createAnimation( atlas )

	atlas = director:createAtlas( "Assets/UI/Cloud3.png" )
	self.cloud3Anim = director:createAnimation( atlas )

	atlas = director:createAtlas( "Assets/UI/signal.png" )
	self.signalAnim = director:createAnimation( atlas )

	atlas = director:createAtlas( { width=378, height=378, numFrames=7, textureName="Assets/UI/signalBlinkingAtlas.png" } )
	self.signalBlinkAnim = director:createAnimation( { start=1, count=7, atlas=atlas, delay=1/16 } )

	atlas = director:createAtlas( "Assets/UI/logo.png" )
	self.logoAnim = director:createAnimation( atlas )

	atlas = director:createAtlas( "Assets/UI/playButton.png" )
	self.playButtonAnim = director:createAnimation( atlas )

	atlas = director:createAtlas( "Assets/UI/playButtonGlow2.png" )
	self.playGlowAnim = director:createAnimation( atlas )

	atlas = director:createAtlas( "Assets/UI/facebook_button.png" )
	self.facebookAnim = director:createAnimation( atlas )

	atlas = director:createAtlas( "Assets/UI/music.png" )
	self.settingsAnim = {}
	self.settingsAnim[BUTTON_MUSIC] = director:createAnimation( atlas )

	atlas = director:createAtlas( "Assets/UI/sound.png" )
	self.settingsAnim[BUTTON_SOUND] = director:createAnimation( atlas )

	atlas = director:createAtlas( "Assets/UI/credits.png" )
	self.settingsAnim[BUTTON_CREDITS] = director:createAnimation( atlas )

	atlas = director:createAtlas( "Assets/UI/setting_pop_up.png" )
	self.settingsPopUpAnim = director:createAnimation( atlas )

	atlas = director:createAtlas( "Assets/UI/redCross.png" )
	self.redCrossAnim = director:createAnimation( atlas )

--	atlas = director:createAtlas( "Assets/sprRestart_0.png" )
--	self.editAnim = director:createAnimation( atlas )
	
end

function mainMenu:backKey(event)
	--only android
	dbg.print("mainMenu:backKey")
	dbg.printTable(event)
	dialogManager:displayOkCancelDialog(getString(S_IAP_DIALOG_TITLE_QUIT), getString(S_QUIT_DIALOG), getString(S_IAP_DIALOG_BUTTON_OK_LABEL), getString(S_IAP_DIALOG_BUTTON_CANCEL_LABEL), G_QUIT_APPLICATION_DIALOG_ID, nil, nil)
end

function mainMenu:UnloadAssets()
    self.bgAnim = nil
	self.cloud1Anim = nil
	self.cloud2Anim = nil
	self.cloud3Anim = nil
	self.signalAnim = nil
	self.signalBlinkAnim = nil
	self.logoAnim = nil
    self.playButtonAnim = nil
	self.playGlowAnim = nil
	self.facebookAnim = nil
	self.settingsAnim = nil
    self.settingsPopUpAnim = nil
    self.redCrossAnim = nil
--	self.editAnim = nil
end

function mainMenu:setUp(event)
--    dbg.print("mainMenu:setUp")
--	dbg.print("UI_SCALE " ..UI_SCALE)
--	dbg.print("director.displayCenterX " .. director.displayCenterX)
--	dbg.print("director.displayCenterY " .. director.displayCenterY)

    LoadFrontEndAssets()

    self.node = director:createNode()
	self.scene:addChild(self.node)

    -- Create the background sprite
	self.bg = director:createSprite( {
        source = self.bgAnim,
        x = director.displayCenterX,
        y = director.displayCenterY,
        xAnchor = 0.5,
        yAnchor = 0.5,
        xScale = UI_SCALE,
        yScale = UI_SCALE,
        isTouchable = false
    })
	self.node:addChild(self.bg)

	self.cloudFrameNumber = -1
	self.cloudKeyFrames = {
		{
			animBegan = { },
			currentKeyFrame = { },

			{
			frame = 0,
			posX = director.displayCenterX + 287 * UI_SCALE,
			posY = director.displayCenterY +  63 * UI_SCALE,
			scale = UI_SCALE * 0.5,
			rot = 308,
			alpha = 0.11
			},

			{
			frame = 76 * 2,
			posX = director.displayCenterX + 283 * UI_SCALE,
			posY = director.displayCenterY + 139 * UI_SCALE,
			scale = UI_SCALE * 0.886,
			rot = 340.7,
			alpha = 0.5
			},

			{
			frame = 174 * 2,
			posX = director.displayCenterX + 279 * UI_SCALE,
			posY = director.displayCenterY + 238 * UI_SCALE,
			scale = UI_SCALE * 1.38,
			rot = 383,
			alpha = 0
			}
		},

		{
			animBegan = { },
			currentKeyFrame = { },

			{
			frame = 6 * 2,
			posX = director.displayCenterX + 283 * UI_SCALE,
			posY = director.displayCenterY +  54 * UI_SCALE,
			scale = UI_SCALE * 0.5,
			rot = 322,
			alpha = 0.1
			},

			{
			frame = 83 * 2,
			posX = director.displayCenterX + 285 * UI_SCALE,
			posY = director.displayCenterY + 135 * UI_SCALE,
			scale = UI_SCALE * 1.0,
			rot = 339,
			alpha = 0.38
			},

			{
			frame = 179 * 2,
			posX = director.displayCenterX + 287 * UI_SCALE,
			posY = director.displayCenterY + 253 * UI_SCALE,
			scale = UI_SCALE * 1.5,
			rot = 360,
			alpha = 0
			}
		},

		{
			animBegan = { },
			currentKeyFrame = { },

			{
			frame = 31 * 2,
			posX = director.displayCenterX + 284 * UI_SCALE,
			posY = director.displayCenterY +  33 * UI_SCALE,
			scale = UI_SCALE * 0.5,
			rot = 268,
			alpha = 0
			},

			{
			frame = 109 * 2,
			posX = director.displayCenterX + 288 * UI_SCALE,
			posY = director.displayCenterY + 120 * UI_SCALE,
			scale = UI_SCALE * 1.09,
			rot = 329,
			alpha = 0.56
			},

			{
			frame = 204 * 2,
			posX = director.displayCenterX + 293 * UI_SCALE,
			posY = director.displayCenterY + 244 * UI_SCALE,
			scale = UI_SCALE * 2.09,
			rot = 360,
			alpha = 0
			}
		}
	}


    -- Create the smoke clouds for the house
	self.cloud1a = director:createSprite( { source = self.cloud1Anim, x = 0, y = 0, xAnchor = 0.5, yAnchor = 0.5, alpha = 0, xScale = UI_SCALE*0.5, yScale = UI_SCALE*0.5 })
	self.cloud1b = director:createSprite( { source = self.cloud1Anim, x = 0, y = 0, xAnchor = 0.5, yAnchor = 0.5, alpha = 0, xScale = UI_SCALE*0.5, yScale = UI_SCALE*0.5 })
	self.cloud1c = director:createSprite( { source = self.cloud1Anim, x = 0, y = 0, xAnchor = 0.5, yAnchor = 0.5, alpha = 0, xScale = UI_SCALE*0.5, yScale = UI_SCALE*0.5 })
	self.node:addChild(self.cloud1a)
	self.node:addChild(self.cloud1b)
	self.node:addChild(self.cloud1c)

	self.cloud2a = director:createSprite( { source = self.cloud1Anim, x = 0, y = 0, xAnchor = 0.5, yAnchor = 0.5, alpha = 0, xScale = UI_SCALE*0.5, yScale = UI_SCALE*0.5 })
	self.cloud2b = director:createSprite( { source = self.cloud1Anim, x = 0, y = 0, xAnchor = 0.5, yAnchor = 0.5, alpha = 0, xScale = UI_SCALE*0.5, yScale = UI_SCALE*0.5 })
	self.cloud2c = director:createSprite( { source = self.cloud1Anim, x = 0, y = 0, xAnchor = 0.5, yAnchor = 0.5, alpha = 0, xScale = UI_SCALE*0.5, yScale = UI_SCALE*0.5 })
	self.node:addChild(self.cloud2a)
	self.node:addChild(self.cloud2b)
	self.node:addChild(self.cloud2c)

	self.cloud3a = director:createSprite( { source = self.cloud1Anim, x = 0, y = 0, xAnchor = 0.5, yAnchor = 0.5, alpha = 0, xScale = UI_SCALE*0.5, yScale = UI_SCALE*0.5 })
	self.cloud3b = director:createSprite( { source = self.cloud1Anim, x = 0, y = 0, xAnchor = 0.5, yAnchor = 0.5, alpha = 0, xScale = UI_SCALE*0.5, yScale = UI_SCALE*0.5 })
	self.cloud3c = director:createSprite( { source = self.cloud1Anim, x = 0, y = 0, xAnchor = 0.5, yAnchor = 0.5, alpha = 0, xScale = UI_SCALE*0.5, yScale = UI_SCALE*0.5 })
	self.node:addChild(self.cloud3a)
	self.node:addChild(self.cloud3b)
	self.node:addChild(self.cloud3c)


	-- Make a list of the clouds, so we can cycle through and animate them all
	self.clouds = { { self.cloud1a, self.cloud1b, self.cloud1c }, { self.cloud2a, self.cloud2b, self.cloud2c }, { self.cloud3a, self.cloud3b, self.cloud3c } }

    -- Create the game logo
	local logo = director:createSprite( {
        source = self.logoAnim,
        x = director.displayCenterX,
        y = director.displayCenterY + 292 * UI_SCALE,	-- 860
        xAnchor = 0.5,
        yAnchor = 0.5,
        xScale = UI_SCALE,
        yScale = UI_SCALE,
    })
	self.node:addChild(logo)

    -- Create Signal's body
	local signalBody = director:createSprite( {
        source = self.signalAnim,
        x = director.displayCenterX - 145 * UI_SCALE, -- 175,
        y = director.displayCenterY - 133 * UI_SCALE, -- 435,
        xAnchor = 0.5,
        yAnchor = 0.5,
        xScale = UI_SCALE,
        yScale = UI_SCALE,
    })
	self.node:addChild( signalBody)

    -- Create Signal's head
	self.signalHead = director:createSprite( {
        source = self.signalBlinkAnim,
        x = director.displayCenterX - 145 * UI_SCALE, -- 175,
        y = director.displayCenterY - 18 * UI_SCALE, -- 550,
        xAnchor = 0.5,
        yAnchor = 0.5,
        xScale = UI_SCALE,
        yScale = UI_SCALE,
    })
	self.node:addChild(self.signalHead)
	self.signalHead:play({startFrame=1, loopCount=1})
	self.signalHeadCount = math.random(60,120)

    -- Create the play button panel
	self.playButton = director:createSprite( {
        source = self.playButtonAnim,
        x = director.displayCenterX,
        y = director.displayCenterY - 358 * UI_SCALE, -- 210,
        xAnchor = 0.5,
        yAnchor = 0.5,
        xScale = UI_SCALE,
        yScale = UI_SCALE,
    })
	
	-- Create the play button glow
	-- TO DO: THIS NEEDS TO STROBE SLOWLY
	self.playGlow = director:createSprite( {
        source = self.playGlowAnim,
        x = self.playButton.w / 2 - 8,
        y = self.playButton.h / 2 - 8,
        xAnchor = 0.5,
        yAnchor = 0.5,
        xScale = 1,
        yScale = 1,
    })

	self.playGlowDelta = -1


    -- Create the play button text
	-- TO DO: THIS NEEDS TO PULSE SLOWLY
	self.playText = director:createLabel( {
        source = self.playAnim,
        x = self.playButton.w / 2,
        y = self.playButton.h / 2,
        xAnchor = 0.5,
        yAnchor = 0.25,
		color = GAME_FONT_COLOR,
		font = GAME_FONT,
		text = getString(S_MENU_PLAY),
		hAlignment = "centre",
		vAlignment = "bottom"
    })
	
	self.node:addChild(self.playButton)
	self.playButton:addChild(self.playGlow)
	self.playButton:addChild(self.playText)

	SpriteButton.addAnimatedButtonToSprite(self.playText, "Assets/Audio/SFX_Confirm.raw", DefaultButtonLabelAnimation.new(self.playText))
	

	self.fbButton = director:createSprite( {
        source = self.facebookAnim,
        x = director.displayWidth - 48 * UI_SCALE,
        y = 48 * UI_SCALE,
        xAnchor = 0.5,
        yAnchor = 0.5,
        xScale = UI_SCALE,
        yScale = UI_SCALE,
    })
	self.fbButton:addEventListener("touch", self)
	self.node:addChild(self.fbButton)
	
    -- Create the settings panel
	self.buttons = {}
   	self.settingsPanelOpened = false

    -- Create a container for the settings buttons
    local settingsPanel = director:createNode( {
        x = director.displayWidth - 64 * UI_SCALE,
        y = director.displayCenterY - 168 * UI_SCALE,
        xScale = UI_SCALE,
        yScale = UI_SCALE,
        zOrder = 2,
    } )
    self.node:addChild( settingsPanel)

    -- Create the expand button
    local button = director:createSprite( {
        source = self.settingsPopUpAnim,
        x = 0,
        y = 0,
        xAnchor = 0.5,
        yAnchor = 1,
    })
	--button:addEventListener("touch", self)
	settingsPanel:addChild(button)
	self.buttons[BUTTON_SETTINGS] = button
	SpriteButton.addAnimatedButtonToSprite(self.buttons[BUTTON_SETTINGS], "Assets/Audio/SFX_Confirm.raw")

	local height = button.h
	local settingsWidth = button.w
	local settingsHeight = button.h * BUTTON_COUNT

    -- Setup a node to clip and scroll the settings buttons
    local clipping_node = director:createNode()
    clipping_node.y = (1-BUTTON_COUNT) * height
	clipping_node.clipX = 0
	clipping_node.clipY = settingsPanel.y
	clipping_node.clipW = director.displayWidth
	clipping_node.clipH = director.displayHeight - settingsPanel.y
	
	settingsPanel:addChild(clipping_node)
	self.clippingNode = clipping_node
	self.clippingNode.isTouchable = false

	for i = 1, BUTTON_COUNT-1 do
		button = director:createUIButton( {
            source = self.settingsAnim[i + 1],
            x = 0,
            y= i * height * 0.9,
            xAnchor = 0.5,
            yAnchor = 1,
            defaultFrame = i,
            pressedFrame = i
        })
		self.clippingNode:addChild(button)
        self.buttons[i + 1] = button
		button:addEventListener("activated", self)
		SpriteButton.addAnimatedButtonToSprite(self.buttons[i + 1], "Assets/Audio/SFX_Confirm.raw")

		button.toggle = director:createSprite( {
            source=self.redCrossAnim,
            isVisible= false,
            x = button.h * 0.5 + 8 * UI_SCALE,
            y = button.w * 0.5 - 8 * UI_SCALE,
            xAnchor = 0.5,
            yAnchor = 0.5,
			xScale = 0.5,
			yScale = 0.5,
            isTouchable = false,
            })
		button:addChild(button.toggle)
	end

	if audio.musicVolume == 0 then
		self.buttons[BUTTON_MUSIC].toggle.isVisible = true
	end
	if audio.sfxVolume == 0 then
		self.buttons[BUTTON_SOUND].toggle.isVisible = true
	end
	
	--[[
    editor depreciated
    self.editButton = director:createSprite(self.dw/2, self.dh/2 - 80, EDITOR_BUTTON)
	self.editButton.xAnchor = 0.5
	self.editButton.yAnchor = 0.5
	self.editButton:addEventListener("touch", self)	
	self.node:addChild(self.editButton)
	]]

	if CMW_G_SHOW_DEBUG_BUTTONS then
--		local b = director:createAnimation( director:createAtlas("Assets/UI/close_button.png") )
        local atlas =  director:createAtlas( {
	    width = 81,
	    height = 105,
	    numFrames = 2,
	    textureName = "Assets/UI/close_button.png"
	    } )
	
	    local b = director:createAnimation( {
	    start = 1,
	    count = 2,
	    atlas = atlas
	    } )

		local button = function(x,y,name)
			newButton = director:createUIButton( { source = b, xScale=0.5 * UI_SCALE, yScale=0.5*UI_SCALE, x=x, y=y, xAnchor = 0, yAnchor = 1, defaultFrame = 1, pressedFrame = 1 })
			SpriteButton.addAnimatedButtonToSprite(newButton, "Assets/Audio/SFX_Confirm.raw")
			newButton:addEventListener("activated", self)	
			self.node:addChild(newButton)

			local text = director:createLabel( { x=newButton.w, w=director.displayWidth/newButton.xScale, color = GAME_FONT_COLOR, font = GAME_FONT, text=name, hAlignment = "left", vAlignment = "middle" })
			newButton:addChild(text)
			return newButton
		end

		self.unlockAllButton = button(0, self.dh, "UNLOCK ALL")
		self.lockAllButton = button(self.dw/2, self.dh, "RESET ALL")
		self.world1Button = button(0, self.dh*0.92, "END OF WORLD 1")
		self.billingTest = button(self.dw/5, self.dh*0.85, "UNLCK restore test mode")
		self.purchaseUnlckTest = button(self.dw/5, self.dh*0.80, "Execute purchase UNLCK")
		self.purchaseUnlckTest.isVisible = false
		self.unlockFullGameCheat = button(self.dw/5, self.dh*0.75, "UNLCK full game cheat")
		self.unlockPurchaseTestMode = button(self.dw/5, self.dh*0.7, "UNLCK purchase test enabled")
		self.zeroHints = button(self.dw/5, self.dh*0.65, "Zero hints")
	end

	self:updateSoundOptions(false)
	if(not audio:isStreamPlaying()) then
		if(audio.musicVolume > 0) then
			audio:playStream("Assets/Audio/MUS_FrontEnd.mp3", true)
		end
	end
	system:addEventListener( { "buttonActionPerformed", "update", "backKey"}, self)
	dialogManager:setUp()
end


function mainMenu:update(event)

	-- Make signal blink once or twice every few seconds
	self.signalHeadCount = self.signalHeadCount - 1

	if self.signalHeadCount <= 0 then
		self.signalHead:play({startFrame=1, loopCount=math.random(1,2)})
		self.signalHeadCount = math.random(120,240)
	end

	-- Strobe the glow around the play button
	self.playGlow.alpha = self.playGlow.alpha + self.playGlowDelta / 60

	--[[ removed slow pulsing on text size and color for now as it didn't look good in-game
	self.playText.xScale = self.playText.xScale + self.playGlowDelta / 120
	self.playText.yScale = self.playText.yScale + self.playGlowDelta / 120
	self.playText.color.r = self.playText.color.r + (GAME_FONT_COLOR.r - GAME_FONT_COLOR_GREEN.r) * self.playGlowDelta / 15
	]]

	if self.playGlow.alpha >= 1.0 then

		self.playGlow.alpha = 1.0
		self.playGlowDelta = -1
		--[[
		self.playText.xScale = 1.0
		self.playText.yScale = 1.0
		self.playText.color.r = GAME_FONT_COLOR.r
		]]

	elseif self.playGlow.alpha <= 0.5 then

		self.playGlow.alpha = 0.5
		self.playGlowDelta = 1
		--[[
		self.playText.xScale = 0.75
		self.playText.yScale = 0.75
		self.playText.color.r = GAME_FONT_COLOR_GREEN.r
		]]
	end

	-- Handle smoke animation
	self.cloudFrameNumber = self.cloudFrameNumber + 1

	if self.cloudFrameNumber >= 516 then
		self.cloudFrameNumber = 0
	end

	-- Cycle through all the clouds
	for i = 1, #self.clouds do

		for k = 1, #self.clouds[i] do

			-- Pattern repeats every 43*4 frames, so modify frame number based on which cloud instance we are
			local instanceFrameNumber = self.cloudFrameNumber - (172 * (k-1))

			-- Loop around when we reach the end of the animation
			if instanceFrameNumber < 0 and self.cloudKeyFrames[i].animBegan[k] ~= nil then
			
				instanceFrameNumber = instanceFrameNumber + 516
			end

			-- If we've not initialised this cloud yet...
			if self.cloudKeyFrames[i].currentKeyFrame[k] == nil then

				-- Do so now
				self.cloudKeyFrames[i].currentKeyFrame[k] = 0
			end

			-- If we've reached the next keyframe...
			if instanceFrameNumber == self.cloudKeyFrames[i][self.cloudKeyFrames[i].currentKeyFrame[k] + 1].frame then

				-- Update data
				self.cloudKeyFrames[i].currentKeyFrame[k] = self.cloudKeyFrames[i].currentKeyFrame[k] + 1

				-- If this is the first or last keyframe, jump to these values
				if self.cloudKeyFrames[i].currentKeyFrame[k] == 1 or self.cloudKeyFrames[i].currentKeyFrame[k] == #self.cloudKeyFrames[i] then

					self.clouds[i][k].x = self.cloudKeyFrames[i][self.cloudKeyFrames[i].currentKeyFrame[k]].posX
					self.clouds[i][k].y = self.cloudKeyFrames[i][self.cloudKeyFrames[i].currentKeyFrame[k]].posY
					self.clouds[i][k].alpha = self.cloudKeyFrames[i][self.cloudKeyFrames[i].currentKeyFrame[k]].alpha
					self.clouds[i][k].xScale = self.cloudKeyFrames[i][self.cloudKeyFrames[i].currentKeyFrame[k]].scale
					self.clouds[i][k].yScale = self.cloudKeyFrames[i][self.cloudKeyFrames[i].currentKeyFrame[k]].scale
					self.clouds[i][k].rotation = self.cloudKeyFrames[i][self.cloudKeyFrames[i].currentKeyFrame[k]].rot

					self.cloudKeyFrames[i].animBegan[k] = true
				end

				-- If this is the last keyframe, go back to the beginning
				if self.cloudKeyFrames[i].currentKeyFrame[k] == #self.cloudKeyFrames[i] then

					self.cloudKeyFrames[i].currentKeyFrame[k] = 0

				-- Otherwise, figure out delta to next keyframe
				else

					local nextX = self.cloudKeyFrames[i][self.cloudKeyFrames[i].currentKeyFrame[k] + 1].posX - self.clouds[i][k].x 
					local nextY = self.cloudKeyFrames[i][self.cloudKeyFrames[i].currentKeyFrame[k] + 1].posY - self.clouds[i][k].y
					local nextAlpha = self.cloudKeyFrames[i][self.cloudKeyFrames[i].currentKeyFrame[k] + 1].alpha - self.clouds[i][k].alpha
					local nextScale = self.cloudKeyFrames[i][self.cloudKeyFrames[i].currentKeyFrame[k] + 1].scale - self.clouds[i][k].xScale
					local nextRot = self.cloudKeyFrames[i][self.cloudKeyFrames[i].currentKeyFrame[k] + 1].rot - self.clouds[i][k].rotation
					local frames = self.cloudKeyFrames[i][self.cloudKeyFrames[i].currentKeyFrame[k] + 1].frame - self.cloudKeyFrames[i][self.cloudKeyFrames[i].currentKeyFrame[k]].frame

					self.clouds[i][k].dx = nextX / frames
					self.clouds[i][k].dy = nextY / frames
					self.clouds[i][k].dalpha = nextAlpha / frames
					self.clouds[i][k].dscale = nextScale / frames
					self.clouds[i][k].drotation = nextRot / frames
				end
			end

			-- If we're animating
			if self.cloudKeyFrames[i].currentKeyFrame[k] > 0 then

				self.clouds[i][k].x = self.clouds[i][k].x + self.clouds[i][k].dx
				self.clouds[i][k].y = self.clouds[i][k].y + self.clouds[i][k].dy
				self.clouds[i][k].alpha = self.clouds[i][k].alpha + self.clouds[i][k].dalpha
				self.clouds[i][k].xScale = self.clouds[i][k].xScale + self.clouds[i][k].dscale
				self.clouds[i][k].yScale = self.clouds[i][k].yScale + self.clouds[i][k].dscale
				self.clouds[i][k].rotation = self.clouds[i][k].rotation + self.clouds[i][k].drotation
			end
		end
	end
end

function mainMenu:tearDown(event)
    dbg.print("mainMenu:tearDown")
	dialogManager:tearDown()
	system:removeEventListener({"buttonActionPerformed","update", "backKey", "dialog"}, self)
	SpriteButton.tearDown()
	
	self.bg = nil
	self.clouds = nil
	self.playButton = nil
	self.playGlow = nil
	self.playText = nil
	self.playGlow = nil	
    self.buttons = nil
    self.clippingNode = nil
    self.fbButton = nil
    self.signalHead = nil
--   Editor depreciated self.editButton = nil

    self.node:removeFromParent()
    self.node = nil

--    dbg.printTable(self)

end

function mainMenu:updateSoundOptions(saveChanges)
	local oldVolume = audio.musicVolume
	if self.buttons[BUTTON_MUSIC].toggle.isVisible then
		audio.musicVolume = 0
	else
		audio.musicVolume = 1
	end
	--should we start play the music?
	if(oldVolume ~= audio.musicVolume)then
		if(audio.musicVolume > 0) then
			audio:playStream("Assets/Audio/MUS_FrontEnd.mp3", true)
		else
			audio:stopStream()
		end
	end
	
	if self.buttons[BUTTON_SOUND].toggle.isVisible then
		audio.sfxVolume = 0
	else
		audio.sfxVolume = 1
	end	
	
	dbg.print("audio.sfxVolume " ..audio.sfxVolume)
	dbg.print("audio.musicVolume " ..audio.musicVolume)
	if(saveChanges == true)then
    	SaveOptions()
	end
end

function mainMenu:dialogCheat1()
	for i = 1,#worldInfos do
		worldInfos[i].isLocked = false
		for j = 1, #worldInfos[i].levels do
			worldInfos[i].levels[j].isLocked = false
		end
	end
	if worldInfos[1].levels[#worldInfos[1].levels].numStars == 0 then
		worldInfos[1].levels[#worldInfos[1].levels].numStars = 1 -- So the "world 1 completed" check succeeds
	end
	hintsPurchaseManager.fullGameUnlocked = true
	g_saveData()	
end

function mainMenu:dialogCheat1_1()
	for i = 1,#worldInfos do
		worldInfos[i].isLocked = (i > 1)
		for j = 1, #worldInfos[i].levels do
			worldInfos[i].levels[j].isLocked = (i>1 or j>1)
			worldInfos[i].levels[j].numStars = 0
		end
	end
	tutorialsManager:resetAllTutorials()
	g_saveData()
end

function mainMenu:dialogCheat1_2()
		for i = 1,#worldInfos do
			worldInfos[i].isLocked = (i > 1)
			for j = 1, #worldInfos[i].levels do
				worldInfos[i].levels[j].isLocked = (i>1)
				worldInfos[i].levels[j].numStars = ((i==1 and j<30) and 3 or 0)
			end
		end
		g_saveData()
end

function mainMenu:dialog(event)
	--dbg.print("DIALOG "..event.id.." "..event.buttonIndex)
	dbg.print("mainMenu:dialog mainMenu:dialog mainMenu:dialog")
	dbg.printTable(event)
	if(event.id == G_QUIT_APPLICATION_DIALOG_ID) then
		if(event.buttonIndex == 0)then
			system:quit()
		end
	end
	--system:removeEventListener("dialog", self)
	if event.id==DIALOGUE_CHEAT1 and event.buttonIndex==0 then
		-- unlock
		self:dialogCheat1()
	end
	if event.id==DIALOGUE_CHEAT1+1 and event.buttonIndex==0 then
		-- reset
		self:dialogCheat1_1()		
	end	
	if event.id==DIALOGUE_CHEAT1+2 and event.buttonIndex==0 then
		-- end of world 1
		self:dialogCheat1_2()
	end
	
end

function mainMenu:activated(event)
    local target = event.target
	if target == self.buttons[BUTTON_MUSIC] then
		self.buttons[BUTTON_MUSIC].toggle.isVisible = not self.buttons[BUTTON_MUSIC].toggle.isVisible 
		self:updateSoundOptions(true)
		--[[
		if self.buttons[BUTTON_MUSIC].toggle.isVisible then
			audio.musicVolume = 1
			self.buttons[BUTTON_MUSIC].toggle.isVisible = false
		else
			audio.musicVolume = 0
			self.buttons[BUTTON_MUSIC].toggle.isVisible = true
		end
		--]]
	elseif target == self.buttons[BUTTON_SOUND] then
		self.buttons[BUTTON_SOUND].toggle.isVisible = not self.buttons[BUTTON_SOUND].toggle.isVisible
		self:updateSoundOptions(true)
		--[[
		if self.buttons[BUTTON_SOUND].toggle.isVisible then
			audio.sfxVolume = 1
			self.buttons[BUTTON_SOUND].toggle.isVisible = false
		else
			audio.sfxVolume = 0
			self.buttons[BUTTON_SOUND].toggle.isVisible = true
		end
		--]]
	elseif target == self.buttons[BUTTON_CREDITS] then
		system:sendEvent("goToCredits")
	elseif self.unlockAllButton ~= nil and target == self.unlockAllButton then
		--system:addEventListener("dialog", self)
		if(not dialogManager:displayOkCancelDialog("unlock all", "Are you sure?", "Yes", "No", DIALOGUE_CHEAT1)) then
			self:dialogCheat1()--on blackberry there is not dialog extenison so we assume yes
		end
	elseif self.lockAllButton ~= nil and target == self.lockAllButton then
		--system:addEventListener("dialog", self)
		if(not dialogManager:displayOkCancelDialog("Reset all", "Are you sure? This will reset all progress, MTX and tutorials", "Yes", "No", DIALOGUE_CHEAT1+1)) then
			self:dialogCheat1_1()--on blackberry there is not dialog extenison so we assume yes
		end
	elseif self.world1Button ~= nil and target == self.world1Button then
		--system:addEventListener("dialog", self)
		if(not dialogManager:displayOkCancelDialog("End of world 1", "Are you sure you want to set the first 29 levels to complete?", "Yes", "No", DIALOGUE_CHEAT1+2)) then
			self:dialogCheat1_2()--on blackberry there is not dialog extenison so we assume yes
		end
	end
end

function mainMenu:buttonActionPerformed(event)
	if(event.sprite == self.playText) then
		system:sendEvent("onPlay", {levelId = 1})
	elseif (event.sprite == self.buttons[BUTTON_SETTINGS]) then
		audio:playSound("Assets/Audio/SFX_Swipe.raw")
		local height = self.buttons[BUTTON_SETTINGS].h
		if self.settingsPanelOpened then
			tween:to(self.clippingNode, { x = 0, y = -3 * height, time=0.25 } )
			for i = 2, BUTTON_COUNT do
				tween:to(self.buttons[i], { alpha = 0, time=0.25 } )
			end
			self.settingsPanelOpened = false
			self.clippingNode.isTouchable = false
		else
			tween:to(self.clippingNode, { x = 0, y = 0, time=0.25 } )
			for i = 2, BUTTON_COUNT do
				tween:to(self.buttons[i], { alpha=1 , time=0.25 } )
			end
			self.settingsPanelOpened = true
			self.clippingNode.isTouchable = true
		end
	else
		
	end
	if(self.billingTest ~= nil) then
		if (event.sprite == self.billingTest) then
			g_enableFullGameUnlockRestoreTest()
			self.purchaseUnlckTest.isVisible = true
		end
	end
	if(self.unlockPurchaseTestMode ~= nil) then
		if (event.sprite == self.unlockPurchaseTestMode) then
			g_enableFullGameUnlockPurchaseTestMode()
		end
	end
	if(self.zeroHints ~= nil)then
		if(event.sprite == self.zeroHints)then
			g_zeroHintsAndSave()
		end
	end
	if(self.unlockFullGameCheat ~= nil) then
		if (event.sprite == self.unlockFullGameCheat) then
			g_unlockFullGameCheat()
		end		
	end
	if(self.purchaseUnlckTest ~= nil) then
		if (event.sprite == self.purchaseUnlckTest) then
			g_startProductPurchase(billingManager.UNLOCK_FULL_GAME_PROD_ID)
		end
	end

end

function mainMenu:touch(event)
    local target = event.target
	--dbg.print("hello touch event!!")
	if event.phase == "began" then
		if target == self.playText then
            --system:setFocus(target)
			--audio:playSound("Assets/Audio/SFX_Confirm.raw")
            --return true
			--system:sendEvent("onPlay", {levelId = 1})
		elseif target == self.fbButton then
            system:setFocus(target)
			audio:playSound("Assets/Audio/SFX_Confirm.raw")
            return true
		elseif target == self.editButton then
            system:setFocus(target)
            return true
		elseif target == self.buttons[BUTTON_SETTINGS] then
			--[[
			audio:playSound("Assets/Audio/SFX_Swipe.raw")
			local height = self.buttons[BUTTON_SETTINGS].h
			if self.settingsPanelOpened then
				tween:to(self.clippingNode, { x = 0, y = -3 * height, time=0.25 } )
				for i = 2, 4 do
					tween:to(self.buttons[i], { alpha = 0, time=0.25 } )
				end
				self.settingsPanelOpened = false
			else
				tween:to(self.clippingNode, { x = 0, y = 0, time=0.25 } )
				for i = 2, 4 do
					tween:to(self.buttons[i], { alpha=1 , time=0.25 } )
				end
				self.settingsPanelOpened = true
			end
            return true
			--]]
		end
    elseif event.phase == "ended" and system:getFocus()==target then
        system:setFocus(nil)
		if target == self.playText then
			--system:sendEvent("onPlay", {levelId = 1})
            --return true
		elseif target == self.editButton then
			system:sendEvent("onEdit")
            return true
		elseif target == self.fbButton then
			browser:launchURL("http://www.facebook.com/SignalToTheStars")
            return true
        end
	end

    return false
end
