--

outroCutscene = {}
outroCutscene_mt = { __index = outroCutscene }

function outroCutscene.create()
	local instance = {}
	setmetatable( instance, outroCutscene_mt)
	instance.scene = director:createScene()
	instance.scene.name = "Scene: Outro Cutscene"
	instance.scene:addEventListener( { "setUp" , "tearDown"}, instance)
	return instance
end

function outroCutscene:setUp(event)

	self.node = director:createNode()
	self.scene:addChild(self.node)

	self.fadeScreen = director:createRectangle( {
        x=director.displayCenterX,
        y=director.displayCenterY,
        w=director.displayWidth,
        h=director.displayHeight,
	    xAnchor = 0.5,
        yAnchor = 0.5,
        zOrder = 10000,
        IsVisible = true,
        color={0, 0, 0},
        strokeColor={0, 0, 0},
	    strokeAlpha = 1,	
	    alpha = 0
    } )
	self.node:addChild(self.fadeScreen)

	self.bg = director:createSprite( {
        source = "Assets/UI/Outro_Bg.png",
        x = director.displayCenterX,
        y = director.displayCenterY,
        xAnchor = 0.5,
        yAnchor = 0.5,
        xScale = UI_SCALE,
        yScale = UI_SCALE,
    })
	self.node:addChild(self.bg)
	
	self.pic1 = director:createSprite( {
        source = "Assets/UI/Outro_pic_1.png",
        x = 211,
        y = self.bg.h - 233,
        xAnchor = 0.5,
        yAnchor = 0.5,
        xScale = 1.0,
        yScale = 1.0,
		alpha = 0,
    })
	self.bg:addChild(self.pic1)

	self.pic2 = director:createSprite( {
        source = "Assets/UI/Outro_pic_2.png",
        x = 523,
        y = self.bg.h - 272,
        xAnchor = 0.5,
        yAnchor = 0.5,
        xScale = 1.0,
        yScale = 1.0,
		alpha = 0,
    })
	self.bg:addChild(self.pic2)

	self.pic3 = director:createSprite( {
        source = "Assets/UI/Outro_pic_3.png",
        x = 282,
        y = self.bg.h - 541,
        xAnchor = 0.5,
        yAnchor = 0.5,
		xScale = 1.0,
        yScale = 1.0,
		alpha = 0,
    })
	self.bg:addChild(self.pic3)

	self.pic4 = director:createSprite( {
        source = "Assets/UI/Outro_pic_4.png",
        x = 246,
        y = self.bg.h - 878,
        xAnchor = 0.5,
        yAnchor = 0.5,
        xScale = 1.0,
        yScale = 1.0,
		alpha = 0,
    })
	self.bg:addChild(self.pic4)

	self.pic5 = director:createSprite( {
        source = "Assets/UI/Outro_pic_5.png",
        x = 474,
        y = self.bg.h - 855,
        xAnchor = 0.5,
        yAnchor = 0.5,
        xScale = 1.0,
        yScale = 1.0,
		alpha = 0,
    })
	self.bg:addChild(self.pic5)
	
	-- Work-around for tweening onComplete function being run on target object and unable to reference back to main scene
	self.pic1.backRef = self
	self.pic2.backRef = self
	self.pic3.backRef = self
	self.pic4.backRef = self
	self.pic5.backRef = self
	self.fadeScreen.backRef = self
	
	-- Count progress through cutscene
	self.stepCount = 1
	self.stepTrigger = { self.pic1, self.pic2, self.pic3, self.pic4, self.pic5, self.fadeScreen }
	
	-- Add a listener for the player tapping the screen to rush through the pics
	system:addEventListener("touch", self)
	system:addEventListener("buttonActionPerformed", self)

	self.next = director:createSprite( {
        source = "Assets/UI/Popup_icon_next_button .png",
        x = director.displayWidth - 64 * UI_SCALE,
        y = 64 * UI_SCALE,
        xAnchor = 0.5,
        yAnchor = 0.5,
        xScale = UI_SCALE,
        yScale = UI_SCALE,
    })

	self.node:addChild(self.next)

	self.next.isVisible = false
	SpriteButton.addAnimatedButtonToSprite(self.next, "Assets/Audio/SFX_Confirm.raw")

	-- Start first tween
	tween:to(self.pic1, { alpha = 1, time = 2, onComplete = outroCutscene.onStepComplete })
	dialogManager:setUp()
end

function outroCutscene.onStepComplete(target)

	if target.backRef.stepCount < 5 and target == target.backRef.stepTrigger[target.backRef.stepCount] then
	
		target.backRef.stepCount = target.backRef.stepCount + 1
		tween:to(target.backRef.stepTrigger[target.backRef.stepCount], { alpha = 1, time = 2, onComplete = outroCutscene.onStepComplete })

	elseif target.backRef.stepCount == 5 and target == target.backRef.stepTrigger[5] then
	
		target.backRef.stepCount = target.backRef.stepCount + 1
		target.backRef.next.isVisible = true

	elseif target.backRef.stepCount == 6 and target == target.backRef.stepTrigger[6] then

		target.backRef.stepCount = target.backRef.stepCount + 1
		system:sendEvent("onBackToWorldSelect", {fromOutro = true})
	end
end

function outroCutscene:touch(event)

    local target = event.target

	if event.phase == "began" then
	
		if target == self.next and self.next.isVisible then
		
			-- Do nothing, this is handled in a separate event
			
		elseif self.stepCount < 6 then

			dbg.print("trying to skip step " .. self.stepCount)
			outroCutscene.onStepComplete(self.stepTrigger[self.stepCount])
		end
	end
end

function outroCutscene:buttonActionPerformed(event)
	if (event.sprite == self.next and self.next.isVisible) then
		tween:to(self.fadeScreen, { alpha = 1, delay = 0.5, time = 0.5, onComplete = outroCutscene.onStepComplete })
		self.next.isVisible = false
	end
end

function outroCutscene:tearDown(event)
	dialogManager:tearDown()
	
	self.bg:removeFromParent()
	self.bg = nil
	self.pic1 = nil
	self.pic2 = nil
	self.pic3 = nil
	self.pic4 = nil
	self.pic5 = nil
	self.next = nil

	self.fadeScreen:removeFromParent()
    self.fadeScreen = nil

    self.node:removeFromParent()
    self.node = nil
	
	system:removeEventListener("touch", self)
	system:removeEventListener("buttonActionPerformed", self)
	SpriteButton.tearDown()
end
