--

worldSelectScreen = {}
worldSelect_mt = { __index = worldSelectScreen }

worldCoordinates = {{x=242,y=379},{x=528,y=379},{x=247,y=792},{x=528,y=792}}

function worldSelectScreen.create()
	local instance = {}
	setmetatable( instance, worldSelect_mt)
	instance.scene = director:createScene()
	instance.scene.name = "Scene: WorldSelect"
	instance.dw = director.displayWidth
	instance.dh = director.displayHeight
	instance.scene:addEventListener({ "setUp" , "tearDown"}, instance)
	return instance
end

function worldSelectScreen:backKey(event)
	dbg.print("worldSelectScreen:backKey")
	dbg.printTable(event)
	system:sendEvent("onBackToMainMenu")
end

function worldSelectScreen:LoadAssets()
    local atlas

	atlas = director:createAtlas( "Assets/UI/WORLD_SELECTION_2 .png" )
	self.bgAnim = director:createAnimation( atlas )

	atlas = director:createAtlas( "Assets/UI/back_arrow.png" )
	self.backButtonAnim = director:createAnimation( atlas )
    
	atlas = director:createAtlas( "Assets/UI/padlock_large.png" )
	self.padlockAnim = director:createAnimation( atlas )

	atlas = director:createAtlas( "Assets/UI/star.png" )
	self.starAnim = director:createAnimation( atlas )

    self.worldIcons = {}

	for i = 1,#worldInfos do

        local anims = {}

	    atlas = director:createAtlas( worldInfos[i].menuIcon )
	    anims[false] = director:createAnimation( atlas )

	    atlas = director:createAtlas( worldInfos[i].menuIconLocked )
	    anims[true] = director:createAnimation( atlas )

        self.worldIcons[i] = anims

    end

end

function worldSelectScreen:UnloadAssets()
    
    self.bgAnim = nil
    self.backButtonAnim = nil
    self.padlockAnim = nil
    self.starAnim = nil
    self.worldIcons = nil
    
end

function worldSelectScreen:setUp(event)
    dbg.print("worldSelectScreen:setUp")

	self.node = director:createNode()
	self.scene:addChild(self.node)

	local bg = director:createSprite( {
        source = self.bgAnim,
        x = director.displayCenterX,
        y = director.displayCenterY,
        xAnchor = 0.5,
        yAnchor = 0.5,
        xScale = UI_SCALE,
        yScale = UI_SCALE,
        zOrder = 1,
        isTouchable = false
    })
	self.node:addChild(bg)

	self.backButton = director:createSprite( {
        source=self.backButtonAnim,
        x = 16 * UI_SCALE,
        y = director.displayHeight - (16 * UI_SCALE),
        xAnchor = 0, yAnchor = 1,
        xScale = UI_SCALE, yScale = UI_SCALE,
        zOrder = 10
    })
	SpriteButton.addAnimatedButtonToSprite(self.backButton, "Assets/Audio/SFX_Back.raw")
	self.node:addChild(self.backButton)

	local title = director:createLabel( {
        x = 0, y = self.backButton.y,
        w = director.displayWidth, h = self.backButton.h * UI_SCALE,
        textXScale = UI_SCALE * 0.7, textYScale = UI_SCALE * 0.7,
        xAnchor = 0, yAnchor = 1,
        text = getString(S_MENU_WORLD_SELECT),
        font = GAME_FONT_TITLE,
        color = GAME_FONT_COLOR,
        hAlignment="centre", vAlignment="middle",
        zOrder = 10
    })
	self.node:addChild( title)

	local buttonWidth = 367
	local numHorizontalSprites = math.floor(DEFAULT_UI_WIDTH / (buttonWidth + buttonWidth * 0.25))
	local offset = ((DEFAULT_UI_WIDTH - (numHorizontalSprites * buttonWidth)) / 2) * UI_SCALE
	
	for i = 1,#worldInfos do 
		local x = director.displayCenterX + (worldCoordinates[i].x - 384) * UI_SCALE
		local y = director.displayCenterY - (worldCoordinates[i].y - 568) * UI_SCALE

		--a dirty patch
		local spriteSource = self.worldIcons[i][worldInfos[i].isLocked]

		local sprite = director:createSprite( {
            --source = self.worldIcons[i][worldInfos[i].isLocked],
			source = spriteSource,
            x = x,
            y = y,
            xAnchor = 0.5,
            yAnchor = 0.5,
			zOrder = -5.0,
            xScale = UI_SCALE,
            yScale = UI_SCALE
        })
		sprite.worldId = i
		self.node:addChild(sprite)
		
		local star = director:createSprite( {
            source = self.starAnim,
            x = sprite.x + 40 * UI_SCALE,
            y = sprite.y - 155 * UI_SCALE,
            xAnchor=0.5,
            yAnchor=0.5,
            xScale = UI_SCALE,
            yScale = UI_SCALE,
            zOrder = 10,
            isTouchable = false
        })
		self.node:addChild(star)
		
		local startCount = director:createLabel({
            x = 14,
            w = star.w * 2,
            hAlignment = "right", vAlignment = "middle",
            text = self:getStarCount(worldInfos[i]).."/"..#worldInfos[i].levels * 3,
            xAnchor = 1,
			textXScale = 0.35, textYScale = 0.5,
            font = GAME_FONT,
            color = GAME_FONT_COLOR,
            isTouchable = false
        })
		star:addChild(startCount)
		
		-- Show padlocks if the world is locked
		if worldInfos[i].isLocked == false then	
			dbg.print(" dbg 1 i = " ..i)
			SpriteButton.addAnimatedButtonToSprite(sprite, "Assets/Audio/SFX_Confirm.raw", FlashOnlyButtonAnimation.new(sprite))
		else
			dbg.print(" dbg 2 i = " ..i)
			local padlock = director:createSprite( {
				source = self.padlockAnim,
				xAnchor = 0.5,
				yAnchor = 0.5,
				x =sprite.w / 2,
				y= sprite.h / 2,
				isTouchable = false
			})
			sprite:addChild(padlock)
			SpriteButton.addAnimatedButtonToSprite(sprite, "Assets/Audio/SFX_BlockedTile.raw", DefaultButtonAnimation.new(padlock))
		end

	end

	if(not audio:isStreamPlaying()) then
		if(audio.musicVolume > 0) then
			audio:playStream("Assets/Audio/MUS_FrontEnd.mp3", true)
		end
	end

	system:addEventListener({"buttonActionPerformed", "backKey"}, self)
	dialogManager:setUp()
end

function worldSelectScreen:getStarCount(worldInfo)
	local starCount = 0
	for i = 1, #worldInfo.levels do
		starCount = starCount + worldInfo.levels[i].numStars
	end
	return starCount
end

function worldSelectScreen:buttonActionPerformed(event)
	--dbg.print("worldSelectScreen:buttonActionPerformed")
	--dbg.printTable(event)
	--self.worldIcons[i][worldInfos[i].isLocked]
		
	if(event.sprite == self.backButton) then
		system:sendEvent("onBackToMainMenu")
	-- if we tapped on a chapter...
	elseif (event.sprite.worldId ~= nil) then
		-- if the chapter is unlocked
		if not worldInfos[event.sprite.worldId].isLocked then
			-- if the chapter needs to be purchased
			--if event.sprite.worldId > 1 and not hintsPurchaseManager:isFullGameUnlocked() then
			if event.sprite.worldId > 1 and not g_isFullGameUnlocked() then
				system:sendEvent("goToUnlockScreen")
			else
				system:sendEvent("onWorldSelect", {worldInfo = worldInfos[event.sprite.worldId]})
			end
		end
	end
end

function worldSelectScreen:tearDown(event)
    dbg.print("worldSelectScreen:tearDown")
	SpriteButton.tearDown()
	dialogManager:tearDown()
	dbg.print("worldSelectScreen:tearDown 2")
	system:removeEventListener( {"buttonActionPerformed", "backKey"}, self)
	dbg.print("worldSelectScreen:tearDown 3")
    self.backButton = nil
	dbg.print("worldSelectScreen:tearDown 4")
    self.node:removeFromParent()
	dbg.print("worldSelectScreen:tearDown 5")
    self.node = nil
	dbg.print("worldSelectScreen:tearDown 6")

end
