--------------------------------------------------------------------------------
-- UIButtons
--------------------------------------------------------------------------------
CLUIButton = {}
CLUIButton.__index = CLUIButton

function CLUIButton:touch( event)
	if self:isEnabled() then
		local node = event.target
		if event.phase == "began" then
			system:setFocus(node)
			node:setFrame(self.pressedFrame)
			if node.labelTextPressed ~= nil then
				node.label.text=node.labelTextPressed
			end
			if node.labelColorPressed ~= nil then
				node.label.color=node.labelColorPressed
			end
			
			if node.pressSound ~= nil then
				audio:playSound(node.pressSound)
			end

		elseif event.phase == "ended" and system:getFocus() == node then
			gestureManager:tapConsumed()
			system:setFocus(nil)
			node:setFrame(self.defaultFrame)
			--system:sendEvent("activated", { node = node })
			local e = QEvent:create("activated", { target = node } )
			node:handleEvent(e)
			if node.labelText ~= nil then
				node.label.text=node.labelText
			end
			if node.labelColor ~= nil then
				node.label.color=node.labelColor
			end

			if node.releaseSound ~= nil then
				audio:playSound(node.releaseSound)
			end

		end
	else
		return false
		--dbg.print("button " ..self.name .." is NOT enabled")
	end
	return true
end

--------------------------------------------------------------------------------
-- Private API
--------------------------------------------------------------------------------
function CLUIButton:initUIButton(l,values)
    -- Setup an inheritence from CLSprite
    local lp = {}
    setmetatable(lp,CLUIButton)

    l.touch = CLUIButton.touch

    -- Add a touch event listener to this button
   l:addEventListener("touch", l)

   l.defaultFrame = 1
   l.pressedFrame = 2
   if values.defaultFrame ~= nil then
		l.defaultFrame = values.defaultFrame
   end
   if values.pressedFrame ~= nil then
		l.pressedFrame = values.pressedFrame
   end
    -- Set the correct frame, stop the animation and update the w/h
    l:setFrame(l.defaultFrame)

    -- Setup the label
    if values.labelText ~= nil then
        l.labelText=values.labelText
        l.labelColor=values.labelColor
        l.labelTextPressed=values.labelTextPressed
        l.labelColorPressed=values.labelColorPressed

        l.label = director:createLabel( {
        x=0, y=0, w=l.w, h=l.h,
   		hAlignment="centre", vAlignment="middle",
        text=l.labelText,color=l.labelColor,
        font=l.labelFont
        } )
        l:addChild(l.label)
    end

    if values.pressSound ~= nil then
        l.pressSound = values.pressSound
        audio:loadSound(l.pressSound)
    end

    if values.releaseSound ~= nil then
        l.releaseSound = values.releaseSound
        audio:loadSound(l.releaseSound)
    end
end

--------------------------------------------------------------------------------
-- Public API
--------------------------------------------------------------------------------

-- See director:createUIButton() for factory function

--[[
/**
Create a UI Button, specifying arbitrary input values.
@param values Lua table specifying name/value pairs.
@return The created button.
*/
--]]
function director:createUIButton(values)
    local l = self:createSprite( values)
    CLUIButton:initUIButton(l, values)

	--initializing the states of the button and their methods
	l.enabled = true --default value
	l.disable = 	function(buttonInstance)
						--dbg.print("disabling button " ..buttonInstance.name)
						buttonInstance.enabled = false
					end
	l.enable = 		function(buttonInstance)
						--dbg.print("enabling button " ..buttonInstance.name)
						buttonInstance.enabled = true
					end
	l.isEnabled = 	function(buttonInstance)
						return buttonInstance.enabled
					end

	--TODO should we define a state machine instead?
    return l
end

--[[
/**
The following methods are defined inside the createUIButton function call
*/
--]]

--[[
function CLUIButton:enable()
	self.enabled = true
end

function CLUIButton:disable()
	self.enabled = false
end

function CLUIButton:isEnabled()
	return self.enabled 
end
--]]