--------------------------------------------------------------------------------
-- UIToggles
--------------------------------------------------------------------------------
CLUIToggle = {}
CLUIToggle.__index = CLUIToggle

function CLUIToggle:touch( event)
	if not self.enableInputs then
		return false
	end
	local node = event.target
    if event.phase == "began" then
        if node.state == "on" then
            node.state = "off"

            if node.toggleOffSound ~= nil then
                audio:playSound(node.toggleOffSound)
            end
        else
            node.state = "on"

            if node.toggleOnSound ~= nil then
                audio:playSound(node.toggleOnSound)
            end
        end

        node:setFrame( (node.state == "on" and 2) or 1)

		local e = QEvent:create("toggled", { target = node } )
        node:handleEvent(e)
    end

    return true
end

--------------------------------------------------------------------------------
-- Private API
--------------------------------------------------------------------------------
function CLUIToggle:initUIToggle(l,values)
    -- Setup an inheritence from CLSprite
    local lp = {}
    setmetatable(lp,CLUIToggle)

    l.touch = CLUIToggle.touch

    -- Add a touch event listener to this button
    l:addEventListener("touch", l)

    -- Set the initial state
    l.state = values.state or "off"
	if values.enableInputs ~= nil then
		l.enableInputs = values.enableInputs
	else
		l.enableInputs = true
	end
    -- Set the correct frame, stop the animation and update the w/h
    l:setFrame( (l.state == "on" and 2) or 1)

    if values.toggleOnSound ~= nil then
        l.toggleOnSound = values.toggleOnSound
        audio:loadSound(l.toggleOnSound)
    end

    if values.toggleOffSound ~= nil then
        l.toggleOffSound = values.toggleOffSound
        audio:loadSound(l.toggleOffSound)
    end
	
	l.toggle = function(instance, value)
		instance.state = value or "off"
		instance:setFrame( (instance.state == "on" and 2) or 1)
	end
end

--------------------------------------------------------------------------------
-- Public API
--------------------------------------------------------------------------------

-- See director:createUIToggle() for factory function

--[[
/**
Create a UI Toggle, specifying arbitrary input values.
@param values Lua table specifying name/value pairs.
@return The created toggle.
*/
--]]
function director:createUIToggle(values)
    local l = self:createSprite( values)
    CLUIToggle:initUIToggle(l, values)
    return l
end
