FlashOnlyButtonAnimation = {}

FlashOnlyButtonAnimation.ANIMATION_TIMER_LENGTH		= 0.1
FlashOnlyButtonAnimation.SCALE_FACTOR					= 1.1

FlashOnlyButtonAnimation.new = function(sprite)
	local self = {}
	local animationTimer = 0.0
	local enabled = true
	
	local glowingSprite = director:createSprite({x=sprite.x, y=sprite.y, 
							xAnchor=sprite.xAnchor, yAnchor=sprite.yAnchor, 
							xScale=sprite.xScale, yScale=sprite.yScale,
							zOrder=sprite.zOrder+1,
							rotation = sprite.rotation,
							xFlip = sprite.xFlip,
							yFlip = sprite.yFlip,
							source=sprite.animation,
							alpha = 0.0,
							blendMode = "add"
							})

	if(sprite.parent ~= nil) then
		sprite.parent:addChild(glowingSprite)
	end
	

	self.animationInUpdate =  function(dt)
		local finished = false
		if(not enabled)then
			finished = true
		else
			animationTimer = animationTimer + dt
			if (animationTimer > FlashOnlyButtonAnimation.ANIMATION_TIMER_LENGTH) then
				animationTimer = FlashOnlyButtonAnimation.ANIMATION_TIMER_LENGTH
				finished = true
			end
			local progress = animationTimer / FlashOnlyButtonAnimation.ANIMATION_TIMER_LENGTH
			glowingSprite.alpha = progress
		end
		return finished
	end
	
	
	self.animationOutUpdate = function(dt)
		local finished = false
		if(not enabled)then
			finished = true
		else
			animationTimer = animationTimer - dt
			if (animationTimer < 0.0) then
				animationTimer = 0.0
				finished = true
			end	
			local progress = animationTimer / FlashOnlyButtonAnimation.ANIMATION_TIMER_LENGTH
			glowingSprite.alpha = progress
		end
		return finished
	end
	
	
	self.reset = function()
		animationTimer = 0.0
		glowingSprite.alpha = 0.0
		glowingSprite.x = sprite.x
		glowingSprite.y = sprite.y
		glowingSprite.rotation = sprite.rotation
	end
	
	self.enable = function()
		enabled = true
	end
	
	self.disable = function()
		enabled = false
		self.reset()
	end
	
	return self
end