--dofile("examples/Sprites And Animation/Create Sprites/defaultButtonAnimation.lua")

SpriteButton = {}

SpriteButton.IDLE_STATE					= -100000
SpriteButton.TRANSITION_IN_STATE		= -200000
SpriteButton.TRANSITION_OUT_STATE		= -300000

SpriteButton.instancesList = {}

function SpriteButton.cancelAllOtherActions(triggeringButton)
	for k,v in pairs(SpriteButton.instancesList) do
		if(v ~= triggeringButton)then
			v.cancelAction()
		end
	end		
end

function SpriteButton.cancelAllActions()
	for k,v in pairs(SpriteButton.instancesList) do
		v.cancelAction()
	end		
end

function SpriteButton.enableButton(controlledEntity)
	for k,v in pairs(SpriteButton.instancesList) do
		if(v.getSprite() == controlledEntity) then
			v.enable()
			--we found the button to enable, therefore the function can return
			return
		end
	end	
end

function SpriteButton.disableButton(controlledEntity)
	for k,v in pairs(SpriteButton.instancesList) do
		if(v.getSprite() == controlledEntity) then
			v.cancelAction()
			v.disable()
			--we found the button to disable, therefore the function can return
			return
		end
	end	
end

function SpriteButton.addAnimatedButtonToSprite(sprite, soundPath, buttonAnimation)
	local animatedButton = SpriteButton.new(sprite, soundPath, buttonAnimation)
	table.insert(SpriteButton.instancesList, animatedButton)
	return animatedButton
end

function SpriteButton.resetAll()
	for k,v in pairs(SpriteButton.instancesList) do
		v.reset()
	end	
end

function SpriteButton.tearDown()
	for k,v in pairs(SpriteButton.instancesList) do
		v.unregisterEvents()
		v.reset()
	end
	SpriteButton.instancesList = {}
end


SpriteButton.new = function(sprite, soundPath, buttonAnimation)
	local self = {}
	local state = SpriteButton.IDLE_STATE
	local actionsCount = 0
	local enabled = true
	local timer = 0.0

	buttonAnimation = buttonAnimation or nil
	if (buttonAnimation == nil) then
		buttonAnimation = DefaultButtonAnimation.new(sprite)
	end
	
	soundPath = soundPath or nil

	local getStateString = function(stateVal)
		local str = "UNKNOWN"
		if (stateVal == SpriteButton.IDLE_STATE) then
			str = "SpriteButton.IDLE_STATE"
		elseif (stateVal == SpriteButton.TRANSITION_IN_STATE) then
			str = "SpriteButton.TRANSITION_IN_STATE"
		elseif (stateVal == SpriteButton.TRANSITION_OUT_STATE) then
			str = "SpriteButton.TRANSITION_OUT_STATE"
		else
		
		end
		return str
	end
	
	local changeState = function(newState)
		if(state ~= newState) then
			if (newState == SpriteButton.IDLE_STATE) then
				
			elseif (newState == SpriteButton.TRANSITION_IN_STATE) then
				
			elseif (newState == SpriteButton.TRANSITION_OUT_STATE) then
				
			else
			
			end
			--dbg.print("change state from " ..getStateString(state) .." to " ..getStateString(newState))	
			state = newState
		end
	end
	
	local onPointerEnter = function()
		changeState(SpriteButton.TRANSITION_IN_STATE)
	end
	
	local onPointerExit = function()
		changeState(SpriteButton.TRANSITION_OUT_STATE)
	end	

	local playAssociatedSound = function()
		if(soundPath ~= nil) then
			audio:playSound(soundPath)
		end
	end

--PUBLIC METHODS

	self.getSprite = function()
		return sprite
	end
	
	self.disable = function()
		enabled = false
		buttonAnimation.disable()
	end
	
	self.enable = function()
		enabled = true
		buttonAnimation.enable()
	end

	self.cancelAction = function()
		actionsCount = 0
		self.pointerOut()		
	end

	self.touch = function(instance, event)
		if(enabled)then
			if(event.target == sprite) then
				if (event.phase == "began") then
					if(actionsCount ~= 2) then--it means that the action cannot be canceled once started, see ended event phase
						self.pointerOver()
						actionsCount = 1
					end
				elseif (event.phase == "ended") then
					if (actionsCount == 1) then
						actionsCount = 2
					elseif (actionsCount == 2) then
						system:sendEvent("buttonActionStarted", {sprite = sprite})
					else
						--actionsCount = 0
						--self.pointerOut()	
						self.cancelAction()				
					end
				elseif (event.phase == "moved") then
					self.pointerOver()
				else
					--ERROR
				end
				SpriteButton.cancelAllOtherActions(self)
				return true
			else
				if(actionsCount ~= 2) then
					self.cancelAction()
				end
			end
		end
	end

	--TODO this might be a private local method
	self.pointerOver = function()
		if(state ~= SpriteButton.TRANSITION_IN_STATE) then
			onPointerEnter()
		end
	end
	
	--TODO this might be a private local method
	self.pointerOut = function()
		if(state ~= SpriteButton.TRANSITION_OUT_STATE) then
			onPointerExit()
		end
	end
	
	
	self.update = function(instance, event)
		local dt = (event.time - timer) * 0.001
		timer = event.time
		
		if (state == SpriteButton.IDLE_STATE) then
			
		elseif (state == SpriteButton.TRANSITION_IN_STATE) then
			if(buttonAnimation.animationInUpdate(dt))then
				--ended
				if(actionsCount == 2) then
					playAssociatedSound()
					changeState(SpriteButton.TRANSITION_OUT_STATE)
				end	
			end
		elseif (state == SpriteButton.TRANSITION_OUT_STATE) then
			if(buttonAnimation.animationOutUpdate(dt))then
				--ended
				if(actionsCount == 2) then
					--fire action performed event
					actionsCount = 0
					changeState(SpriteButton.IDLE_STATE)
					system:sendEvent("buttonActionPerformed", {sprite = sprite})
				end				
			end
		else
			
		end
		
	end
	
	
	self.unregisterEvents = function()
		system:removeEventListener({"update", "touch"}, self)
	end
	
	self.reset = function()
		state = SpriteButton.IDLE_STATE
		actionsCount = 0
		timer = 0.0
		buttonAnimation.reset()
	end
	
	system:addEventListener({"update", "touch"}, self)
	sprite:addEventListener("touch", self)
	
	
	return self
end