--

dofile("gameBG.lua")
GRID_WIDTH = 8
GRID_HEIGHT = 8
TILE_SIZE = 128

HINT_SPRITE_START_FILENAME = "Assets/hintSpriteStart.png"
HINT_SPRITE_END_FILENAME = "Assets/hintSpriteEnd.png"
HINT_SPRITE_START_TYPE = 2
HINT_SPRITE_END_TYPE = 3
HINT_SPRITE_START_END_TYPE = 4

DEFAULT_BLOCK_FILENAME= "Assets/sprBlock_0.png"
WALL_BLOCK_FILENAME= "Assets/sprWall_0.png"
CURSOR_BLOCK_FILENAME= "Assets/sprPlayer_0.png"

BLOCK_TYPE_DEFAULT = 1
BLOCK_TYPE_WALL = 2
BLOCK_TYPE_ARROW = 3
BLOCK_TYPE_NUM = 4
BLOCK_TYPE_GHOST = 5
BLOCK_TYPE_FLIP = 6
BLOCK_TYPE_FARMER = 7
BLOCK_TYPE_PIPE = 8

GHOST_UP = 1
GHOST_DOWN = 2
GHOST_LEFT = 3
GHOST_RIGHT = 4

GHOST_FX_ROTATION = { }
GHOST_FX_ROTATION[GHOST_UP]		= 90
GHOST_FX_ROTATION[GHOST_DOWN]	= 270
GHOST_FX_ROTATION[GHOST_LEFT]	= 0
GHOST_FX_ROTATION[GHOST_RIGHT]	= 180

GHOST_FX_ANCHOR_X = { }
GHOST_FX_ANCHOR_X[GHOST_UP]		= 0
GHOST_FX_ANCHOR_X[GHOST_DOWN]	= 0
GHOST_FX_ANCHOR_X[GHOST_LEFT]	= -1
GHOST_FX_ANCHOR_X[GHOST_RIGHT]	= 1

GHOST_FX_ANCHOR_Y = { }
GHOST_FX_ANCHOR_Y[GHOST_UP]		= 1
GHOST_FX_ANCHOR_Y[GHOST_DOWN]	= -1
GHOST_FX_ANCHOR_Y[GHOST_LEFT]	= 0
GHOST_FX_ANCHOR_Y[GHOST_RIGHT]	= 0

BLOCK_NUM_MAX = 3

BLOCK_TYPE_SPRITE_ID = {1,4,5,6,9,10,11,10}

BLOCK_LINE_SPRITE_ID =
{
    {1},                -- TL corner
    {6},                -- TR corner
    {31},               -- BL corner
    {24},               -- BR corner
    {2,3,4,5},          -- horizontal
    {7,12,18,19},       -- vertical
    {21,22,28,35},      -- circle
    {23},               -- start/end
    {8,9,10,11,15},     -- plain grass
    {25},               -- crossroads
    {29},               -- cow U/D
    {26},               -- cow L/R
    {27},               -- cow D/L
    {33},               -- cow U/L
    {36},               -- cow D/R
    {42},               -- cow U/R
    {39},               -- cow R
    {47},               -- cow U
    {41},               -- cow D
    {38},               -- cow L
	{13}				-- ghost crossroads
}

-- Note: using cow tiles for up/down half lines to fix visual issue with the line appearing ahead of the cursor
CURSOR_HALF_LINE_SPRITE_ID = { L = 20, R = 14, U = 41, D = 47 }

AREA_COLORS = {{ r=255, g=255, b=255 }, { r=0, g=255, b=0 }, { r=255, g=0, b=0 }}

grid = {}
grid.REWIND_TILE_DELAY = 0.04
grid_mt = { __index = grid }

local FarmerVoice = 0

function grid.init()
	local instance = {}
	setmetatable( instance, grid_mt)
--	instance:preLoad()
	return instance
end

local preload_phase = 1

function grid:getBlockTypeDescription(blockType)
	local str = ""
	if (blockType == BLOCK_TYPE_DEFAULT) then
		str = "BLOCK_TYPE_DEFAULT"
	elseif (blockType == BLOCK_TYPE_WALL) then
		str = "BLOCK_TYPE_WALL"	
	elseif (blockType == BLOCK_TYPE_ARROW) then
		str = "BLOCK_TYPE_ARROW"
	elseif (blockType == BLOCK_TYPE_NUM) then
		str = "BLOCK_TYPE_NUM"
	elseif (blockType == BLOCK_TYPE_GHOST) then
		str = "BLOCK_TYPE_GHOST"
	elseif (blockType == BLOCK_TYPE_FLIP) then
		str = "BLOCK_TYPE_FLIP"
	elseif (blockType == BLOCK_TYPE_FARMER) then
		str = "BLOCK_TYPE_FARMER"
	elseif (blockType == BLOCK_TYPE_PIPE) then
	else
		str = "BLOCK_TYPE_UNKNOWN"
	end
	return str
end

function grid:preLoad()
 
    if preload_phase > 16 then
        return true
    end

 	local atlas
     dbg.print("grid:preLoad "..preload_phase)

    if preload_phase == 1 then
        collectgarbage("collect")
	
    elseif preload_phase == 2 then

        atlas = director:createAtlas( {
	        width = TILE_SIZE,
	        height = TILE_SIZE,
	        numFrames = 48,
	        textureName = "Assets/Glowing_line.png",
        } )
        self.completionAnim = director:createAnimation( {
	        start = 1,
	        count = 48,
	        atlas = atlas
        } )

    elseif preload_phase == 3 then
	    atlas = director:createAtlas( {
		    width = 256,
		    height = 256,
		    numFrames = 19,
		    textureName = "Assets/RemovalAtlas.png",
	    } )
	    self.removalAnim = director:createAnimation( {
		    start = 1,
		    count = 19,
		    atlas = atlas,
		    delay = 0.066
        } )

    elseif preload_phase == 4 then
	    atlas = director:createAtlas( {
		    width = 256,
		    height = 256,
		    numFrames = 60,
		    textureName = "Assets/CursorIdleAtlas.png",
        } )
	    self.cursorAnim = director:createAnimation( {
		    start = 1,
		    count = 60,
		    atlas = atlas,
		    delay = 0.066
        } )

    elseif preload_phase == 5 then
	    atlas = director:createAtlas( {
		    width = 128,
		    height = 128,
		    numFrames = 11,
		    textureName = "Assets/DirectionTileAtlas.png",
        } )
	    self.arrowAnim = director:createAnimation( {
		    start = 1,
		    count = 11,
		    atlas = atlas
        } )

    elseif preload_phase == 6 then
	    atlas = director:createAtlas( {
		    width = 128,
		    height = 155,
		    numFrames = 10,
		    textureName = "Assets/CowChewingGrass.png",
        } )
	    self.flipIdleAnim = director:createAnimation( {
	        start = 1,
	        count = 10,
	        atlas = atlas,
	        delay = 0.0666
        } )

    elseif preload_phase == 7 then
	    atlas = director:createAtlas( {
		    width = 128,
		    height = 155,
		    numFrames = 20,
		    textureName = "Assets/CowFloatUpAtlas.png",
        } )	
	    self.flipStartRemoveAnim = director:createAnimation( {
	        start = 1,
	        count = 20,
	        atlas = atlas,
	        delay = 0.066
        } )
	
    elseif preload_phase == 8 then
	    atlas = director:createAtlas( {
		    width = 128,
		    height = 155,
		    numFrames = 20,
		    textureName = "Assets/CowFloatingLoopAtlas.png",
        } )
	    self.flipStartRemoveLoopAnim = director:createAnimation( {
	        start = 1,
	        count = 20,
	        atlas = atlas,
	        delay = 0.066
        } )

    elseif preload_phase == 9 then
        atlas = director:createAtlas( {
		    width = 128,
		    height = 155,
		    numFrames = 10,
		    textureName = "Assets/RemoveCowAtlas.png",
        } )
	    self.flipEndRemoveAnim = director:createAnimation( {
	        start = 1,
	        count = 10,
	        atlas = atlas,
	        delay = 0.066
        } )
	
    elseif preload_phase == 10 then
	    atlas = director:createAtlas( {
		    width = 256,
		    height = 128,
		    numFrames = 9,
		    textureName = "Assets/GhostTileEntryAtlas.png",
        } )
	    self.ghostStartAnim = director:createAnimation( {
	        start = 1,
	        count = 9,
	        atlas = atlas,
	        delay = 0.066
        } )
	
    elseif preload_phase == 11 then
	    atlas = director:createAtlas( {
		    width = 128,
		    height = 128,
		    numFrames = 40,
		    textureName = "Assets/GhostTileLoopAtlas.png",
        } )
	    self.ghostLoopAnim = director:createAnimation( {
	        start = 1,
	        count = 40,
	        atlas = atlas,
	        delay = 0.066
        } )
		
    elseif preload_phase == 12 then
	    self.farmerlratlas = director:createAtlas( {
		    width = 102,
		    height = 102,
		    numFrames = 50,
		    textureName = "Assets/LRFarmerAnimAtlas.png",
        } )

    elseif preload_phase == 13 then
	    self.farmerdownatlas = director:createAtlas( {
		    width = 102,
		    height = 102,
		    numFrames = 50,
		    textureName = "Assets/DownFarmerAnimAtlas.png",
        } )

    elseif preload_phase == 14 then
    	atlas = director:createAtlas( {
		    width = 102,
		    height = 102,
		    numFrames = 50,
		    textureName = "Assets/FarmerLookingUpAnimAtlas.png",
        } )


	    local idleFrames = {}
	    for i = 0, 89 do
		    idleFrames[i + 1] = (i% 15) + 1
	    end
	    for i = 1, 35 do
		    idleFrames[i + 90] = i + 15
	    end

	    self.idleHorizontalFarmerAnim = director:createAnimation( {
	        atlas = self.farmerlratlas,
	        frames = idleFrames,
	        delay = 0.066
        })
	
	    self.idleBotFarmerAnim = director:createAnimation( {
	        atlas = self.farmerdownatlas,
	        frames = idleFrames,
	        delay = 0.066
        } )
	    self.idleTopFarmerAnim = director:createAnimation( {
    	    atlas = atlas,
	        frames = idleFrames,
	        delay = 0.066
        } )

        self.farmerlratlas = nil
        self.farmerdownatlas = nil

    elseif preload_phase == 15 then
	    self.numAnims = {}
        for i=1,3 do
	        atlas = director:createAtlas( {
		        width = 128,
		        height = 128,
		        numFrames = 20,
		        textureName = "Assets/Number"..i.."_Atlas.png",
            } )
	
	        self.numAnims[i] = director:createAnimation( {
	            start = 1,
	            count = 20,
	            atlas = atlas,
	            delay = 0.066
            } )
        end	
	
    elseif preload_phase == 16 then
	    atlas = director:createAtlas( {
	        width = 130,
	        height = 222,
	        numFrames = 20,
	        textureName = "Assets/HintMarkerAtlas.png",
        } )
	    self.hintStartAnim = director:createAnimation( {
	        start = 1,
	        count = 10,
	        atlas = atlas,
	        delay = 0.066
        } )
	    self.hintEndAnim = director:createAnimation( {
	        start = 11,
	        count = 10,
	        atlas = atlas,
	        delay = 0.066
        } )

        atlas = nil
        collectgarbage("collect")

        preload_phase = 999
        return true
	end

    preload_phase = preload_phase + 1

    return false
end

function grid:create(posX, posY, width, height, parentScene, levelId, worldInfo)

    local atlas

	self.gridWidth = GRID_WIDTH
	self.gridHeight = GRID_HEIGHT
	self.worldInfo = worldInfo
	self.levelId = levelId
	self.scene = parentScene
	self.height = height
	self.currentBlock = nil
	self.isEditing = false
	
	self.blocks = {}
	self.lastBlock = nil
	self.currentBlock = nil
	self.gridLocked = true

	self.rewindPathParameters = self:initializeRewindPath()
   	self.blockNumIndex = 0
	self.numMoves = 0
	
	atlas = director:createAtlas( {
		width = 128,
		height = 176,
		numFrames = 5,
		textureName = worldInfo.blockedTileIcon,
    } )
	self.treeAnim = director:createAnimation( {
		start = 1,
		count = 5,
		atlas = atlas
	} )

	atlas = director:createAtlas( {
	    width = TILE_SIZE,
	    height = TILE_SIZE,
	    numFrames = 48,
	    textureName = worldInfo.tileSetRevealed,
    } )
	self.lineAnim = director:createAnimation( {
	    start = 1,
	    count = 48,
	    atlas = atlas
    } )

	atlas = director:createAtlas( self.worldInfo.gridEdgeTL)
	self.gridCornerAnim = director:createAnimation( atlas)

	atlas = director:createAtlas( self.worldInfo.gridEdgeL)
	self.gridEdgeAnim = director:createAnimation( atlas)

	atlas = director:createAtlas( {
		width = 256,
		height = 256,
		numFrames = 30,
		textureName = worldInfo.dustTrail,
	} )
	self.dustTrailAnim = {}
	for i=1, 2 do
		self.dustTrailAnim[i] = director:createAnimation( {
			start = 1 + (i-1)*15,
			count = 15,
			atlas = atlas,
			delay = 0.066
		} )
	end

	self.levelData = nil
	if(levelId ~= nil) then
		self.levelData = self:loadLevel(worldInfo.levels[levelId].name)
	end
	
	self.gridPixelSize = 640 * UI_SCALE
	self.node = director:createNode()
    self.node.isTouchable = false
	self.node.zOrder = 2
	self.node.y =  (director.displayHeight - self.gridPixelSize) * (1/3)
	self.scene:addChild(self.node)
	self.completionNode = director:createNode()
	self.completionNode.isTouchable = false
	self.completionNode.zOrder = 50
	self.completionNode.y = self.node.y
	self.scene:addChild(self.completionNode)
	
	self.darkenGridBG = director:createRectangle( {
    x=director.displayWidth/2, y=director.displayHeight/2 - self.completionNode.y, w=director.displayWidth, h=director.displayHeight,
    color={0, 0, 0},
	xAnchor = 0.5, yAnchor = 0.5,
    strokeColor={0, 0, 0},
	strokeAlpha = 0,	
	alpha = 0, zOrder = 0
    } )
	self.completionNode:addChild(self.darkenGridBG)

	self.flashBG = director:createRectangle( {
        x=director.displayWidth/2, y=director.displayHeight/2 - self.completionNode.y, w=director.displayWidth, h=director.displayHeight,
        color={255, 255, 255},
	    xAnchor = 0.5, yAnchor = 0.5,
        strokeColor={255, 255, 255},
	    strokeAlpha = 0,	
	    alpha = 0, zOrder = 5,
	    isVisible = false
    } )
	self.completionNode:addChild(self.flashBG)
	
	self.bg = gameBG.create( self.scene, worldInfo.gameTopBG, worldInfo.gameBotBG, worldInfo.gameLeftBG, worldInfo.gameRightBG, worldInfo.gameOverlay, worldInfo.propsFile, worldInfo.cloudColor)
	
	self.cursor = director:createSprite( {
        source = self.cursorAnim, 
	    xAnchor = 0.5,
	    yAnchor = 0.5,
	    zOrder = 41,
	    isVisible = false,
    })
	self.cursor:play({startFrame=1, loopCount=0})
	self.cursor.isTouchable = false
	self.scene:addChild(self.cursor)

	self.cursorHalfTile = director:createSprite( {
        source = self.lineAnim,
        xAnchor = 0.5,
        yAnchor = 0.5,
        zOrder = 1,
        isVisible = false
    })
	self.cursorHalfTile:setFrame(CURSOR_HALF_LINE_SPRITE_ID.U)
	self.cursorHalfTile.isTouchable = false
	self.node:addChild(self.cursorHalfTile)
	
	self.redEffect = director:createSprite( {
        source = "Assets/red.png",
        xAnchor = 0.5,
        yAnchor = 0.5,
        zOrder = 5,
        isVisible = false
    })
	self.redEffect.timer = system:addTimer( self, 0.25, 0, 0)
	self.redEffect.timer:pause()
	self.redEffect.isTouchable = false
	self.node:addChild(self.redEffect)

	self:generateGrid(true)
	self:createGhostEffect()

	system:addEventListener( { "doubleTap", "update" }, self)

    collectgarbage("collect")
end

function grid:loadLevel(levelName)
	self.levelName = levelName
	local levelData = {};
	local file = io.open("Levels/"..levelName, "r")
	local line = file:read()
	local row = {}
	local pattern = string.format("([^%s]+)", ",")
	local fieldPattern = string.format("([^%s]+)", ":")
    line:gsub(pattern, function(c) row[#row+1] = c end)
	
	self.gridWidth = #row
	self.gridHeight = self.gridWidth
	local index = 1
	while true do
		for i = 1, self.gridWidth do
			fields = {}
			row[i]:gsub(fieldPattern, function(c) fields[#fields+1] = c end)
			levelData[index] = {}
			levelData[index].type = tonumber(fields[1])
			if #fields > 1 then
				levelData[index].area = tonumber(fields[2])
			    if #fields > 2 then
				    levelData[index].param = string.gsub(fields[3], "\n", "")
				    levelData[index].param = string.gsub(levelData[index].param, "\r", "")
			    end
			else
				levelData[index].area = 1
			end
			index = index + 1
		end
		line = file:read()
		if line == nil then
			break
		end
		row = {}
		line:gsub(pattern, function(c) row[#row+1] = c end)
	end
	
	file:close()
	return levelData
end

--[[
Editor depreciated
function grid:saveCurrent()
	local file = io.open(self.levelName, "w")
	local offset = (GRID_WIDTH - self.gridWidth) / 2
	
	for py = 0, GRID_HEIGHT - 1 do
		local line = ""
		for px = 0, GRID_WIDTH - 1 do
			local arrayIndex = px + py * GRID_WIDTH
			if not (px < offset or px >  GRID_WIDTH - offset - 1 or py < offset or py > GRID_HEIGHT - offset - 1) then
				local sprite = self.blocks[arrayIndex]
				local value = sprite.blockType..":"..sprite.area
				if sprite.blockTypeParam ~= nil then
					value = value..":"..sprite.blockTypeParam
				end
				if px == GRID_WIDTH - offset - 1 then
					line = line..value
				else
					line = line..value..","
				end
			end
		end
		if not(py < offset or py > GRID_HEIGHT - offset - 1) then
			if py < GRID_HEIGHT - 1 then
				file:write(line.."\n")
			else
				file:write(line)
			end
		end
	end
	file:close()
end
]]

function grid:generateGrid(createNode)
	self.numStars = 0
	self.oldStars = self.worldInfo.levels[self.levelId].numStars	-- how many stars did we already have on this level?
	self.completionNode.isVisible = false
	self.darkenGridBG.alpha = 0
	self.darkenGridBG.strokeAlpha = 0
	local gridPixelSize = 640 * UI_SCALE
	self.usedTileTypes = {}
	self.hasStartArea = false
	self.hasEndArea = false
	local scale = gridPixelSize / (self.gridWidth * TILE_SIZE)
	self.cursor.xScale = scale
	self.cursor.yScale = scale
	self.tileSize = TILE_SIZE * scale;
--	local offset = GRID_WIDTH - self.gridWidth
    local gridRight = self.gridWidth - 1
    local gridBottom = self.gridHeight - 1

	self.revealedBlocks = {}
	self.blocksToReveal = {}
	self.farmerBlocks = {}
	director.addNodesToScene = false
	self.redEffect.xScale = scale
	self.redEffect.yScale = scale
	self.cursorHalfTile.xScale = scale
	self.cursorHalfTile.yScale = scale

	local gridPixelOffset = (director.displayWidth - gridPixelSize) / 2
	local halfTileSize = self.tileSize / 2
	self.darkenGridBG.x = gridPixelOffset + self.gridPixelSize / 2
    local gridTop = (self.gridHeight - 1) * self.tileSize
	for py = 0, GRID_HEIGHT - 1 do
		for px = 0, GRID_WIDTH - 1 do
			local blockNode = nil
			local sprite = nil
			local arrayIndex = px + py * GRID_WIDTH
			if createNode then
				
				blockNode = director:createNode()
				blockNode.isTouchable = false
				blockNode.x = px * self.tileSize + halfTileSize + gridPixelOffset
				blockNode.y = gridTop - py * self.tileSize + halfTileSize
				blockNode.xScale = scale
				blockNode.yScale = scale
				sprite = director:createSprite( {
                    xAnchor=0.5,
                    yAnchor=0.5,
                    source=self.lineAnim
                } )
				sprite:setFrame(1)
				sprite.name = px + py * GRID_WIDTH				
				local line = director:createSprite( {
                    xAnchor=0.5,
                    yAnchor=0.5,
                    source=self.lineAnim,
                    isVisible = false,
                    zOrder = 2
                } )
				line:setFrame(1)
				sprite.line = line
				
				blockNode:addChild(line)
				blockNode:addChild(sprite)
				
				self.blocks[arrayIndex] = sprite
				self.node:addChild(blockNode)
				
				sprite.completionSprite = director:createSprite({xScale = scale, isVisible = false, yScale=scale, x = blockNode.x, y = blockNode.y, xAnchor=0.5, yAnchor=0.5, source=self.completionAnim })
				
				sprite.completionSprite.alpha = 0.0
				self.completionNode:addChild(sprite.completionSprite)
--[[
 Editor depreciated
			else
				sprite.completionSprite.isVisible = false
				sprite = self.blocks[arrayIndex]
				blockNode = sprite.parent
				blockNode.zOrder = 0
				blockNode.xScale = scale
				blockNode.yScale = scale
				
				blockNode.x = px * self.tileSize + self.tileSize/ 2 + gridPixelOffset
				blockNode.y = ((self.gridWidth - 1) * self.tileSize) - py * self.tileSize + self.tileSize / 2 + gridPixelOffset
				sprite.completionSprite.x = blockNode.x
				sprite.completionSprite.y = blockNode.y
				sprite.completionSprite.xScale = scale
				sprite.completionSprite.yScale = scale
				sprite.completionSprite.isVisible = true
				
				if(self.hintSpriteStart) then
					self.hintSpriteStart.xScale = scale
					self.hintSpriteStart.yScale = scale
					self.hintSpriteStart.x 		= blockNode.x
					self.hintSpriteStart.y 		= blockNode.y					
					self.hintSpriteStart.xGrid 	= px
					self.hintSpriteStart.yGrid 	= py
				end
				
				if(self.hintSpriteEnd) then
					self.hintSpriteEnd.xScale 	= scale
					self.hintSpriteEnd.yScale 	= scale
					self.hintSpriteEnd.x 		= blockNode.x
					self.hintSpriteEnd.y 		= blockNode.y
					self.hintSpriteEnd.xGrid	= px					
					self.hintSpriteEnd.yGrid	= py
				end
]]
			end

            -- Add an edge sprite to the block if it needs one
            local edgeSprite = nil
			if px == 0 then
				if py == 0 then
                    --Top left
					edgeSprite = director:createSprite( {zOrder = 3, xAnchor=0.5, yAnchor=0.5, source=self.gridCornerAnim } )
				elseif py == gridBottom then
                    -- Bottom left
					edgeSprite = director:createSprite( {zOrder = 3, xAnchor=0.5, yAnchor=0.5, yFlip = true, source=self.gridCornerAnim } )
				else
					-- Left
					edgeSprite = director:createSprite( {zOrder = 3, xAnchor=0.5, yAnchor=0.5, source=self.gridEdgeAnim } )
				end
			elseif px == gridRight then
				if py == 0 then
					-- Top right
					edgeSprite = director:createSprite( {zOrder = 3, xAnchor=0.5, yAnchor=0.5, xFlip = true, source=self.gridCornerAnim } )
				elseif py == gridBottom then
					-- Bottom right
					edgeSprite = director:createSprite( {zOrder = 3, xAnchor=0.5, yAnchor=0.5, xFlip = true, yFlip = true, source=self.gridCornerAnim } )
				else
					-- Right
					edgeSprite = director:createSprite( {zOrder = 3, xAnchor=0.5, yAnchor=0.5, xFlip = true, source=self.gridEdgeAnim} )
				end
			elseif py == 0 then
				--TOP
				edgeSprite = director:createSprite( {zOrder = 3, xAnchor=0.5, yAnchor=0.5, rotation = 90, source=self.gridEdgeAnim } )
			elseif py == gridBottom then
				--BOTTOM
				edgeSprite = director:createSprite( {zOrder = 3, xAnchor=0.5, yAnchor=0.5, rotation = -90, source=self.gridEdgeAnim} )
			end
			
            if edgeSprite ~= nil then
            	blockNode:addChild(edgeSprite)
            end
				
--[[
Editor depreciated in thoery
            sprite.arrow = nil
            sprite.ghost = nil
            sprite.farmerBlocks = nil
            sprite.wall = nil
            sprite.num = nil
            sprite.farmer = nil
            sprite.pipe = nil
            sprite.farmer = nil
]]
			sprite.isHidden = false
			sprite.isGhostHidden = 0
			
			if px > gridRight or py > gridBottom then
				blockNode.isVisible = false
				sprite.blockTypeParam = nil
				sprite.blockType = 0
				sprite.area = 1
				sprite.completionSprite.isVisible = false
			else
				sprite.area = 1
				if self.levelData ~= nil then
					local levelDataIndex = (px + 1) + py * self.gridWidth
					sprite.blockType = self.levelData[levelDataIndex].type
					if sprite.blockType ~= BLOCK_TYPE_ARROW and sprite.blockType ~= BLOCK_TYPE_FARMER and sprite.blockType ~= BLOCK_TYPE_PIPE then
						sprite.blockTypeParam = tonumber(self.levelData[levelDataIndex].param)
					else
						sprite.blockTypeParam = self.levelData[levelDataIndex].param
					end
					sprite.area = self.levelData[levelDataIndex].area
					--Hint sprite placing test
					if (sprite.area == HINT_SPRITE_START_TYPE or sprite.area == HINT_SPRITE_START_END_TYPE) then
						self.hintSpriteStart = director:createSprite( {
							x=0, y=0,
							zOrder=10,
							xAnchor=0.5,
							yAnchor=0.2,
							xScale=0.8,
							yScale=0.8,
							source=self.hintStartAnim
							} )
						blockNode:addChild(self.hintSpriteStart)
						self.hintSpriteStart.isVisible = false
						self.hintSpriteStart.xGrid = px
						self.hintSpriteStart.yGrid = py
					end
					if (sprite.area == HINT_SPRITE_END_TYPE or sprite.area == HINT_SPRITE_START_END_TYPE) then
						self.hintSpriteEnd = director:createSprite( {
							x=0, y=0,
							xAnchor=0.5,
							yAnchor=0.2,
							zOrder=10,
							xScale=0.8,
							yScale=0.8,
							source=self.hintEndAnim
							} )
						blockNode:addChild(self.hintSpriteEnd)
						self.hintSpriteEnd.isVisible = false
						self.hintSpriteEnd.xGrid = px
						self.hintSpriteEnd.yGrid = py
					end
				else
					sprite.blockType = BLOCK_TYPE_DEFAULT
					sprite.area = 1
				end
				blockNode.isVisible = true	

				self:setBlockType(sprite,sprite.blockType, false)
				self.usedTileTypes[sprite.blockType] = true
			end
			
			if sprite.blockType ~= 0 and sprite.blockType ~= BLOCK_TYPE_WALL then
				table.insert(self.blocksToReveal, sprite)
			end
			
			if sprite.blockType == BLOCK_TYPE_FARMER then
				table.insert(self.farmerBlocks, sprite)
			end
			sprite.xGrid = px
			sprite.yGrid = py
		end --x
	end	--y

	self:initFarmerBlocks();
	self:resetRewindCounter()
	director.addNodesToScene = true

	system:addEventListener("touch", self)
end

function grid:createGhostEffect()
	self.ghost = director:createNode()
	self.ghost.isVisible = false
	self.ghost.isTouchable = false
	self.ghost.zOrder = 5
	self.scene:addChild(self.ghost)
	self.ghost.animChild = {}

	for i = 1, 4 do
		self.ghost.animChild[i]	= director:createSprite({xAnchor = 0.75, yAnchor=0.5, rotation=GHOST_FX_ROTATION[i], source=self.ghostStartAnim})
		self.ghost.animChild[i].looper = director:createSprite({xAnchor = 0.5, yAnchor=0.5, x=self.tileSize * GHOST_FX_ANCHOR_X[i], y=self.tileSize * GHOST_FX_ANCHOR_Y[i], rotation=0, zOrder = 1, source=self.ghostLoopAnim})
		self.ghost.animChild[i].direction = i
		self.ghost:addChild(self.ghost.animChild[i])
		self.ghost:addChild(self.ghost.animChild[i].looper)
	end
end

-- Fixes bug that causes each ghost effect to only appear once in the level
-- Similar to previous issue with cows not being restored properly on rewinding over them
function grid:restoreGhostEffect()

	for i = 1, 4 do
	
		if (self.ghost.animChild[i].parent ~= self.ghost) then
			self.ghost:addChild(self.ghost.animChild[i])
		end
	
		if (self.ghost.animChild[i].looper.parent ~= self.ghost) then
			self.ghost:addChild(self.ghost.animChild[i].looper)
		end
	end
end

function grid:destroyGhostEffect()
    self.ghost:removeFromParent()
	self.ghost = nil
end

--[[
editor depreciated
function grid:resize(gridSize)
	self.levelData = nil
	self.gridWidth = gridSize
	self.gridHeight = gridSize
	self:generateGrid(false)
end]]

function grid:destroy()
	self.currentBlock = nil
	self.lastTouchedBlock = nil
	
    self.treeAnim = nil
	self.lineAnim = nil
  	self.gridCornerAnim = nil
	self.gridEdgeAnim = nil

	self.completionNode:removeFromParent()
    self.completionNode = nil
    self.darkenGridBG = nil
    self.flashBG = nil
    self.bg:clear()
    self.bg = nil
	self.cursor:removeFromParent()
	self.cursor = nil
    self.cursorHalfTile = nil
    self.redEffect = nil

    self:destroyGhostEffect()

	self.hintSpriteStart = nil
	self.hintSpriteEnd = nil


   	self.rewindPathParameters = nil

    self.completionNode = nil

	self.revealedBlocks = nil
	self.blocksToReveal = nil
	self.farmerBlocks = nil

    self.blocks = nil

    self.node:removeFromParent()
    self.node = nil
	    	
	system:removeEventListener( { "doubleTap", "update", "touch" }, self)
end

function grid:initializeRewindPath()
	local valuesTable = {}
	valuesTable.delay = 0.350 -- seconds
	valuesTable.revealedElementsToRemove = -1
	valuesTable.rewindTimer = 0
	valuesTable.perfTimer = 0
	return valuesTable
end

function grid:update(event)
	--only on the first frame it will be a meaningless number
	if(self.rewindPathParameters ~= nil) then
		self.rewindPathParameters.rewindTimer = self.rewindPathParameters.rewindTimer + (system.gameTime - self.rewindPathParameters.perfTimer)
		self.rewindPathParameters.perfTimer = system.gameTime 
		while ((self.rewindPathParameters.rewindTimer >= grid.REWIND_TILE_DELAY) and (self.rewindPathParameters.revealedElementsToRemove > 0)) do
			self:rewindLastPathElement()
			self.rewindPathParameters.rewindTimer = self.rewindPathParameters.rewindTimer - grid.REWIND_TILE_DELAY
		end
	end
end

function grid:rewindLastPathElement()
	local count = table.getn(self.revealedBlocks)
	self:processBlockHitTouched(self.revealedBlocks[count])--this removes the last revealed block
	self.rewindPathParameters.revealedElementsToRemove = self.rewindPathParameters.revealedElementsToRemove - 1		
end

--[[
/**
Queries the grid to retrieve a tile using a point in screen coordinates (i.e those returned by a touch event).
@param x the x screen coordinate of the point in which to query clamped to allowed grid values.
@param y the y screen coordinate of the point in which to query clamped to to allowed grid values.
@return a grid tile or nil if not found.
*/
--]]
function grid:retrieveTileByClampedToGridScreenPosition(x, y)
	local tile = nil
	if (self.blocks ~= nil) then
		if (self.blocks[0] ~= nil) then
			local gridMinX = self.blocks[0].parent.x - self.tileSize * 0.5 + self.node.x--+ (director.displayWidth - (640 * UI_SCALE)) / 2--the top left corner of the grid
			local gridMaxY = self.blocks[0].parent.y + self.tileSize * 0.5 + self.node.y--+ director.displayHeight * 0.5 --the top left corner of the grid
			local gridMaxX = gridMinX + self.gridWidth * self.tileSize
			local gridMinY = gridMaxY - self.gridHeight * self.tileSize
			--dbg.print("gridMinY " ..gridMinY .." gridMaxY " ..gridMaxY)
			--dbg.print("point " ..x ..", " ..y .." ")
			x = math.max(gridMinX, x)
			x = math.min(gridMaxX, x)
			
			y = math.max(gridMinY, y)
			y = math.min(gridMaxY, y)
			--dbg.print("clamped point " ..x ..", " ..y .." ")
			tile = self:retrieveTileByScreenPosition(x, y)
		end
	end
	return tile
end

--[[
/**
Queries the grid to retrieve a tile using a point in screen coordinates (i.e those returned by a touch event).
@param x the x screen coordinate of the point in which to query.
@param y the y screen coordinate of the point in which to query.
@return a grid tile or nil if not found.
*/
--]]
function grid:retrieveTileByScreenPosition(x, y)
	local tile = nil
	if (self.blocks ~= nil) then
		if (self.blocks[0] ~= nil) then
			local gridX = self.blocks[0].parent.x - self.tileSize * 0.5 + self.node.x--+ (director.displayWidth - (640 * UI_SCALE)) / 2--the top left corner of the grid
			local gridY = self.blocks[0].parent.y + self.tileSize * 0.5 + self.node.y--+ director.displayHeight * 0.5 --the top left corner of the grid
			
			--dbg.print("queryTileByScreenPosition x " ..x .." y " ..y .. " grid x " ..gridX .." y " ..gridY .." self.gridHeight * self.tileSize " ..(self.gridHeight * self.tileSize) .. " self.gridWidth * self.tileSize " ..self.gridWidth * self.tileSize)
			if(x < gridX or y > gridY or x > (gridX + self.gridWidth * self.tileSize) or y < (gridY - self.gridHeight * self.tileSize))then
				--do nothing double queryTileByScreenPosition out of grid
				--dbg.print("queryTileByScreenPosition out of grid " )
			else
				local xInd = math.floor((x - gridX) / self.tileSize)
				local yInd = math.floor((gridY - y) / self.tileSize)
				xInd = math.max(0, xInd)--TODO try and find a better way
				xInd = math.min(self.gridWidth - 1, xInd)
				
				yInd = math.max(0, yInd)--TODO try and find a better way
				yInd = math.min(self.gridHeight - 1, yInd)
				--dbg.print("double queryTileByScreenPosition in grid xind " ..xInd .. "yind " ..yInd)
				--dbg.assert(xInd < self.gridWidth and yInd < self.gridHeight, "Error out of range double tap grid indices. x = " ..xInd.." y " ..yInd)

				--local target = self.blocks[xInd + yInd * self.gridWidth]
				--the index is encoded with the maximum possible grid width!!, see function generateGrid 
				tile = self.blocks[xInd + yInd * GRID_WIDTH]--see generateGrid 
			end
		end
	end
	return tile
end

function grid:doubleTap(event)
	--dbg.print("grid:doubleTap")
	if self.gridLocked == true then
		--do nothing
	else
		local target = self:retrieveTileByScreenPosition(event.x, event.y)
		if (target ~= nil) then
			self.rewindPathParameters.revealedElementsToRemove = self:searchRevealdElementIndex(target)
			if (self.rewindPathParameters.revealedElementsToRemove > 0) then
				--dbg.print("double tap on a tile in path")
				self:rewindLastPathElement()
				--self.rewindPathParameters.rewindTimer = system.gameTime
				self.rewindPathParameters.rewindTimer = 0
			else
				--dbg.print("double tap on a tile NOT in path")
			end
		end
	end
end

--[[
/**
Searches the passed block among the revealed elements, if it is found it returns its index in the array, -1 otherwise.
@param block the block to search for
@return the index in the revealed array, -1 if not found
*/
--]]
function grid:searchRevealdElementIndex(block)
	local i = 1
	local elementsToRemoveFromLastCount = -1
	local foundIndex = -1
	local count = table.getn(self.revealedBlocks) --TODO check if this actually works
	--dbg.print("searchRevealdElementIndex count " ..count)
	--dbg.printTable(self.revealedBlocks)
	while ((i <= count) and (foundIndex < 0)) do
		--dbg.print(""..self.revealedBlocks[i])
		if (self.revealedBlocks[i].xGrid == nil or self.revealedBlocks[i].yGrid == nil or block.xGrid == nil or block.yGrid == nil) then
			dbg.print("ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR ERROR")
		else
			if (self.revealedBlocks[i].xGrid == block.xGrid and self.revealedBlocks[i].yGrid == block.yGrid) then
				foundIndex = i
				--dbg.print("found it at index " ..i)
			else
				i = i +1
			end
		end
	end
	if (foundIndex > 0 ) then
		elementsToRemoveFromLastCount = count - foundIndex + 1
	end
	return elementsToRemoveFromLastCount
end

function grid:touch(event)

	if self.gridLocked == true then
		return false
	end
	
	--local target = self:retrieveTileByClampedToGridScreenPosition(event.x, event.y)
	local target = self:retrieveTileByScreenPosition(event.x, event.y)
	if (target ~= nil) then
		if self.isEditing == false then
			if self.currentBlock ~= nil and self.currentBlock == target then
				self.lastTouchedBlock = target				
				return true

			end
			if event.phase == "began" then
				self:processBlockHitTouched(target)

			elseif event.phase == "ended" then
				self:endRedEffect()

			elseif event.phase == "moved" then
				if self.currentBlock == nil then
					--if target is not adjacent to the last one perform the "autocompletion"
					self:processBlockHitTouched(target)
					
				elseif self.lastTouchedBlock == nil or self.lastTouchedBlock ~= target then
					self:processBlockHitUpdate(target)
					self.lastTouchedBlock = target
				end
			end
		elseif event.phase == "began" then
			system:sendEvent("onBlockTouched", { block = target})
		end
	else
		--dbg.print("grid:touch(event) (" ..event.x ..", " ..event.y ..") did NOT find any tile")
		--process out of grid touch event only on release

		if self.isEditing == false then	
			local target = self:retrieveTileByClampedToGridScreenPosition(event.x, event.y)
			if self.currentBlock ~= nil and self.currentBlock == target then
				self.lastTouchedBlock = target

				return true
			end
			if event.phase == "began" then
				self:processBlockHitTouched(target)
				
			elseif event.phase == "moved" then
				if self.currentBlock == nil then
					--if target is not adjacent to the last one perform the "autocompletion"
					self:processBlockHitTouched(target)
					
				elseif self.lastTouchedBlock == nil or self.lastTouchedBlock ~= target then
					self:processBlockHitUpdate(target)
					self.lastTouchedBlock = target
				end
			end
		end
	end

	return true
end

function grid:processBlockHitTouched(block)
	--[[
	--dbg.print("--------------- grid:processBlockHitTouched --------------")
	if self.currentBlock == nil or #self.revealedBlocks == 0 then
		self.blockNumIndex = 0
		
		if block.farmersCount ~= nil then
			self:playRedEffect(block.parent.x,block.parent.y)
			return
		elseif block.blockType == BLOCK_TYPE_WALL  then
			self:playRedEffect(block.parent.x,block.parent.y)
			return
		elseif block.blockType == BLOCK_TYPE_NUM and block.blockTypeParam ~= self.blockNumIndex + 1 then
			self:playRedEffect(block.parent.x,block.parent.y)
			return
		elseif self.hasStartArea and block.area ~= 2 then
			return
		end
		
		if self.currentBlock ~= nil and #self.revealedBlocks == 0 and self.currentBlock.blockType == BLOCK_TYPE_FARMER then
			self:processTileFarmer(self.currentBlock, false)
		end
		
		self.currentBlock = block
		
		if block.blockType == BLOCK_TYPE_NUM then
			self.blockNumIndex = self.blockNumIndex + 1
		elseif block.blockType == BLOCK_TYPE_FARMER then
			self:processTileFarmer(block, true)
		end

		self:updateCursorPosition(block, false)
	else]]
		self:processBlockHitUpdate(block)
	--end
end

function grid:isBlockFitDirection(from, to, dir)
	fromX = (from.parent.x - self.tileSize / 2) / self.tileSize
	fromY = (from.parent.y - self.tileSize / 2) / self.tileSize
	
	toX = (to.parent.x - self.tileSize / 2) / self.tileSize
	toY = (to.parent.y - self.tileSize / 2) / self.tileSize
	
	if dir == "U" and fromY < toY then
		return true
	elseif dir == "D" and fromY > toY then
		return true
	elseif dir == "L" and fromX > toX then
		return true
	elseif dir == "R" and fromX < toX then
		return true
	end
	return false
end

function grid:isBlockFitPipe(from, to, dir)
	fromX = (from.parent.x - self.tileSize / 2) / self.tileSize
	fromY = (from.parent.y - self.tileSize / 2) / self.tileSize
	
	toX = (to.parent.x - self.tileSize / 2) / self.tileSize
	toY = (to.parent.y - self.tileSize / 2) / self.tileSize
	
	if dir == "H" and fromY == toY then
		return true
	elseif  dir == "V" and fromX == toX then
		return true
	end
	return false
end

function grid:getDir(from, to)
	fromX = (from.parent.x - self.tileSize / 2) / self.tileSize
	fromY = (from.parent.y - self.tileSize / 2) / self.tileSize
	
	toX = (to.parent.x - self.tileSize / 2) / self.tileSize
	toY = (to.parent.y - self.tileSize / 2) / self.tileSize
	if fromY < toY then
		return "U"
	elseif fromY > toY then
		return "D"
	elseif fromX > toX then
		return "L"
	end
	return "R"
end

function grid:isBlockAdjacentToCurrent(block)
	local lengthSq = (self.currentBlock.xGrid - block.xGrid) * (self.currentBlock.xGrid - block.xGrid) + (self.currentBlock.yGrid - block.yGrid) * (self.currentBlock.yGrid - block.yGrid)
	return lengthSq <= 1.1
end

function grid:isBlockReachable(block)
	if block.blockType == BLOCK_TYPE_WALL and self.currentBlock.blockType ~= BLOCK_TYPE_GHOST then
		return false
	elseif block.blockType == BLOCK_TYPE_NUM and block.blockTypeParam ~= self.blockNumIndex + 1 then
		return false
	elseif self.currentBlock.blockType == BLOCK_TYPE_ARROW and self:isBlockFitDirection(self.currentBlock, block, self.currentBlock.blockTypeParam:sub(-1)) == false then
		return false
	elseif block.blockType == BLOCK_TYPE_ARROW and self:isBlockFitDirection(self.currentBlock, block , block.blockTypeParam:sub(1,1)) == false then
		return false
	elseif block.blockType == BLOCK_TYPE_PIPE and self:isBlockFitPipe(self.currentBlock, block , block.blockTypeParam) == false then
		return false
	elseif  self.currentBlock.blockType == BLOCK_TYPE_PIPE and self:isBlockFitPipe(self.currentBlock, block ,  self.currentBlock.blockTypeParam) == false then
		return false
	elseif block.farmersCount ~= nil and block.farmersCount > 0 then
		return false
	elseif block.isHidden then
		return false
	end
	return true
end
function grid:hasBlockUnrevealedSiblingSquares(block)
	local leftBlockIndex = (block.xGrid - 1 ) + block.yGrid * GRID_WIDTH
	local rightBlockIndex = (block.xGrid + 1 ) + block.yGrid * GRID_WIDTH
	local topBlockIndex = block.xGrid + (block.yGrid - 1) * GRID_WIDTH
	local botBlockIndex = block.xGrid + (block.yGrid + 1) * GRID_WIDTH
	
	if block.xGrid > 0 and self:isBlockReachable(self.blocks[leftBlockIndex])then
		return true
	end
	
	if block.xGrid < self.gridWidth - 1 and self:isBlockReachable(self.blocks[rightBlockIndex]) then
		return true
	end
	
	if block.yGrid > 0 and self:isBlockReachable(self.blocks[topBlockIndex]) then
		return true
	end
	
	if block.yGrid < self.gridWidth - 1 and self:isBlockReachable(self.blocks[botBlockIndex]) then
		return true
	end
	
	return false
	
end

function grid:unhideBlock(block)

	-- Unhide the base block
	block.isVisible = true
	
	-- Manually unhide any children
	if block.blockType == BLOCK_TYPE_NUM then
		block.num.isVisible = true
	elseif block.blockType == BLOCK_TYPE_FARMER then
		block.farmer.isVisible = true
	elseif block.blockType == BLOCK_TYPE_FLIP then
		if block.numTransit == 2 then
			--dbg.print("Should be dropping cow now")
			if(block.flip.parent ~= self.node) then
				self.node:addChild(block.flip)
			end
			block.flip.isVisible = true
			block.flip.isTouchable = false
			block.flip:setAnimation (self.flipIdleAnim)
			block.flip:removeEventListener("anim", self)
			block.flip:play({startFrame=1, loopCount=0})
			block.flip.raisesAnimEvents = false
		end
	elseif block.blockType == BLOCK_TYPE_WALL then
		if(block.wall ~= nil)then
			block.wall.alpha = 1.0
			block.wall.isVisible = true
		else
			--ERROR!!
		end
	else
	
	end
end

function grid:logTileSpriteZorders(blockTileSprite)
	if(blockTileSprite.isVisible) then
		dbg.print("sprite " ..tostring(blockTileSprite) .." zOrder = " ..blockTileSprite.zOrder)
		for k,v in pairs(blockTileSprite.children) do
			self:logTileSprite(v)
		end	
	end
end

function grid:processBlockHitUpdate(block)
	
	--dbg.print("grid:processBlockHitUpdate ->> " ..block.xGrid ..", " ..block.yGrid .." block type " ..self:getBlockTypeDescription(block.blockType))
	
	local numRevealedBlocks = table.getn(self.revealedBlocks)
	local firstMove = false

	-- Is this our first move?
	if self.currentBlock == nil then
		firstMove = true
		
	-- Are we resetting our start position?
	elseif numRevealedBlocks == 0 and not self:isBlockAdjacentToCurrent(block) then
		firstMove = true
	end

	if firstMove or self:isBlockAdjacentToCurrent(block) then

		local backward = block.isHidden or block.isGhostHidden > 0
		backward = backward  or (block.blockType == BLOCK_TYPE_FLIP and (block.nextBlock == self.currentBlock or block.numTransit == 0))
		
		if backward then
			--dbg.print("backward")
			if block == self.revealedBlocks[numRevealedBlocks] then
				block.isHidden = false
				block.isGhostHidden = block.isGhostHidden - 1
				
				if block.blockType == BLOCK_TYPE_FLIP then
					block.numTransit = block.numTransit + 1
					self:updatePath(block)
				elseif block.blockType == BLOCK_TYPE_GHOST then
					self:checkGhostAdjacentSquares(block)
				end
				
				
				
				if self.currentBlock.blockType == BLOCK_TYPE_NUM then
					self.blockNumIndex = self.blockNumIndex - 1
				elseif self.currentBlock.blockType == BLOCK_TYPE_GHOST then
					self:hideGhost()
				elseif self.currentBlock.blockType == BLOCK_TYPE_FARMER then
					self:processTileFarmer(self.currentBlock, false)
				end

				-- Make the block and any number / farmer / cow on it visible, as appropriate
				if (block.blockType ~= BLOCK_TYPE_WALL) then
					self:unhideBlock(block)
				end
				if numRevealedBlocks >= 2 and self.revealedBlocks[numRevealedBlocks -1].blockType == BLOCK_TYPE_FLIP then
					if self.revealedBlocks[numRevealedBlocks -1].numTransit == 1 then
						if(self.revealedBlocks[numRevealedBlocks -1].flip.parent ~= self.node) then
							self.node:addChild(self.revealedBlocks[numRevealedBlocks -1].flip)
						end
						self.revealedBlocks[numRevealedBlocks -1].flip.isVisible = true
						self.revealedBlocks[numRevealedBlocks -1].flip.isTouchable = false
						self.revealedBlocks[numRevealedBlocks -1].flip:setAnimation (self.flipStartRemoveLoopAnim)
						self.revealedBlocks[numRevealedBlocks -1].flip:play({startFrame=1, loopCount=0})
						self.revealedBlocks[numRevealedBlocks -1].flip:removeEventListener("anim", self)
                        self.revealedBlocks[numRevealedBlocks -1].flip.raisesAnimEvents = false
						--dbg.print("------ <<<<<<<<<<< Should be returning cow now -------- >>>>>>>>>>>>>> at tile " ..self.revealedBlocks[numRevealedBlocks -1].xGrid ..", " ..self.revealedBlocks[numRevealedBlocks -1].yGrid )

					end
				end
				
				-- For the cow tile, we want to leave the partial line there when undoing a crossroads piece
				if block.blockType ~= BLOCK_TYPE_FLIP or block.numTransit~=1 then
					block.line.isVisible = false
				end
				
				self.currentBlock = block
				
				self.numMoves = self.numMoves + 1
				system:sendEvent("onCursorPositionChanged", { numMoves = self.numMoves})
				
				self:incrementRewindCounter()
				
				local removedElement = table.remove(self.revealedBlocks)
				self:updateCursorPosition(block, false)
				
				audio:playSound("Assets/Audio/SFX_Reverse.raw")
				system:sendEvent("onBlockRewinded", {gameGrid = self})
				
				if table.getn(self.revealedBlocks) == 0 then
					self.cursorHalfTile.isVisible = false
				end
				
				if self.redEffect.isVisible == true then
					self.redEffect.isVisible = false
					self.redEffect.timer:pause()
				end
				if self.numStars > 0 then
					self.numStars = 0
					system:sendEvent("onNumStarsChanged", {numStars = self.numStars})
				end
				return

			-- If we're trying to cross our own path...
			elseif  block.isHidden then
				tutorialsManager:onBlockChanged(self, block)
				audio:playSound("Assets/Audio/SFX_BlockedTile.raw")
				self:playRedEffect(block.parent.x,block.parent.y)
				return
			end
		else
			--dbg.print("forward")	
		end

		if firstMove then
			self.blockNumIndex = 0
		else
			self:addDust(self.currentBlock)
		end

		if block.blockType == BLOCK_TYPE_WALL and (firstMove or self.currentBlock.blockType ~= BLOCK_TYPE_GHOST) then
			audio:playSound("Assets/Audio/SFX_BlockedTile.raw")
			self:playRedEffect(block.parent.x,block.parent.y)
			return
		elseif block.blockType == BLOCK_TYPE_NUM and block.blockTypeParam ~= self.blockNumIndex + 1 then
			audio:playSound("Assets/Audio/SFX_BlockedTile.raw")
			self:playRedEffect(block.parent.x,block.parent.y)
			return
		elseif block.farmersCount ~= nil and block.farmersCount > 0 then
			audio:playSound("Assets/Audio/SFX_BlockedTile.raw")
			self:playRedEffect(block.parent.x,block.parent.y)
			return

		-- Check for player starting on a farmer then trying to change their starting position to a square that farmer was blocking
		elseif numRevealedBlocks == 0 and self.currentBlock ~= nil and not self:isBlockAdjacentToCurrent(block) and self.currentBlock.blockType == BLOCK_TYPE_FARMER
			and (block == self.currentBlock.farmerBlocks[1] or block == self.currentBlock.farmerBlocks[2] or block == self.currentBlock.farmerBlocks[3]) then
			audio:playSound("Assets/Audio/SFX_BlockedTile.raw")
			self:playRedEffect(block.parent.x,block.parent.y)
			return

		elseif firstMove then
		
			-- Don't bother checking arrows and pipes on first move - can always start on them

		elseif self.currentBlock.blockType == BLOCK_TYPE_ARROW and self:isBlockFitDirection(self.currentBlock, block, self.currentBlock.blockTypeParam:sub(-1)) == false then
			audio:playSound("Assets/Audio/SFX_BlockedTile.raw")
			self:playRedEffect(block.parent.x,block.parent.y)
			return
			
		elseif block.blockType == BLOCK_TYPE_ARROW and self:isBlockFitDirection(self.currentBlock, block , block.blockTypeParam:sub(1,1)) == false then
			audio:playSound("Assets/Audio/SFX_BlockedTile.raw")
			self:playRedEffect(block.parent.x,block.parent.y)
			return
			
		elseif block.blockType == BLOCK_TYPE_PIPE and self:isBlockFitPipe(self.currentBlock, block , block.blockTypeParam) == false then
			audio:playSound("Assets/Audio/SFX_BlockedTile.raw")
			self:playRedEffect(block.parent.x,block.parent.y)
			return
			
		elseif self.currentBlock.blockType == BLOCK_TYPE_PIPE and self:isBlockFitPipe(self.currentBlock, block ,  self.currentBlock.blockTypeParam) == false then
			audio:playSound("Assets/Audio/SFX_BlockedTile.raw")
			self:playRedEffect(block.parent.x,block.parent.y)
			return
		end

		if block.blockType==BLOCK_TYPE_FARMER or block.blockType==BLOCK_TYPE_NUM then
			self:addRemovalEffect(block)
		end

		if block.blockType == BLOCK_TYPE_NUM then
			if self.blockNumIndex == 0 then
				audio:playSound("Assets/Audio/SFX_Number_1.raw")
			elseif self.blockNumIndex == 1 then
				audio:playSound("Assets/Audio/SFX_Number_2.raw")
			elseif self.blockNumIndex == 2 then
				audio:playSound("Assets/Audio/SFX_Number_3.raw")
			end
			self.blockNumIndex = self.blockNumIndex + 1
			
			-- If this was our first move, stop the person hanging around under the UFO
			if firstMove then
				block.num.isVisible = false
			end

		elseif block.blockType == BLOCK_TYPE_GHOST then
			self:checkGhostAdjacentSquares(block)

		elseif block.blockType == BLOCK_TYPE_FARMER then

			-- Increment the sample to use
			FarmerVoice = FarmerVoice + 1
			
			-- Loop round when we reach the last sound
			if FarmerVoice > 5 then
				FarmerVoice = 1
			end
			
			-- Play sound and remove firing range marker
			audio:playSound("Assets/Audio/SFX_Farmer_" .. FarmerVoice .. ".raw")
			self:processTileFarmer(block, true)
			
			-- If this was our first move, stop the person hanging around under the UFO
			if firstMove then
				block.farmer.isVisible = false
			end
		end

		-- Lift cow into air and play sound if we've moved off the cow (but NOT if changing our starting position)
		if (not firstMove) and self.currentBlock.blockType == BLOCK_TYPE_FLIP then
			self.currentBlock.numTransit = self.currentBlock.numTransit - 1
			if self.currentBlock.numTransit > 0 then
				--dbg.print("Lift cow into air and play sound ")
				audio:playSound("Assets/Audio/SFX_Cow.raw")
				self.currentBlock.flip:setAnimation(self.flipStartRemoveAnim)
				self.currentBlock.flip:play({startFrame=1, loopCount=1})
				self.currentBlock.flip:addEventListener("anim", self)
                self.currentBlock.flip.raisesAnimEvents = true
				self.currentBlock.nextBlock = block
			end
		end

		-- Remove ghost effect if we've moved off the tile (don't check if this was our first move!)
		if self.currentBlock ~= nil and self.currentBlock.blockType == BLOCK_TYPE_GHOST then
			self:hideGhost()
		end

		if numRevealedBlocks > 0 then
			if self.revealedBlocks[numRevealedBlocks].blockType == BLOCK_TYPE_FLIP then
				if self.revealedBlocks[numRevealedBlocks].numTransit > 0 then
					--dbg.print("cow vanish")
					audio:playSound("Assets/Audio/SFX_CowVanish.raw")
					self.revealedBlocks[numRevealedBlocks].flip:setAnimation(self.flipEndRemoveAnim)				
					self.revealedBlocks[numRevealedBlocks].flip:play({startFrame=1, loopCount=1})
					self.revealedBlocks[numRevealedBlocks].flip:removeEventListener("anim", self)
					self.revealedBlocks[numRevealedBlocks].flip:addEventListener("anim", self)
                    self.revealedBlocks[numRevealedBlocks].flip.raisesAnimEvents = true
				end
			end
		end
	
		if firstMove then
		
			-- Don't need to do this stuff on first move, it's for when you move off a tile
		
		elseif self.currentBlock.blockType ~= BLOCK_TYPE_WALL then
			if self.currentBlock.blockType ~= BLOCK_TYPE_FLIP or self.currentBlock.numTransit == 0 then
				self.currentBlock.isHidden = true
			end
		else
			-- JB: This handles drawing over trees, which you can do twice
			self.currentBlock.isGhostHidden = self.currentBlock.isGhostHidden + 1
		end
		
		-- If we've just changed start positions...
		if self.currentBlock ~= nil and numRevealedBlocks == 0 and not self:isBlockAdjacentToCurrent(block) then
		
			-- Remove ghost effect if old start position was a ghost tile
			if self.currentBlock.blockType == BLOCK_TYPE_GHOST then
				self:hideGhost()
			
			-- Return farmer's warning area if old start position was a farmer
			elseif self.currentBlock.blockType == BLOCK_TYPE_FARMER then
				self:processTileFarmer(self.currentBlock, false)
			end
			
			-- Bring back any farmer / number / cow on the old tile
			self:unhideBlock(self.currentBlock)
		end
		
		-- If this wasn't our first move, add the tile we just moved off to the list of revealed blocks
		if not firstMove then
			self.numMoves = self.numMoves + 1
			--system:sendEvent("onCursorPositionChanged", { numMoves = self.numMoves})
			--dbg.print("revealed tile " ..self.currentBlock.xGrid ..", " ..self.currentBlock.yGrid)
			table.insert(self.revealedBlocks, self.currentBlock)
		end

		self:resetRewindCounter()
		
		self.currentBlock = block
		
		-- On first move, don't update path (only kicks in once you move off a tile), and do different cursor position update
		if firstMove then
			self:updateCursorPosition(block, false)
		else
			self:updatePath(nil)
			self:updateCursorPosition(block, true)
		end

		
		self.levelCompleted = false
		local numBlockAchieved = 0
		local numFlipBlocks = 0
		for i = 1, table.getn(self.revealedBlocks) do
			if self.revealedBlocks[i].blockType ~= BLOCK_TYPE_WALL then
				if self.revealedBlocks[i].blockType ~= BLOCK_TYPE_FLIP or self.revealedBlocks[i].numTransit == 0 then
					numBlockAchieved = numBlockAchieved + 1
					
					if  self.revealedBlocks[i].numTransit == 0 then
						numFlipBlocks = numFlipBlocks + 1
					end
				end
				
			end
		end
		
		numBlockAchieved = numBlockAchieved - (numFlipBlocks / 2)
		if self.redEffect.isVisible == true then
			self.redEffect.isVisible = false
			self.redEffect.timer:pause()
		end
		if numBlockAchieved >= table.getn(self.blocksToReveal) - 3 then
			if not self:hasBlockUnrevealedSiblingSquares(self.currentBlock) then
				self.numStars = 4 - (table.getn(self.blocksToReveal) - numBlockAchieved)
				system:sendEvent("onNumStarsChanged", {numStars = self.numStars})
			end
		end
		if numBlockAchieved == table.getn(self.blocksToReveal) - 1 then
			if self.hasEndArea == false or (self.currentBlock.area == 3) then
				
				if(not (block.blockType == BLOCK_TYPE_WALL or connectionRulesManager:isTileBlockaded(block))) then
					--dbg.print("completeLevel triggered by block update function")
					self:completeLevel()
				end
			end
		end
		
		if (not self.levelCompleted) then
			tutorialsManager:onBlockChanged(self, block)
		end

	elseif (block == self.currentBlock and self.redEffect.isVisible) then
		self.redEffect.isVisible = false
		self.redEffect.timer:pause()
	end

end

function grid:completeLevel()
	--dbg.print("--------->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> completelevel")
	self.levelCompleted = true
	self.currentBlock.isHidden = true
	table.insert(self.revealedBlocks, self.currentBlock)
	self:updatePath(nil)
	self.completionNode.isVisible = true
	self.completionNode.alpha = 1
	
	self:endRedEffect()
	
	for i = 1, #self.blocks do
		--dbg.print("i " ..i)
		self.blocks[i].completionSprite.alpha = 0.0
	end
	
	for i = 1, #self.revealedBlocks do
		--self.revealedBlocks[i].completionSprite.alpha = 0
		tween:to(self.revealedBlocks[i].completionSprite, {alpha = 0.75, time = 0.5, easing=ease.sineInOut, onComplete=grid.completionStep})
	end
	
    if self.cursor.beamLoopHandle ~= nil then
	    audio:stopSound( self.cursor.beamLoopHandle)
        self.cursor.beamLoopHandle = nil
	    audio:playSound("Assets/Audio/SFX_BeamLoopEnd.raw")
    end
	
	self.darkenGridBG.grid = self
	tween:to(self.darkenGridBG, {alpha = 0.75, strokeAlpha = 0.75, time = 3, onComplete = grid.displayFlash})
	self.cursor.isVisible = false
	self.cursorHalfTile.isVisible = false
	system:addTimer(self, 5, 1,0)
	system:sendEvent("onLevelComplete", {numMoves = self.numMoves, numStars = self.numStars})
end

function grid.completionStep(target)
	if target.alpha >= 1 then
		return
	end
	if target.alpha >= 0.85 then
		tween:to(target, {alpha = 0.3, time = 0.5, easing=ease.sineInOut, onComplete=grid.completionStep})
	elseif target.alpha >= 0.75 then
		tween:to(target, {alpha = 0.25, time = 0.5, easing=ease.sineInOut, onComplete=grid.completionStep})
	elseif target.alpha >= 0.3 then
		tween:to(target, {alpha = 1, time = 0.5, easing=ease.sineInOut, onComplete=grid.completionStep})
	elseif target.alpha >= 0.25 then
		tween:to(target, {alpha = 0.85, time = 0.5, easing=ease.sineInOut, onComplete=grid.completionStep})
	end
end

function grid.glow(target)
	if target.glow == true then
		if target.alpha >= 1 then
			tween:to(target, {alpha = 0.3, time = 0.66, onComplete=grid.glow})
		elseif target.alpha >= 0.3 then
			tween:to(target, {alpha =  1, time = 0.66,onComplete=grid.glow})
		end
	else
		target.alpha = 1
	end
end

function  grid.displayFlash(target)
	target.grid.flashBG.isVisible = true
	target.grid.flashBG.alpha = 0
	target.grid.flashBG.grid = target.grid
	target.grid.flashBG.alpha = 0.5
	tween:to(target.grid.flashBG, {alpha = 1, time = 0.35,ease=grid.expoIn, onComplete=grid.flashComplete})
end

function grid.flashComplete(target)
	for i = 1,#target.grid.revealedBlocks do
		target.grid.revealedBlocks[i].completionSprite.isVisible = false
	end
	tween:to(target.grid.darkenGridBG, {alpha = 0, time = 2})
	tween:to(target, {alpha = 0, time = 2})
	system:sendEvent("onLevelCompleteFlashFinished")
end

function grid:timer(timer)
	if self.redEffect.isVisible == true then
		if self.redEffect.alpha == 0 then
			self.redEffect.alpha = 1
		else
			self.redEffect.alpha = 0
		end
	else
		--TODO this "if" might fix the bug causing the "well done" animation to be triggered on the level start
		if(self.levelCompleted) then
			system:sendEvent("onLevelCompleteAnimationFinished", {numStars = self.numStars, oldStars = self.oldStars})
		end
	end
end

function grid:incrementRewindCounter()
	if (self.rewindCounter == nil ) then
		self.rewindCounter = 0
	end
	self.rewindCounter = self.rewindCounter + 1
end

function grid:resetRewindCounter()
	if (self.rewindCounter == nil ) then
		self.rewindCounter = 0
	end
	self.rewindCounter = 0
end

function grid:getRewindCounter()
	if (self.rewindCounter == nil ) then
		self.rewindCounter = 0
	end
	return self.rewindCounter
end

function grid.onComplete(target)
	target.hlCount = target.hlCount + 1
	if target.hlCount < 8 then
		if target.hlCount%2== 1 then
			target.isVisible = false
		else
			target.isVisible = true
		end
		
		tween:to(target, {alpha = 1, time = 0.2, onComplete = grid.onComplete})
	else
		target.isVisible = false
		target.tw = nil
	end
end

function grid:playRedEffect(x,y)
	self.redEffect.x = x
	self.redEffect.y = y
	self.redEffect.isVisible = true
	self.redEffect.timer:resume()
end

function grid:endRedEffect()
	if self.redEffect.isVisible == true then
		self.redEffect.isVisible = false
		self.redEffect.timer:pause()
	end
end

function grid:anim(event)

    local target = event.target
	
	if event.phase == "complete" then
		if target.animation == self.flipStartRemoveAnim then
			target.animation = self.flipStartRemoveLoopAnim
			target:play({startFrame=1, loopCount=0})
		elseif target.animation == self.flipEndRemoveAnim then
			target.isVisible = false
			target:removeEventListener("anim", self)
            target.raisesAnimEvents = false
		end
	end
end

function grid:processTileFarmer(block, forward)
	block.farmerRange.isVisible = not forward
	for i = 1, table.getn(block.farmerBlocks) do
		if forward then
			block.farmerBlocks[i].farmersCount = block.farmerBlocks[i].farmersCount - 1
		else
			block.farmerBlocks[i].farmersCount = block.farmerBlocks[i].farmersCount + 1
		end
	end
end

function grid:hideGhost()

	self.ghost.isVisible = false

	local currentX = self.currentBlock.xGrid
	local currentY = self.currentBlock.yGrid
	
	-- Initialise list of surrounding tiles to check, using -1 for any that are off the edge of the grid
	local blockIndex = {}
	blockIndex[GHOST_UP]	= (( currentY - 1 >= 0				and currentX + (currentY - 1) * GRID_WIDTH) or -1)
	blockIndex[GHOST_DOWN]	= (( currentY + 1 < self.gridWidth	and currentX + (currentY + 1) * GRID_WIDTH)	or -1)
	blockIndex[GHOST_LEFT]	= (( currentX - 1 >= 0				and (currentX - 1) + currentY * GRID_WIDTH)	or -1)
	blockIndex[GHOST_RIGHT]	= (( currentX + 1 < self.gridWidth	and (currentX + 1) + currentY * GRID_WIDTH) or -1)
	
	for i = 1, 4 do
	
		if blockIndex[i] >= 0 then

			local testBlock = self.blocks[blockIndex[i]]
		
			if testBlock.blockType == BLOCK_TYPE_WALL then

				-- Return tree
				testBlock.wall.alpha = 1
			end
		end
	end

	audio:playSound("Assets/Audio/SFX_GhostReappear.raw")
end

function grid:checkGhostAdjacentSquares(block)
	self.ghost.x = block.parent.x
	self.ghost.y = block.parent.y + block.h
	self.ghost.isVisible = true

	self:restoreGhostEffect()
	audio:playSound("Assets/Audio/SFX_GhostFade.raw")
	
	for i = 1, 4 do
		self.ghost.animChild[i].isVisible = false
		self.ghost.animChild[i].looper.isVisible = false
	end

	local currentX = block.xGrid --math.floor((block.parent.x - self.tileSize / 2) / self.tileSize)
	local currentY = block.yGrid--(640 * UI_SCALE - block.parent.y - self.tileSize / 2) / self.tileSize
	
	-- Initialise list of surrounding tiles to check, using -1 for any that are off the edge of the grid
	local blockIndex = {}
	blockIndex[GHOST_UP]	= (( currentY - 1 >= 0				and currentX + (currentY - 1) * GRID_WIDTH) or -1)
	blockIndex[GHOST_DOWN]	= (( currentY + 1 < self.gridWidth	and currentX + (currentY + 1) * GRID_WIDTH)	or -1)
	blockIndex[GHOST_LEFT]	= (( currentX - 1 >= 0				and (currentX - 1) + currentY * GRID_WIDTH)	or -1)
	blockIndex[GHOST_RIGHT]	= (( currentX + 1 < self.gridWidth	and (currentX + 1) + currentY * GRID_WIDTH) or -1)
	
	for i = 1, 4 do
	
		if blockIndex[i] >= 0 then

			local testBlock = self.blocks[blockIndex[i]]
		
			if testBlock.blockType == BLOCK_TYPE_WALL and (testBlock.farmersCount == nil or testBlock.farmersCount == 0) then
		
				-- Add ghost activation effect
				self.ghost.animChild[i].alpha = 1
				self.ghost.animChild[i].isVisible = true
				self.ghost.animChild[i].animation = self.ghostStartAnim
				self.ghost.animChild[i]:play({startFrame=1, loopCount=1})

				self.ghost.animChild[i].looper.alpha = 0
				self.ghost.animChild[i].looper.isVisible = true
				self.ghost.animChild[i].looper.animation = self.ghostLoopAnim
				self.ghost.animChild[i].looper:play({startFrame=1, loopCount=0})
				
				tween:to( self.ghost.animChild[i], { alpha = 0, delay = 0.4, time = 0.2 } )
				tween:to( self.ghost.animChild[i].looper, { alpha = 1, delay = 0.3, time = 0.3 } )

				-- Alpha out tree
				testBlock.wall.alpha = 0.5
				testBlock.wall.isVisible = true
			end
		end
	end
end

function grid:showLines(show)
	local n = table.getn(self.revealedBlocks)
	for i = 1, n do
		self.revealedBlocks[i].line.isVisible = show
	end
end

function grid:updatePath(currentBlock)
	local previous = nil
	if table.getn(self.revealedBlocks) >= 2 then
		previous = self.revealedBlocks[table.getn(self.revealedBlocks) - 1]
	end
	
	local current = self.revealedBlocks[table.getn(self.revealedBlocks)]
	local next = self.currentBlock
	
	if currentBlock ~= nil then
		for i = 1, table.getn(self.revealedBlocks) do
			if self.revealedBlocks[i] == currentBlock.nextBlock then
				next = currentBlock.nextBlock
				current = self.revealedBlocks[i - 1]
				previous = self.revealedBlocks[i - 2]
				break
			end
		end
	end
	
	local frameNumber = 2
	local dirTo = self:getDir(current, next)
	if previous == nil or next == current then
		-- Check for final tile. Use the circular one if we're finishing on a cow, or the starting tile otherwise
		if next == current and current.blockType == BLOCK_TYPE_FLIP then
			frameNumber = BLOCK_LINE_SPRITE_ID[8][1]
		else
			if next == current then
				dirTo = self:getDir(current, previous)
			end
			if dirTo == "L" then
				frameNumber = BLOCK_LINE_SPRITE_ID[7][1]
			elseif dirTo == "D" then
				frameNumber = BLOCK_LINE_SPRITE_ID[7][3]
			elseif dirTo == "R" then
				frameNumber = BLOCK_LINE_SPRITE_ID[7][2]
			else
				frameNumber = BLOCK_LINE_SPRITE_ID[7][4]
			end
		end	
	else
		
		local dirFrom = self:getDir(previous, current)
		if (dirTo == "U" and dirFrom == "U")  or (dirTo == "D" and dirFrom == "D") then
			frameNumber = BLOCK_LINE_SPRITE_ID[6][math.random(1,table.getn(BLOCK_LINE_SPRITE_ID[6]))]
		elseif (dirTo == "R" and dirFrom == "R")  or (dirTo == "L" and dirFrom == "L") then
			frameNumber = BLOCK_LINE_SPRITE_ID[5][math.random(1,table.getn(BLOCK_LINE_SPRITE_ID[5]))]
		elseif (dirFrom == "D" and dirTo == "L") or (dirFrom == "R" and dirTo == "U")then
			frameNumber =  BLOCK_LINE_SPRITE_ID[4][1]
		elseif (dirFrom == "R" and dirTo == "D") or (dirFrom == "U" and dirTo == "L")then
			frameNumber =  BLOCK_LINE_SPRITE_ID[2][1]
		elseif (dirFrom == "D" and dirTo == "R") or (dirFrom == "L" and dirTo == "U")then
			frameNumber =  BLOCK_LINE_SPRITE_ID[3][1]
		elseif (dirFrom == "L" and dirTo == "D") or (dirFrom == "U" and dirTo == "R") then
			frameNumber = BLOCK_LINE_SPRITE_ID[1][1]
		end
		
	end
	
	current.line.xFlip = false
	
	-- Special case for cow tiles
	if (current.blockType == BLOCK_TYPE_WALL or current.blockType == BLOCK_TYPE_FLIP) and next ~= current then
	
		-- If we've been across a tree twice
		if (current.blockType == BLOCK_TYPE_WALL and current.isGhostHidden > 1) then
		
			-- use the empty crossroads piece
			frameNumber = BLOCK_LINE_SPRITE_ID[21][1]
		
		-- If we've been across a cow twice
		elseif (current.blockType == BLOCK_TYPE_FLIP and current.numTransit == 0) then

			-- use the standards crossroads piece
			frameNumber = BLOCK_LINE_SPRITE_ID[10][1]

			-- Check for start/end on cow tile
			if self.revealedBlocks[1] == current then
				frameNumber = BLOCK_LINE_SPRITE_ID[8][1]
			end

		-- If we've started on a cow (not possible for a tree!)
		elseif previous == nil then
			if dirTo == "U" then
				frameNumber = BLOCK_LINE_SPRITE_ID[18][1]
			elseif dirTo == "D" then
				frameNumber = BLOCK_LINE_SPRITE_ID[19][1]
			elseif dirTo == "L" then
				frameNumber = BLOCK_LINE_SPRITE_ID[20][1]
			elseif dirTo == "R" then
				frameNumber = BLOCK_LINE_SPRITE_ID[17][1]
			end
		
		-- Otherwise, we've moved onto the tree or cow once, so draw a partial line
		else
			local dirFrom = self:getDir(previous, current)
			if (dirTo == "U" and dirFrom == "U")  or (dirTo == "D" and dirFrom == "D") then
				frameNumber = BLOCK_LINE_SPRITE_ID[11][1]
			elseif (dirTo == "R" and dirFrom == "R")  or (dirTo == "L" and dirFrom == "L") then
				frameNumber = BLOCK_LINE_SPRITE_ID[12][1]
			elseif (dirFrom == "D" and dirTo == "L") or (dirFrom == "R" and dirTo == "U")then
				frameNumber =  BLOCK_LINE_SPRITE_ID[14][1]
			elseif (dirFrom == "R" and dirTo == "D") or (dirFrom == "U" and dirTo == "L")then
				frameNumber =  BLOCK_LINE_SPRITE_ID[13][1]
			elseif (dirFrom == "D" and dirTo == "R") or (dirFrom == "L" and dirTo == "U")then
				frameNumber =  BLOCK_LINE_SPRITE_ID[16][1]
			elseif (dirFrom == "L" and dirTo == "D") or (dirFrom == "U" and dirTo == "R") then
				frameNumber = BLOCK_LINE_SPRITE_ID[15][1]
			end
		end
	end

	current.line.isVisible = true
	current.line.alpha = 0
	
	tween:to(current.line, {alpha = 1, time = 0.3})

	current.line:setFrame(frameNumber)
	current.completionSprite:setFrame(frameNumber)
	--current.completionSprite.alpha = 0.0
	current.completionSprite.isVisible = true
	current.completionSprite.xFlip = current.line.xFlip
end

g_effects = {}
function effectAnimCB(self, event)
	if event.phase == "complete" then
		g_effects[event.target] = nil
		event.target:removeFromParent()
	end
	return true
end
function clearEffects()
	for dust,_ in pairs(g_effects) do
		dust:removeFromParent()
	end
	g_effects = {}
end
function grid:addRemovalEffect(block)
	self:addEffect(self.removalAnim, block)
end
function grid:addDust(block)
	if self.dustTrailIndex==nil or self.dustTrailAnim[self.dustTrailIndex]==nil then self.dustTrailIndex=1 end
	self:addEffect(self.dustTrailAnim[self.dustTrailIndex], block)
	self.dustTrailIndex = self.dustTrailIndex+1
end
function grid:addEffect(anim, block)

	--HACK
--	if true then return end

	if self.dustTrailIndex==nil or self.dustTrailAnim[self.dustTrailIndex]==nil then self.dustTrailIndex=1 end
	local effect = director:createSprite( { source=anim } )
	effect.isTouchable = false
	self.node:addChild(effect)

	effect.xScale = block.parent.xScale
	effect.yScale = block.parent.yScale
	effect.xAnchor = 0.5
	effect.yAnchor = 0.5
	effect.zOrder = 19
	effect:play({startFrame=1, loopCount=1})
	
	effect.x = block.parent.x
	effect.y = block.parent.y

	g_effects[effect] = true
	effect.anim = effectAnimCB -- add an 'anim' function to recieve animation messages
    effect.raisesAnimEvents = true
	effect:addEventListener("anim", effect)

end

function grid:updateCursorPosition(block, interpolate)
	if interpolate then
	t.starttime = system:getTime()
		if self.cursor.tween ~= nil then
			-- HACKY - reuse old tween object to avoid the overhead of creating a new one all the time
			local cursorTween = self.cursor.tween
			if cursorTween.isComplete then
				table.insert(self.cursor.tweens, self.cursor.tween)
				cursorTween.isComplete = false
			end
			cursorTween.elapsedTime = 0
			--FIX to the latest quick modification
			cursorTween.valuesFrom.x = self.cursor.x
			cursorTween.valuesFrom.y = self.cursor.y
			cursorTween.valuesTo.x = block.parent.x
			cursorTween.valuesTo.y = block.parent.y + self.node.y

			--[[
			cursorTween.tofromvalues.x = self.cursor.x
			cursorTween.tofromvalues.y = self.cursor.y
			cursorTween.values.x = block.parent.x
			cursorTween.values.y = block.parent.y + self.node.y
			--]]
		else
			self.cursor.tween = tween:to(self.cursor, {x = block.parent.x, y = block.parent.y + self.node.y, time=0.2 })
		end
	else
		if self.cursor.tween ~= nil then
			tween:cancel(self.cursor.tween)
			self.cursor.tween.isComplete = true --necessary so the above code can re-use the tween object
		end
		self.cursor.x = block.parent.x
		self.cursor.y = block.parent.y + self.node.y
	end
	self.cursor.isVisible = true
	if self.cursor.beamLoopHandle == nil then
		self.cursor.beamLoopHandle = audio:playSound("Assets/Audio/SFX_BeamLoop.raw", true)
   	end
	if #self.revealedBlocks > 0 then
	    local current = self.revealedBlocks[#self.revealedBlocks]
	    local cursorDir =  self:getDir(current, block)
	    self.cursorHalfTile.isVisible = true
		block.parent:addChild(self.cursorHalfTile)
		self.cursorHalfTile.x = block.x
		self.cursorHalfTile.y = block.y
		self.cursorHalfTile.xScale = 1.0
		self.cursorHalfTile.yScale = 1.0
		self.cursorHalfTile.zOrder = 1
	    --[[
		self.cursorHalfTile.x = block.parent.x
	    self.cursorHalfTile.y = block.parent.y
		--]]
        self.cursorHalfTile:setFrame(CURSOR_HALF_LINE_SPRITE_ID[cursorDir])
    end
end

function grid:removeBlockToReveal(block)
	local index = 1
	for i,v in ipairs(self.blocksToReveal) do
		if v == block then
			index = i
			break
		end
	end
	table.remove(self.blocksToReveal, index)
end
function grid:setBlockType(block, type, increment)
	if block.blockType == BLOCK_TYPE_WALL and type ~= BLOCK_TYPE_WALL then
		table.insert(self.blocksToReveal, block)
	elseif block.blockType ~= BLOCK_TYPE_WALL and type == BLOCK_TYPE_WALL then
		self:removeBlockToReveal(block)
	end
	local lastType = block.blockType
	block.blockType = type
	block:setFrame(BLOCK_LINE_SPRITE_ID [9][math.random(1, table.getn(BLOCK_LINE_SPRITE_ID [9]))])
	block.rotation = 0
	if block.blockType == BLOCK_TYPE_ARROW then
		if block.arrow == nil then
			block.arrow = director:createSprite({zOrder = 2,xAnchor=0.5, yAnchor = 0.5, source=self.arrowAnim }  )
			--block.arrow.x = block.w / 2
			--block.arrow.y = block.h / 2
			block.arrow:setFrame(1)
			block.arrow.isTouchable = false
			block.arrow.zOrder = 5
			--block.arrow.xScale = block.parent.xScale
			--block.arrow.yScale = block.parent.yScale
			block.parent:addChild(block.arrow)
			
			block.arrow.sprite = director:createSprite({xAnchor=0.5, yAnchor = 0.5, source=self.arrowAnim }  )
			block.arrow.sprite.x = block.w / 2
			block.arrow.sprite.y = block.h / 2
			block:addChild(block.arrow.sprite)
		end

		if increment then
			if block.blockTypeParam == "U" then block.blockTypeParam = "R" 
			elseif block.blockTypeParam == "R" then block.blockTypeParam = "D" 
			elseif block.blockTypeParam == "D" then block.blockTypeParam = "L" 
			elseif block.blockTypeParam == "L" then block.blockTypeParam = "U" 
			end
		end

		block.arrow.xFlip = false
		block.arrow.sprite.rotation = 0
		block.arrow.sprite.yFlip = false
		block.arrow.sprite.xFlip = false

		local entry = block.blockTypeParam:sub(1,1)
		local exit  = block.blockTypeParam:sub(-1)

		if (entry ~= exit) then
			block.arrow:setFrame((entry=="U" or exit=="D") and 3 or 4)
			block.arrow.xFlip = (entry=="L" or exit=="R")

			block.arrow.sprite:setFrame(6)

			if (block.blockTypeParam=="UL") then
				block.arrow.sprite:setFrame(6)
			elseif (block.blockTypeParam=="UR") then
				block.arrow.sprite:setFrame(6)
				block.arrow.sprite.xFlip = true
			elseif (block.blockTypeParam=="DR") then
				block.arrow.sprite:setFrame(9)
				block.arrow.sprite.xFlip = true
			elseif (block.blockTypeParam=="DL") then
				block.arrow.sprite:setFrame(9)
			elseif (block.blockTypeParam=="RU") then
				block.arrow.sprite:setFrame(10)
			elseif (block.blockTypeParam=="LD") then
				block.arrow.sprite:setFrame(11)
				block.arrow.sprite.xFlip = true
			elseif (block.blockTypeParam=="LU") then
				block.arrow.sprite:setFrame(10)
				block.arrow.sprite.xFlip = true
			elseif (block.blockTypeParam=="RD") then
				block.arrow.sprite:setFrame(11)
			end
		else
			block.arrow.sprite:setFrame(5)
			if entry == "U" then
				block.arrow.sprite:setFrame(5)
				block.arrow:setFrame(1)
			elseif entry == "R" then
				block.arrow.sprite:setFrame(8)
				block.arrow:setFrame(2)
			elseif entry == "D" then
				block.arrow.sprite:setFrame(7)
				block.arrow:setFrame(1)
			elseif entry == "L" then
				block.arrow.sprite:setFrame(8)
				block.arrow.sprite.xFlip = true
				block.arrow:setFrame(2)
			end
		end
	elseif block.blockType == BLOCK_TYPE_NUM then
		if increment then
			if lastType == BLOCK_TYPE_NUM then
				block.blockTypeParam = block.blockTypeParam + 1
				if block.blockTypeParam > BLOCK_NUM_MAX then
					block.blockTypeParam = 1
				end
			else
				block.blockTypeParam = 1
			end
		
		end
		if block.num == nil then
			block.num = director:createSprite({xAnchor=0.5, yAnchor = 0.5, source=self.numAnims[block.blockTypeParam] }  )
			block.num.x = block.w / 2
			block.num.y = block.h / 2
			block.num.isTouchable = false
			block:addChild(block.num)
		else
			block.num:setAnimation(self.numAnims[block.blockTypeParam])
		end
		--block.flip:play({startFrame=1, loopCount=0})
		--block.num:setFrame(BLOCK_TYPE_SPRITE_ID[type] + block.blockTypeParam - 1)
	elseif block.blockType == BLOCK_TYPE_FLIP then
		block.blockTypeParam = 2
		block.numTransit = 2
		if block.flip == nil then
			block.flip = director:createSprite({zOrder = 2,xAnchor=0.5, yAnchor = 0.5, source=self.flipIdleAnim }  )
			block.flip.x = block.parent.x
			block.flip.y = block.parent.y
			block.flip.xScale = block.parent.xScale
			block.flip.yScale = block.parent.yScale
			block.flip.isTouchable = false
			self.node:addChild(block.flip)			
			
		end
	elseif block.blockType == BLOCK_TYPE_GHOST then
		if block.ghost == nil then
			block.ghost = director:createSprite({xAnchor=0.5, yAnchor = 0.5, source="Assets/ghostTile.png", }  )
			block.ghost.x = block.w / 2
			block.ghost.y = block.h / 2
			block:addChild(block.ghost)
		end
		block.ghost:setFrame(BLOCK_TYPE_SPRITE_ID[type])
	elseif block.blockType == BLOCK_TYPE_WALL then
		if block.wall == nil then
			block.wall = director:createSprite({xAnchor=0.5, yAnchor = 0.5, source=self.treeAnim }  )
			--over here
			block.wall.zOrder = 40
			--[[
			block.wall.x = block.w / 2 
			block.wall.y = block.h / 2 
			block:addChild(block.wall)
			--]]
			
			block.wall.x = block.x
			block.wall.y = block.y
			block.wall.isTouchable = false
			block.parent:addChild(block.wall)
			
		end
		block.wall:setFrame(1 + math.random(0,5))
	elseif block.blockType == BLOCK_TYPE_FARMER then
		if block.farmer == nil then
			
			block.farmer = director:createSprite({xAnchor=0.5, yAnchor = 0.5, source=self.idleBotFarmerAnim }  )
			block.farmer.x = block.w / 2
			block.farmer.y = block.h / 2
			block:addChild(block.farmer)
			local farmerRange = director:createSprite({zOrder = 2, xAnchor=0.75, yAnchor = 0.5, source="Assets/farmerHitAreaGradient.png" } )
			farmerRange.isTouchable = false
			self.node:addChild(farmerRange)
			block.farmerRange = farmerRange
		end
		block.farmerRange.xScale = block.parent.xScale
		block.farmerRange.yScale = block.parent.yScale
		--
		if block.blockTypeParam == "U" then
			if increment then
				block.blockTypeParam = "R"
				block.farmer:setFrame(2)
			else
				block.farmerRange.rotation = 90
				block.farmerRange.x = block.parent.x
				block.farmerRange.y = block.parent.y
				block.farmer:setAnimation(self.idleTopFarmerAnim)
				block.farmer:play({startFrame = math.random(1,90)})
			end
		elseif block.blockTypeParam == "R" then
			if increment then
				block.blockTypeParam = "D"
				block.farmer:setFrame(4)
			else
				block.farmerRange.rotation = 180
				block.farmerRange.x = block.parent.x
				block.farmerRange.y = block.parent.y
				block.farmer:setAnimation(self.idleHorizontalFarmerAnim)
				block.farmer:play({startFrame = math.random(1,90)})
				block.farmer.xFlip = true
			end
		elseif block.blockTypeParam == "D" then
			if increment then
				block.farmer:setFrame(1)
				block.blockTypeParam = "L"
			else
				block.farmerRange.rotation = 270
				block.farmerRange.x = block.parent.x 
				block.farmerRange.y = block.parent.y
				block.farmer.xFlip = false
				block.farmer:setAnimation(self.idleBotFarmerAnim)
				block.farmer:play({startFrame = math.random(1,90)})
			end
		else
			if increment then
				block.farmer:setFrame(3)
				block.blockTypeParam = "U"
			else
				block.farmer:setAnimation(self.idleHorizontalFarmerAnim)
				block.farmer:play({startFrame = math.random(1,90)})
				block.farmer.xFlip = false
				block.farmerRange.rotation = 0
				block.farmerRange.x = block.parent.x
				block.farmerRange.y = block.parent.y
			end
		end
	elseif block.blockType == BLOCK_TYPE_PIPE then
		if block.pipe == nil then
			block.pipe = director:createSprite({zOrder = 2,xAnchor=0.5, yAnchor = 0.5, source=self.arrowAnim }  )
			--block.pipe.x = block.w / 2
			--block.pipe.y = block.h / 2
			--block.pipe.xScale = block.parent.xScale
			--block.pipe.yScale = block.parent.yScale
			block.pipe.isTouchable = false
			block.parent:addChild(block.pipe)
		end
		block.pipe:setFrame(BLOCK_TYPE_SPRITE_ID[type])
		if block.blockTypeParam == "H" then
			block.pipe:setFrame(2)
		else
			block.pipe:setFrame(1)
		end
	else
		block.blockTypeParam = nil
	end
end

function grid:initFarmerBlocks()
	
	local flooredTileSize = math.floor(self.tileSize)
	for i = 1, table.getn(self.farmerBlocks) do
		
		local farmerBlock = self.farmerBlocks[i]
		farmerBlock.farmerBlocks = {}
		local currentX = math.floor((farmerBlock.parent.x - flooredTileSize / 2) / flooredTileSize)
		local currentY =  math.floor(((640 * UI_SCALE - farmerBlock.parent.y) - flooredTileSize / 2) / flooredTileSize)
		local blockIndex = 0
		local blockIndex2 = 0
		local blockIndex3 = 0		
		if farmerBlock.blockTypeParam == "L" then
			blockIndex = (currentX - 1 ) + currentY * GRID_WIDTH
			blockIndex2 = (currentX - 1 ) + (currentY - 1) * GRID_WIDTH
			blockIndex3 = (currentX - 1 ) + (currentY + 1 ) * GRID_WIDTH
		elseif farmerBlock.blockTypeParam == "R" then
			blockIndex = (currentX + 1 ) + currentY * GRID_WIDTH
			blockIndex2 = (currentX + 1 ) + (currentY - 1) * GRID_WIDTH
			blockIndex3 = (currentX + 1 ) + (currentY + 1 ) * GRID_WIDTH
		elseif farmerBlock.blockTypeParam == "U" then
			blockIndex = (currentX) + (currentY - 1) * GRID_WIDTH
			blockIndex2 = (currentX - 1 ) + (currentY - 1) * GRID_WIDTH
			blockIndex3 = (currentX + 1 ) + (currentY - 1 ) * GRID_WIDTH
		elseif farmerBlock.blockTypeParam == "D" then
			blockIndex = (currentX) + (currentY + 1) * GRID_WIDTH
			blockIndex2 = (currentX - 1 ) + (currentY + 1) * GRID_WIDTH
			blockIndex3 = (currentX + 1 ) + (currentY + 1 ) * GRID_WIDTH
		end

		-- Make sure blockIndexes are integers by rounding to nearest whole number!
		--blockIndex = math.floor(blockIndex + 0.5)
		--blockIndex2 = math.floor(blockIndex2 + 0.5)
		--blockIndex3 = math.floor(blockIndex3 + 0.5)
		
		if self.blocks[blockIndex].farmersCount == nil then
			self.blocks[blockIndex].farmersCount = 1
		else
			self.blocks[blockIndex].farmersCount = self.blocks[blockIndex].farmersCount + 1
		end
		if self.blocks[blockIndex2].farmersCount == nil then
			self.blocks[blockIndex2].farmersCount = 1
		else
			self.blocks[blockIndex2].farmersCount = self.blocks[blockIndex2].farmersCount + 1
		end
		if self.blocks[blockIndex3].farmersCount == nil then
			self.blocks[blockIndex3].farmersCount = 1
		else
			self.blocks[blockIndex3].farmersCount = self.blocks[blockIndex3].farmersCount + 1
		end
		
		table.insert(farmerBlock.farmerBlocks, self.blocks[blockIndex])
		table.insert(farmerBlock.farmerBlocks, self.blocks[blockIndex2])
		table.insert(farmerBlock.farmerBlocks, self.blocks[blockIndex3])
	end
end
function grid:resetGrid()
	if self.rewindPathParameters==nil then 
		-- quick out here in case this is called before the grid's been initialised
		return 
	end

	self.rewindPathParameters.revealedElementsToRemove = 0
	self.numStars = 0
	for i = 0, #self.blocks do
        local block = self.blocks[i]
		block.isHidden = false
		block.isGhostHidden = 0
		block.farmersCount = nil
		if block.farmerRange ~= nil then
			block.farmerRange.isVisible = true
		end
		if block.wall ~= nil then
			block.wall.alpha = 1
		end
		if block.flip ~= nil then
			--cow tile bug fixing
			if(block.flip.parent ~= self.node) then
				self.node:addChild(block.flip)
			end
			
			block.flip:removeEventListener("anim", self)
            block.flip.raisesAnimEvents = false
			block.flip.isVisible = true
			block.flip:setAnimation(self.flipIdleAnim)
			block.flip:play()
			block.numTransit = 2
		end
		if block.line ~= nil then
			block.line.isVisible = false
		end
		self:unhideBlock(self.blocks[i])
	end
	self:endRedEffect()
	self:initFarmerBlocks()
	self.ghost.isVisible = false
	self.currentBlock = nil
	self.lastTouchedBlock = nil
	self.cursor.isVisible = false

    if self.cursor.beamLoopHandle ~= nil then
	    audio:stopSound( self.cursor.beamLoopHandle)
        self.cursor.beamLoopHandle = nil
	    audio:playSound("Assets/Audio/SFX_BeamLoopEnd.raw")
    end
	self.revealedBlocks = {}
	self.currentBlock = nil
	self.blockNumIndex = 0
	self.numMoves = 0
	self.cursorHalfTile.isVisible = false

	clearEffects()
end

function grid:clearGrid()
	for i = 0, n do
		self.blocks[i].isVisible = true
		self.blocks[i].isHidden = false
		if self.blocks[i].line ~= nil then
			self.blocks[i].line.isVisible = false
		end
		self.blocks[i].blockType = BLOCK_TYPE_DEFAULT
		self.blocks[i].blockTypeParam = nil
	end
	self.cursorHalfTile.isVisible = false
	self.cursor.isVisible = false
	self.revealedBlocks = {}
	self.currentBlock = nil
	self.blockNumIndex = 0
	self.numMoves = 0
	self.levelComplete = false
end

function grid:setHintSpritesVisible(visibleFlag)
	self.hintSpriteStart.isVisible = visibleFlag
	self.hintSpriteEnd.isVisible = visibleFlag
	if visibleFlag then
		self.hintSpriteStart.glow = true
		self.hintSpriteEnd.glow = true
		--if self.cursor.isVisible == false or self.
		tween:to(self.hintSpriteStart, {alpha = 1, time = 0.66, onComplete=grid.glow})
		tween:to(self.hintSpriteEnd, {alpha = 1, time = 0.66, onComplete=grid.glow})
	end
end

function grid:lock()
	self.gridLocked = true
end

function grid:unLock()
	self.gridLocked = false
end
