--
dofile("gameGrid.lua")
-- editor depreciated dofile("UI/editorHUD.lua")
dofile("UI/gameHUD.lua")
dofile("UI/resultScreen.lua")
dofile("ConnectionRulesManager.lua")
dofile("PathRulesManager.lua")
gameScene = {}

gameScene.STUCK_TUTORIAL_REWIND_THRESHOLD = 10

gameScene_mt = { __index = gameScene }

function gameScene.create()
	local instance = {}
	setmetatable( instance, gameScene_mt)
	instance.scene = director:createScene()
	instance.scene.name = "Scene: Game"
	instance.scene:addEventListener({ "setUp", "tearDown", "enterPreTransition",
	    "enterPostTransition", "exitPreTransition", "exitPostTransition" }, instance)
	instance.gameGrid = grid.init()
-- editor depreciated	instance.editorHud = hud.init()
	instance.gameHud = gHud.init()
	instance.result = resultScreen.init()
	return instance
end

function gameScene:start(levelId, worldInfo, editMode)
	self.levelId = levelId
-- editor depreciated	self.isEditorMode = editMode
	self.worldInfo = worldInfo
	--saving the musicVolume that will be changed for the background to be played as stream
	self.musicVolume = audio.musicVolume
	--tween:to( audio, {musicVolume = 0, time = 0.5})--TODO check this time setting
	director:moveToScene(self.scene)
end

function gameScene:restart(levelId, worldInfo)
	self.levelId = levelId
	self.worldInfo = worldInfo
	self.gameGrid:destroy()
	self.result:destroy()
	local showHintButton = true
	if self.worldInfo.levels[self.levelId].hintUsed == true then
		showHintButton = false
		
	end
	self.gameHud:reset()
	self.gameHud:show(true)
	self.gameHud:showHintButton(showHintButton)
	self.result:create(self.scene)
	self.gameGrid:create(0, 0, director.displayWidth, director.displayHeight - 120, self.scene, self.levelId, self.worldInfo)
	
	
	if self.worldInfo.levels[self.levelId].hintUsed == true then		
		self.gameGrid:setHintSpritesVisible(true)
	end
		
	self.requestFadeOut = true
	self.gameHud:updateTitle(worldInfo.id, levelId)
end

function gameScene:tutorialsEnded(event)
	self.gameGrid:unLock()
	self.gameHud:unLock()
	system:removeEventListener("tutorialsEnded", self)
end

function gameScene:purchaseHints()
	self.gameGrid:lock()
	self.gameHud:lock()
	self.gameHud:show(false)
	system:sendEvent("showTutorial",{show=false})
	system:addEventListener("inAppPurchaseComplete", self)
	hintsPurchaseManager:showHintsRequest(self.scene)
end

function gameScene:inAppPurchaseComplete(event)
	self.gameGrid:unLock()
	self.gameHud:unLock()
	self.gameHud:show(true)
	--self.gameHud:updateHintsCount(hintsPurchaseManager:getHintsCountString())
	--self.gameHud:updateHintsCount(""..g_getHintsCount())
	system:sendEvent("updateHintsCount",{})
	system:sendEvent("showTutorial",{show=true})
	system:removeEventListener("inAppPurchaseComplete", self)
end

function gameScene:launchTutorials()
	--self.gameGrid:lock()
	--self.gameHud:lock()
	--[[
	if(tutorialsManager:launchTutorials("tutorialsEnded")) then
		system:addEventListener("tutorialsEnded", self)
	end
	--]]
end

function gameScene:checkTutorialsOnStart(usedTileTypes)
	tutorialsManager:gridGenerated(usedTileTypes, self.worldInfo.id, self.levelId)
	--[[
	tutorialsManager:gridGenerated(usedTileTypes, self.scene)
	if (tutorialsManager:isAnyTutorialScheduled()) then
		self:launchTutorials()
	end	
	--]]
end

function gameScene:setUp(event)
    dbg.print("gameScene:setUp")

    -- Make sure the front end assets have been unloaded before we continue
    UnloadFrontEndAssets()
    stopPreLoader()

    -- Precache all the sound effects
    audio:loadSound("Assets/Audio/SFX_BeamLoop.raw")
    audio:loadSound("Assets/Audio/SFX_BeamLoopEnd.raw")
    audio:loadSound("Assets/Audio/SFX_BlockedTile.raw")
    audio:loadSound("Assets/Audio/SFX_Complete.raw")
    audio:loadSound("Assets/Audio/SFX_Cow.raw")
    audio:loadSound("Assets/Audio/SFX_CowVanish.raw")
    audio:loadSound("Assets/Audio/SFX_Farmer_1.raw")
    audio:loadSound("Assets/Audio/SFX_Farmer_2.raw")
    audio:loadSound("Assets/Audio/SFX_Farmer_3.raw")
    audio:loadSound("Assets/Audio/SFX_Farmer_4.raw")
    audio:loadSound("Assets/Audio/SFX_Farmer_5.raw")
    audio:loadSound("Assets/Audio/SFX_GhostFade.raw")
    audio:loadSound("Assets/Audio/SFX_GhostReappear.raw")
    audio:loadSound("Assets/Audio/SFX_GlowingCropCircle.raw")
    audio:loadSound("Assets/Audio/SFX_Number_1.raw")
    audio:loadSound("Assets/Audio/SFX_Number_2.raw")
    audio:loadSound("Assets/Audio/SFX_Number_3.raw")
    audio:loadSound("Assets/Audio/SFX_Reverse.raw")


	self.fadeScreen = director:createRectangle( {
        x=director.displayCenterX,
        y=director.displayCenterY,
        w=director.displayWidth,
        h=director.displayHeight,
	    xAnchor = 0.5,
        yAnchor = 0.5,
        zOrder = 10000,
        IsVisible = true,
        color={0, 0, 0},
        strokeColor={0, 0, 0},
	    strokeAlpha = 1,	
	    alpha = 1
    } )
	self.scene:addChild(self.fadeScreen)
	self.gameGrid:create(0, 0, director.displayWidth, director.displayHeight - 120, self.scene, self.levelId, self.worldInfo)
	
--[[
Editor depreciated
   	if self.isEditorMode then
		self.editorHud:create(0, 840, self.scene)
		self.gameGrid.isEditing = true
		system:addEventListener( { "onEditModeChanged", "onBlockTouched", "onGridSizeChanged", "onSaveLevel"} , self)
	else
]]
		local showHintButton = true
		if self.worldInfo.levels[self.levelId].hintUsed == true then
			showHintButton = false
			self.gameGrid:setHintSpritesVisible(true)
		end
		self.result:create(self.scene)
		--self.gameHud:create(0, 840, self.scene, showHintButton,self.worldInfo.id, self.levelId, hintsPurchaseManager:getHintsCountString() )
		self.gameHud:create(0, 840, self.scene, showHintButton,self.worldInfo.id, self.levelId, ""..g_getHintsCount() )
		
		system:addEventListener({"onRestart", "updateHintsCount"},self)
--	end

	tutorialsManager:setUp(self.scene)	
    system:addEventListener( {"onLevelComplete", "onLevelCompleteAnimationFinished",
        "backToLevelSelection", "showHint", "goToNextLevel", "onBlockRewinded",
        "skipCurrentLevel", "update" }, self)
   
    self.fadeScreen.scene = self
	self.requestFadeOut = true
	dbg.print("debug 1 audio.sfxVolume " ..audio.sfxVolume)
	audio:stopStream()
	audio.musicVolume = audio.sfxVolume
	if(audio.musicVolume > 0)then
		dbg.print("debug 2")
		audio:playStream("Assets/Audio/AMB_Loop.mp3", true)
	end
	
	dialogManager:setUp()
--    profiler.start("profile.txt")
end



function gameScene:startShowOutroSequence()
	--halt the hovering beam SFX
	dbg.print("------------------gameScene:startShowOutroSequence()-----------------")
	tutorialsManager:quitTutorials()
	audio:stopSound()
	self.requestFadeIn = true
	self.showOutroFlag = true	
end

function gameScene:backToUpsellScreen()
	tutorialsManager:quitTutorials()
	audio:stopSound()
	self.requestFadeIn = true
	self.showUpsellFlag = true
end

function gameScene:backToLevelSelection()
	tutorialsManager:quitTutorials()

	-- Need to reset the music volume here to stop possible short burst of frontend music if music is disabled
	audio:stopStream()
	audio.musicVolume = self.musicVolume

	audio:playSound("Assets/Audio/SFX_Back.raw")
	self.requestFadeIn = true
	self.backToLevelSelect = true
end

function gameScene:goToNextLevel()
	self.requestFadeIn = true
end

function gameScene:skipCurrentLevel()
	self.gameGrid:completeLevel()
end
function gameScene:update(event)
	if self.requestFadeOut == true then
		self.requestFadeOut = false		

		if self.fadeScreen.alpha == 1 then
			self.fadeScreen.isVisible = true
			self.fadeScreen.alpha = 1
			tween:to(self.fadeScreen, {alpha = 0, time = 1, onComplete=gameScene.onFadeOutComplete})
		end

	elseif self.requestFadeIn == true then
		self.requestFadeIn = false

--		dbg.print("gameScene.update - requested black screen fade in")
	
		if self.fadeScreen.alpha == 0 then
			self.fadeScreen.isVisible = true
			self.fadeScreen.alpha = 0
--			dbg.print("gameScene.update - black screen being faded in")
			tween:to(self.fadeScreen, {alpha = 1, time = 1, onComplete=gameScene.onFadeInComplete})
		else
			-- Inserted to make upsell screen work correctly
			gameScene.onFadeInComplete(self.fadeScreen)
		end
	end
end

function gameScene.onFadeOutComplete(target)
	target.scene:checkTutorialsOnStart(target.scene.gameGrid.usedTileTypes)
	target.isVisible = false
	target.scene.gameGrid:unLock()
	target.scene.gameHud:unLock()
end

function gameScene.onFadeInComplete(target)
	dbg.print("<<<<<<<<<<gameScene.onFadeInComplete>>>>>>>>>>>>>")
	if(target.scene.showOutroFlag ~= nil) then
		if(target.scene.showOutroFlag == true) then
			dbg.print("target.scene.showOutroFlag = TRUE" )
		else
			dbg.print("target.scene.showOutroFlag = FALSE" )
		end
	else
		dbg.print("target.scene.showOutroFlag is nil")
	end
	if target.scene.backToLevelSelect == true then
		system:sendEvent("onBackToLevelSelect", {lastLevelPlayed=target.scene.levelId})
		target.scene.backToLevelSelect = false
	elseif(target.scene.showUpsellFlag == true) then
		target.scene.showUpsellFlag = false
		system:sendEvent("goToUnlockScreen")
	elseif (target.scene.showOutroFlag == true) then
		target.scene.showOutroFlag = false
		system:sendEvent("onOutro")
	else
		system:sendEvent("onGoToNextLevel")
	end
end

function gameScene:onBlockRewinded(event)
	tutorialsManager:onBlockRewinded(event)
	--[[
	--TODO this section has been commented out
	--untill the new tutorial system will be defined
	tutorialsManager:onBlockRewinded(event)
	if(tutorialsManager:isAnyTutorialScheduled()) then--TODO this might be redundant
		self:launchTutorials()
	end
	--]]
end


function gameScene:tearDown(event)
    dbg.print("gameScene:tearDown")
	dialogManager:tearDown()
--    profiler.stop()
	audio.musicVolume = self.musicVolume-- restoring the music volume
	
	tutorialsManager:destroy()

	audio:stopStream()
	self.fadeScreen:removeFromParent()
    self.fadeScreen = nil


	 -- Precache all the sound effects
    audio:unloadSound("Assets/Audio/SFX_BeamLoop.raw")
    --audio:unloadSound("Assets/Audio/SFX_BeamLoopEnd.raw") //this sound might still be playing at this point, deleting it may cause audio corruptions!
    audio:unloadSound("Assets/Audio/SFX_BlockedTile.raw")
    audio:unloadSound("Assets/Audio/SFX_Complete.raw")
    audio:unloadSound("Assets/Audio/SFX_Cow.raw")
    audio:unloadSound("Assets/Audio/SFX_CowVanish.raw")
    audio:unloadSound("Assets/Audio/SFX_Farmer_1.raw")
    audio:unloadSound("Assets/Audio/SFX_Farmer_2.raw")
    audio:unloadSound("Assets/Audio/SFX_Farmer_3.raw")
    audio:unloadSound("Assets/Audio/SFX_Farmer_4.raw")
    audio:unloadSound("Assets/Audio/SFX_Farmer_5.raw")
    audio:unloadSound("Assets/Audio/SFX_GhostFade.raw")
    audio:unloadSound("Assets/Audio/SFX_GhostReappear.raw")
    audio:unloadSound("Assets/Audio/SFX_GlowingCropCircle.raw")
    audio:unloadSound("Assets/Audio/SFX_Number_1.raw")
    audio:unloadSound("Assets/Audio/SFX_Number_2.raw")
    audio:unloadSound("Assets/Audio/SFX_Number_3.raw")
    audio:unloadSound("Assets/Audio/SFX_Reverse.raw")

	self.gameGrid:destroy()
--[[
editor depreciated
	if self.isEditorMode then
		self.editorHud:destroy()
		system:removeEventListener( { "onEditModeChanged", "onBlockTouched"} ,self)
	else
]]
		system:removeEventListener( { "onBlockRewinded", "updateHintsCount" }, self)

		self.gameHud:destroy()
		self.result:destroy()
--	end

	system:removeEventListener( { "skipCurrentLevel", "backToLevelSelection", "goToNextLevel",
                                  "update", "onLevelComplete", "onLevelCompleteAnimationFinished" }, self)
   
    self.scene:releaseResources()
    
end

function gameScene:enterPreTransition(event)
	dbg.print("gameScene:enterPreTransition")
end

function gameScene:enterPostTransition(event)
	dbg.print("gameScene:enterPostTransition")
end

function gameScene:exitPreTransition(event)
	dbg.print("gameScene:exitPreTransition")
end

function gameScene:exitPostTransition(event)
	dbg.print("gameScene:exitPostTransition")
end

function gameScene:onRestart(event)
	self.gameHud:reset()
	self.gameGrid:resetGrid()
	tutorialsManager:levelReset()
end


--[[
Editor depreciated
function gameScene:onGridSizeChanged(event)
	self.gameGrid:resize(event.gridSize)
end

function gameScene:onEditModeChanged(event)
	self.gameGrid.isEditing =  event.isEditing
	if event.isEditing then
		self.gameGrid:resetGrid()
	end
end

function gameScene:onBlockTouched(event)
	if self.editorHud.isEditing then
		self.gameGrid:setBlockType(event.block, self.editorHud.selectedBlockType, true)
	end
end

function gameScene:onSaveLevel(event)
	self.gameGrid:saveCurrent()
end
]]

function gameScene:onLevelComplete(event)
	tutorialsManager:levelComplete()
	audio:stopSound()
	dbg.print(" MIKEB: stopping beam looping sound")
	audio:playSound("Assets/Audio/SFX_GlowingCropCircle.raw")
	self.gameGrid:lock()
	self.gameHud:lock()
	self.gameHud:show(false)
end

function gameScene:isWorldComplete(oldStars)

--	dbg.print("This is level " .. self.levelId .. " of " .. #self.worldInfo.levels .. ", and we had " .. oldStars .. " stars on it before")

	if self.levelId == #self.worldInfo.levels and oldStars == 0 then
		return true
	end

	return false
end

function gameScene:onLevelCompleteAnimationFinished(event)
	audio:playSound("Assets/Audio/SFX_Complete.raw")
	self.result:show(self, event.numStars, self:isWorldComplete(event.oldStars))
--[[
    editor depreciated
	system:addTimer(self, 3, 1,0)
]]
end

--[[
Editor depreciated
function gameScene:timer(event)
	if self.isEditorMode then
		self.editorHud:reset()
		self.gameGrid:resetGrid()
	end
end
]]

function gameScene:getHintsCount()
	--return hintsPurchaseManager:getHintsCount()
	return g_getHintsCount()
end

function gameScene:updateHintsCount(event)
	self.gameHud:updateHintsCount(""..g_getHintsCount())
end

function gameScene:hintConsumed()
	--hintsPurchaseManager:hintConsumed()
	if(g_hintConsumed() == true) then
		--unable to save the updated hint count!!
			tutorialsManager:hintUsed(self.gameGrid.hintSpriteStart.xGrid, self.gameGrid.hintSpriteStart.yGrid)
		--self.gameHud:updateHintsCount(hintsPurchaseManager:getHintsCountString())
		--self.gameHud:updateHintsCount(""..g_getHintsCount())
		system:sendEvent("updateHintsCount", {})
	else
		--ERROR management: unable to save updated hints count
	end
	
end


function gameScene:showHint(event)
	--tutorialsManager:hintButtonPressed()--> moved to gameHud "activate" event handler
	if (self:getHintsCount() > 0) then
		self.worldInfo.levels[self.levelId].hintUsed = true
		self.gameHud:showHintButton(false)
		self.gameGrid:setHintSpritesVisible(true)
		self:hintConsumed()
		
		--[[
		--TODO this section has been commented out
		--untill the new tutorial system will be defined
		
		tutorialsManager:checkTutorialSchedule("S_TUTORIAL_INSTRUCTIONS6")
		if(tutorialsManager:isAnyTutorialScheduled()) then--TODO this might be redundant
			self:launchTutorials()
		end
		--]]
	else
		self:purchaseHints()
	end
end

