gestureManager = {}

gestureManager.IDLE_STATE = -100
gestureManager.DOUBLE_TAP_STATE = -200
gestureManager.DOUBLE_TAP_MAX_TIME = 0.500 --seconds
gestureManager.TAP_BEGAN_ENDED_MAX_TIME = 0.300 --seconds
gestureManager.DOUBLE_TAP_DISTANCE_TOLLERANCE = 50--TODO pixels in screen coordinates, should this be a percentage of screen instead? or scaled by UI_SCALE global parameter?

gestureManager.taps_count = 0

gestureManager.state = gestureManager.IDLE_STATE

gestureManager.tapsData = {}
gestureManager.tapTime = 0
gestureManager.touchStartTime = 0


function gestureManager:changeState(newState)
	--dbg.print("gestureManager change state from " ..self.state .. " to " ..newState)
	if (newState ~= gestureManager.DOUBLE_TAP_STATE or
			newState == gestureManager.IDLE_STATE) then
		
		self:resetTaps()
	elseif (newState == gestureManager.DOUBLE_TAP_STATE) then
		gestureManager.tapTime = system.gameTime
	else
		
	end
	gestureManager.state = newState
end

function gestureManager:incrementTapsCount(touchEvent)
	self.taps_count = self.taps_count + 1
	local count = #(self.tapsData)
	--dbg.print(" ******* gestureManager:incrementTapsCount count = " ..count)
	self.tapsData[count + 1] = touchEvent
	--dbg.print(" ***** gestureManager:incrementTapsCount count = " .. (#(self.tapsData)) )
	dbg.print(" **** TAP DETECTED ********* ")
	
	--system:sendEvent("singleTap", touchEvent)--TODO should we trigger this in the update function?
	
	--TODO should we need a tap gesture, just uncomment the line below
	--or add it in the update function on double tap timer expipring
	--system:sendEvent("tap", touchEvent)
end

function gestureManager:decrementTapsCount()
	self.taps_count = self.taps_count - 1
	local count = #(self.tapsData)
	--dbg.print("gestureManager:decrementTapsCount count = " ..count)
	self.tapsData[count] = nil
	--dbg.print("gestureManager:decrementTapsCount count = " ..(#(self.tapsData)))
end

function gestureManager:resetTaps()
	self.taps_count = 0
	gestureManager.tapsData = {}
end

function gestureManager:reset()
	changeState(gestureManager.IDLE_STATE)
end

function gestureManager:touch(event)
	--dbg.print("gestureManager:touch event phase " ..event.phase)
	if (event.phase == "began") then
		self.touchStartTime = system.gameTime
	end
	--the timer expired event is managed in the system update event handler
	if (self.state == gestureManager.IDLE_STATE) then
		if (event.phase == "ended") then
			if ((system.gameTime - self.touchStartTime) > gestureManager.TAP_BEGAN_ENDED_MAX_TIME) then
				--do nothing this is not an actual tap
			else
				self:changeState(gestureManager.DOUBLE_TAP_STATE)
				self:incrementTapsCount(event)
				--system:sendEvent("singleTap",{x=event.x,y=event.y})
			end
		end
	elseif (self.state == gestureManager.DOUBLE_TAP_STATE) then
		if ((system.gameTime - self.touchStartTime) > gestureManager.TAP_BEGAN_ENDED_MAX_TIME) then
			--do nothing this is not an actual tap
		else
			if (event.phase == "ended") then
				self:incrementTapsCount(event)
				--system:sendEvent("singleTap",{x=event.x,y=event.y})
			end
		end
	else
		--unknown state
	end
end

function gestureManager:update(event)
	--TODO add timer to cancel double tap
	if (self.state == gestureManager.IDLE_STATE) then
		
	elseif (self.state == gestureManager.DOUBLE_TAP_STATE) then
		--TODO check timer, cancel if expired
		if (self.taps_count >= 2) then
			--TODO multiple taps?
			--TODO WARNING currently we are just forwarding the update event as a double tap event
			--checking the taps positions, if they are the same withint a tollerance we have an actual double (or multiple tap)
			local count = #(self.tapsData)
			local minx = nil
			local maxx = nil
			local miny = nil
			local maxy = nil
			for i = 1, count do--in self.tapsData do
				if(minx == nil) then
					minx = self.tapsData[i].x
				else
					minx = math.min(self.tapsData[i].x, minx)
				end
				if(maxx == nil) then
					maxx = self.tapsData[i].x
				else
					maxx = math.max(self.tapsData[i].x, maxx)
				end

				if(miny == nil) then
					miny = self.tapsData[i].y
				else
					miny = math.min(self.tapsData[i].y, miny)
				end
				if(maxy == nil) then
					maxy = self.tapsData[i].y
				else
					maxy = math.max(self.tapsData[i].y, maxy)
				end
			end			
			
			if((maxx - minx) > gestureManager.DOUBLE_TAP_DISTANCE_TOLLERANCE * UI_SCALE or (maxy - miny) > gestureManager.DOUBLE_TAP_DISTANCE_TOLLERANCE * UI_SCALE) then
				--do nothing the taps are too scattered
				dbg.print("!!!!! WARNING taps are too scattered !!!!")
				self:changeState(gestureManager.IDLE_STATE)--timer expired
			else
				dbg.print("******** DOUBLE TAP EVENT ********")
				system:sendEvent("doubleTap", {x=(maxx + minx) * 0.5, y=(maxy + miny)*0.5})--just a rough average
				--system:sendEvent("doubleTap", self.tapsData)
				self:changeState(gestureManager.IDLE_STATE)
			end
		elseif(self.taps_count <= 0) then
			self:changeState(gestureManager.IDLE_STATE)
		else
			if ((system.gameTime - gestureManager.tapTime) > gestureManager.DOUBLE_TAP_MAX_TIME) then
				--dbg.print("!!!!! double tap timer expired !!!!!!")
				self:changeState(gestureManager.IDLE_STATE)--timer expired
			else
				--dbg.print(" **** TAP DETECTED ********* ")
			end
		end
	else
		--UNKNOWN STATE
	end
end

function gestureManager:tapConsumed()
	self:decrementTapsCount()
end

system:addEventListener( { "update", "touch" }, gestureManager)
