-- Fix the audio output speed (user function)
CMDFixAudio()

RegisterOnQuitCallBack()

dofile("addons/QAppStore.lua")

ScoreLoopEnabled = false
if appStore:getStorePlatform() == quick.QAppStore.BLACKBERRY_APP_WORLD then
    ScoreLoopEnabled = true
end

-- Setup scoreloop as soon as we can
if ScoreLoopEnabled then
    ScoreLoopSync = false
    ScoresLoaded = false
    dofile("addons/QScoreLoop.lua")
    function ScoreLoopCallback(event)
        if event.phase == "loadcontext" then
            if event.succeeded then
                ScoreLoopSync = true
                if ScoresLoaded then
                    MergeScoreLoopScores()
                end
            end
    --        dbg.print("Scoreloop context loaded")
    --    elseif event.phase == "savecontext" then
    --        dbg.print("Scoreloop context saved")
        end
    end
    system:addEventListener("ScoreLoop", ScoreLoopCallback)

    ScoreLoop:loadContextFromCloud()
end

-- Scoreloop end

G_BACK_KEY_VALUE = 126

DEFAULT_UI_WIDTH = 768
DEFAULT_UI_HEIGHT = 1024

CMW_G_DEFAULT_ALL_LEVEL_UNLOCKED	= false
CMW_G_SHOW_DEBUG_BUTTONS			= false


-- We cope with aspect ratios between 4:3 and 16:9
-- For squarer screens, we use the width. For taller screens, the height.
-- This is set up so we get a scale of 1 for ipad, iphone4 and iphone5
local x = director.displayWidth / 640
local y = director.displayHeight / 1024
UI_SCALE = (x < y and x or y)

function openFileForInts(filename)
	file = io.open(filename, "wb")
	if file==nil then return nil end
	obj = {}
	obj.file = file
	obj.hash = 123
	obj.write = function(self, value)
		self.file:write(math.floor(value).."\n")
		self.hash = (self.hash*7801 + value*33) % 0x100000000
	end
	obj.close = function(self)
		-- just to make the hash harder to guess
		self:write(math.random(10000,40000))
		self:write(math.random(100000,400000))
		self:write(self.hash)
		self.file:close()
	end
	obj.file:write("\255\254") -- Start file with UTF-16 header, just to confuse anyone opening it as a text file :-)
	return obj
end

function readIntsFromFile(filename)
	local file = io.open(filename, "rb")
	if file == nil then return nil end
	local data = file:read("*all")
	file:close()

	if string.sub(data,1,2) ~= "\255\254" then return nil end

	local out = {}
	string.gsub(data, "[0-9a-zA-Z]+", function(v) table.insert(out, v+0) end)

	if #out<3 then return nil end

	fileHash = 123
	for i=1,#out-1 do
		fileHash = (fileHash*7801 + out[i]*33) % 0x100000000
	end
	if fileHash ~= out[#out] then
		dbg.print("----HASH check failed!")
		dbg.print(fileHash.." vs "..out[#out])
		dialogManager:displayOkDialog(getString(S_IAP_DIALOG_TITLE_ERROR), getString(S_ERROR_SAVE_CORRUPT), getString(S_IAP_BUTTON_CLOSE_LABEL), -10000)--the dialog id is not important here
		return nil
	end

	-- remove the hash & salt numbers
	for i=1,3 do 
		table.remove(out, #out)
	end

	out.pos = 1
	out.read = function(self)
		self.pos = self.pos + 1
		return self[self.pos-1]
	end
	out.close = function() end -- dummy
	return out
end

--[[
	Some of the resolutions we want to support:
	  320  480	iphone 3
	  640  960	iphone 4 (retina)
	  640 1136	iphone 5 (retina)
	  768 1024	ipad
	 1536 2048	ipad 3
	  720 1280	Galaxy S III
	  480  800	Galaxy S
--]]

--dofile("MemoryReferenceDebugger.lua")


function FixupAppStoreStrings()

    if appStore:getStorePlatform() == quick.QAppStore.APPLE_APP_STORE then
        dbg.print( "QAppStore: Setting up for Apple App Store")
        -- This is the default.  No need to do anything

    elseif appStore:getStorePlatform() == quick.QAppStore.ANDROID_GOOGLE_PLAY_STORE then
        dbg.print( "QAppStore: Setting up for Google Play Store")
        S_MENU_MTX_STORE_ACCESSING = S_MENU_MTX_STORE_ACCESSING_GOOGLE
        S_MENU_MTX_STORE_FAILED = S_MENU_MTX_STORE_ACCESSING_GOOGLE
        S_ERROR_IAP_DISCONNECTED = S_ERROR_IAP_DISCONNECTED_GOOGLE
        S_ERROR_IAP_INVALID_ID = S_ERROR_IAP_INVALID_ID_GOOGLE

    elseif appStore:getStorePlatform() == quick.QAppStore.ANDROID_AMAZON_APP_STORE then
        dbg.print( "QAppStore: Setting up for Amazon App Store")
        S_MENU_MTX_STORE_ACCESSING = S_MENU_MTX_STORE_ACCESSING_AMAZON
        S_MENU_MTX_STORE_FAILED = S_MENU_MTX_STORE_ACCESSING_AMAZON
        S_ERROR_IAP_DISCONNECTED = S_ERROR_IAP_DISCONNECTED_AMAZON
        S_ERROR_IAP_INVALID_ID = S_ERROR_IAP_INVALID_ID_AMAZON

    elseif appStore:getStorePlatform() == quick.QAppStore.BLACKBERRY_APP_WORLD then
        dbg.print( "QAppStore: Setting up for Blackberry App World")
        S_MENU_MTX_STORE_ACCESSING = S_MENU_MTX_STORE_ACCESSING_BWORLD
        S_MENU_MTX_STORE_FAILED = S_MENU_MTX_STORE_ACCESSING_BWORLD
        S_ERROR_IAP_DISCONNECTED = S_ERROR_IAP_DISCONNECTED_BWORLD
        S_ERROR_IAP_INVALID_ID = S_ERROR_IAP_INVALID_ID_BWORLD

    else
        dbg.print( "QAppStore: Reported unknown app store")
    end
end

-- Do the splashscreen first, to reduce how long you see the blank screen for
dbg.print("locFile.lua")
dofile("locFile.lua")

dbg.print("addons/ui/defaultButtonLabelAnimation.lua")
dofile("addons/ui/defaultButtonLabelAnimation.lua")

dbg.print("addons/ui/spriteButton.lua")
dofile("addons/ui/spriteButton.lua")

dbg.print("addons/QDialogManager.lua")
dofile("addons/QDialogManager.lua")

dbg.print("UI/splashScreen.lua")
dofile("UI/splashScreen.lua")
local splash = splashScreen.create()

-- it seems like director:moveToScene doesn't behave correctly if you call it a newly created scene, so make a dummy one here
director:createScene()

-- Add global events + move to first scene
director:moveToScene(splash.scene)

currentWorldInfo = nil
currentLevelId = nil
worldInfos = {}

function getString(stringId)
	return stringId[1]
end

function g_manageKeyEvent(event)
	dbg.print("g_manageKeyEvent")
	dbg.printTable(event)

	if(event.keyCode == G_BACK_KEY_VALUE and event.phase == "released") then
		system:sendEvent("backKey", {})
	end
end

function FinishMainLoading()
	dbg.print("loading addons/ui/UIButton.lua")
	dofile("addons/ui/UIButton.lua")
	dbg.print("addons/ui/UIToggle.lua")
	dofile("addons/ui/UIToggle.lua")

	dbg.print("addons/ui/defaultButtonAnimation.lua")
	dofile("addons/ui/defaultButtonAnimation.lua")


	dbg.print("addons/ui/flashOnlyButtonAnimation.lua")
	dofile("addons/ui/flashOnlyButtonAnimation.lua")


    FixupAppStoreStrings()  -- This makes sure the right app store strings are being used

	dbg.print("gameScene.lua")
	dofile("gameScene.lua")
	dbg.print("UI/mainMenu.lua")
	dofile("UI/mainMenu.lua")
	dbg.print("UI/levelSelect.lua")
	dofile("UI/levelSelect.lua")
	dbg.print("UI/worldSelect.lua")
	dofile("UI/worldSelect.lua")

	dbg.print("tutorialsManager.lua")
	dofile("tutorialsManager.lua")
	dbg.print("gestureManager.lua")
	dofile("gestureManager.lua")
	dbg.print("facebook.lua")
	dofile("facebook.lua")
	dofile("UI/introCutscene.lua")
	dbg.print("introCutscene.lua")
	dofile("UI/outroCutscene.lua")
	dbg.print("outroCutscene.lua")
	dofile("UI/unlockFullGame.lua")
	dbg.print("unlockFullGame.lua")
	dbg.print("creditsScreen.lua")
	dofile("UI/creditsScreen.lua")

	dbg.print("billingManager.lua")
	dofile("billingManager.lua")
	
	dbg.print("loading hintsPurchaseManager.lua")
	dofile("hintsPurchaseManager.lua")


	dbg.print("main : loading complete")

	menu = mainMenu.create()
	game = gameScene.create()
	levelSelection = levelSelect.create()
	worldSelection = worldSelectScreen.create()
	introCutscene = introCutscene.create()
	outroCutscene = outroCutscene.create()
	unlockFullGame = unlockFullGame.create()
	creditsScreen = creditsScreen.create()
	editMode = false

	dbg.print("main loading Levels/worlds.txt")
	-- Init worlds and levels structure --
	local file = io.open("Levels/worlds.txt", "r")
	worldCount = 0
	while true do
		local worldName = file:read()
		if worldName == nil then
			break
		else
			worldCount = worldCount + 1
			worldInfos[worldCount] = {}
			worldInfos[worldCount].id = worldCount
			worldInfos[worldCount].name = string.gsub(worldName, "[\n\r]", "")
			worldInfos[worldCount].isLocked = false
		end
	end
	file:close()
	dbg.print("main loading Levels/worlds.txt...done")

	dbg.print("main loading levels infos")
	for i = 1,#worldInfos do
		file = io.open("Levels/"..worldInfos[i].name, "r")

		worldInfos[i].chapterNumber = i
		worldInfos[i].menuIcon = string.gsub(file:read(), "[\n\r]", "")
		worldInfos[i].menuIconLocked = string.gsub(file:read(), "[\n\r]", "")
		worldInfos[i].blockedTileIcon = string.gsub(file:read(), "[\n\r]", "")
		worldInfos[i].tileSetRevealed = string.gsub(file:read(), "[\n\r]", "")
		worldInfos[i].gameTopBG = string.gsub(file:read(), "[\n\r]", "")
		worldInfos[i].gameBotBG = string.gsub(file:read(), "[\n\r]", "")
		worldInfos[i].gameLeftBG = string.gsub(file:read(), "[\n\r]", "")
		worldInfos[i].gameRightBG = string.gsub(file:read(), "[\n\r]", "")
		worldInfos[i].gridEdgeTL = string.gsub(file:read(), "[\n\r]", "")
		worldInfos[i].gridEdgeL = string.gsub(file:read(), "[\n\r]", "")
		worldInfos[i].dustTrail = string.gsub(file:read(), "[\n\r]", "")
		worldInfos[i].gameOverlay = string.gsub(file:read(), "[\n\r]", "")
		worldInfos[i].cloudColor = { }
		worldInfos[i].cloudColor.r = string.gsub(file:read(), "[\n\r]", "")
		worldInfos[i].cloudColor.g = string.gsub(file:read(), "[\n\r]", "")
		worldInfos[i].cloudColor.b = string.gsub(file:read(), "[\n\r]", "")
		worldInfos[i].cloudColor.r = tonumber(worldInfos[i].cloudColor.r)
		worldInfos[i].cloudColor.g = tonumber(worldInfos[i].cloudColor.g)
		worldInfos[i].cloudColor.b = tonumber(worldInfos[i].cloudColor.b)
		worldInfos[i].propsFile = "Levels/" .. string.gsub(file:read(), "[\n\r]", "")
	
		worldInfos[i].levels = {}
		local levelCount = 0
		while true do
			local levelName = file:read()
			if levelName == nil then
				break
			else
				levelCount = levelCount + 1
				local levelinfo = {}
				levelinfo.name = string.gsub(levelName, "[\n\r]", "")
				levelinfo.numStars = 0
			
				if CMW_G_DEFAULT_ALL_LEVEL_UNLOCKED then
					levelinfo.isLocked = false
				else
					levelinfo.isLocked = true
				end
			
				levelinfo.hintUsed = false
				worldInfos[i].levels[levelCount] = levelinfo
			end
		end
		file:close()
	end

	--dbg.printTable(worldInfos)
	--setting all of the world as locked by default
	if not CMW_G_DEFAULT_ALL_LEVEL_UNLOCKED then
		for i=1, #worldInfos do
			worldInfos[i].isLocked = true
		end
	end

	dbg.print("main loading levels infos...done")
	--setting world 1 unlocked by default
	worldInfos[1].isLocked = false
	worldInfos[1].levels[1].isLocked = false

	--dbg.print("main hintsPurchaseManager:initializeDefaultData()")
	--hintsPurchaseManager:initializeDefaultData()

	--dbg.print("main hintsPurchaseManager:initializeDefaultData()...done")
	-- Load saved data
	--g_loaddata(hintsPurchaseManager)
	g_loaddata()

	dbg.print("main load Save/save.txt...done")

	director:setCurrentScene(splash.scene)

	if(audio.musicVolume > 0) then
		audio:playStream("Assets/Audio/MUS_FrontEnd.mp3", true)
	end

	--audio.musicVolume = 0
	--audio.sfxVolume = 0
	system:addEventListener("key", g_manageKeyEvent)
	system:addEventListener("goToMainMenu",goToMainMenu)
	system:addEventListener("goToCredits",goToCredits)
	system:addEventListener("onWorldSelect",worldSelect)
	system:addEventListener("onStartGame",startGame)
	system:addEventListener("onStartFirstGame",startFirstGame)
	system:addEventListener("onPlay", playGame)
	system:addEventListener("onEdit", editGame)
	system:addEventListener("onBackToMainMenu",goToMainMenu)
	system:addEventListener("onBackToLevelSelect",goToSelectionScreen)
	system:addEventListener("onBackToWorldSelect",goToWorldScreen)
	system:addEventListener("onOutro",goToOutroScreen)
	system:addEventListener("onGoToNextLevel",goToNextLevel)
	system:addEventListener("onLevelComplete", completeLevel)
	system:addEventListener("goToUnlockScreen",goToUnlockScreen)
    -- Userfunc to call QAdds::update();
	system:addEventListener("update",QAddsUpdate)

	if(g_billingAvailable) then
		if(not g_retrieveProductsInfoData()) then
			dbg.print("=========================>>>>> WARNING: unable to retrieve products data")
		else
			dbg.print("=========================>>>>> successfully launched product info data retrieving")
		end
	else
		dbg.print("=========================>>>>> WARNING: Billing unavailable")
	end

	dbg.print("main complete")
end

function playGame(event)
	editMode = false
	
	-- If we've not completed the first level yet...
	if worldInfos[1].levels[1].numStars == 0 and not CMW_G_DEFAULT_ALL_LEVEL_UNLOCKED then

		-- Play the intro cutscene
		dbg.print("Playing intro cutscene")
		director:moveToScene(introCutscene.scene)
		
	-- Otherwise
	else
		director:moveToScene(worldSelection.scene)	
	end
end

function startGame(event)
	currentLevelId = event.levelId
	game:start(currentLevelId, levelSelection.worldInfo, editMode)
end

function startFirstGame(event)
	currentLevelId = 1
	currentWorldInfo = worldInfos[1]
	levelSelection.worldInfo = currentWorldInfo
	game:start(currentLevelId, levelSelection.worldInfo, editMode)
end

function editGame(event)
	editMode = true
	director:moveToScene(worldSelection.scene)
end

function goToCredits(event)
	director:moveToScene(creditsScreen.scene)
end

function goToSelectionScreen(event)
	if(not audio:isStreamPlaying()) then
		if(audio.musicVolume > 0) then
			audio:playStream("Assets/Audio/MUS_FrontEnd.mp3", true)
		end
	end
	-- If we're not entering from the game...
	if event.lastLevelPlayed == nil then
	
		dbg.print("event.lastLevelPlayed = nil")

		-- Default to the first page
		event.lastLevelPlayed = 1
	
		-- If we've not completed this chapter yet, show the next level we should play to progress
		if levelSelection.worldInfo.levels[#levelSelection.worldInfo.levels].numStars == 0 then
		
			-- Cycle through all the levels in this chapter
			for i = #levelSelection.worldInfo.levels, 1, -1 do
			
				-- If we've got any stars on this level, the next level to play is the one after it
				if levelSelection.worldInfo.levels[i].numStars > 0 then
					event.lastLevelPlayed = i + 1
					break
				end
			end
		end
	end

	dbg.print("event.lastLevelPlayed = " .. event.lastLevelPlayed)
	
	levelSelection.worldInfo.lastLevelPlayed = event.lastLevelPlayed
	
	director:moveToScene(levelSelection.scene)
	game.gameGrid:resetGrid()
end

function worldSelect(event)
dbg.print("worldSelect");
	currentWorldInfo = event.worldInfo
	levelSelection.worldInfo = event.worldInfo
--	director:moveToScene(levelSelection.scene)
	goToSelectionScreen(event)	-- use function to set correct starting page
dbg.print("worldSelect4");
end

function goToUnlockScreen(event)
	director:moveToScene(unlockFullGame.scene)
end

function goToWorldScreen(event)
	if(event.fromGame or event.fromOutro) then
		LoadFrontEndAssets()
	end
	event.fromGame = nil
	event.fromOutro = nil
	director:moveToScene(worldSelection.scene)
end

function goToOutroScreen(event)
	director:moveToScene(outroCutscene.scene)
end

function goToMainMenu(event)
	director:moveToScene(menu.scene)
end

function LoadOptions()
	file = io.open("save/options.bin", "rb")
    if file == nil then
        SaveOptions()
        return
    end

   	audio.musicVolume = file:read(1)
	audio.sfxVolume = file:read(1)
end

function SaveOptions()
	file = io.open("save/options.bin", "wb")
    if file == nil then
        return
    end
	file:write(audio.musicVolume)
	file:write(audio.sfxVolume)
    file:close()
end

function g_saveData()
	local result = true

	file = openFileForInts("save/save.bin")
	if(file ~= nil)  then
		file:write(101) -- version number
		--file:write(appPurchaseData:getHintsCount())
		local hintsCountString = tostring(g_getHintsCount())
		dbg.print("saving hints count " ..hintsCountString)
		file:write(hintsCountString)
		--file:write(appPurchaseData.fullGameUnlocked and 1 or 0)
		local fullGameUnlockedString = (g_isFullGameUnlocked() and 1 or 0)
		dbg.print("saving fullGameUnlockedString " ..fullGameUnlockedString)
		file:write(fullGameUnlockedString)
		for i = 1,#worldInfos do
			if worldInfos[i].isLocked == false then
				file:write(i)
				for j = 1,#worldInfos[i].levels do
					if worldInfos[i].levels[j].isLocked == false then
						file:write(j)
						file:write(worldInfos[i].levels[j].numStars)
						file:write(worldInfos[i].levels[j].hintUsed == true and 1 or 0)
					end
				end
				file:write(0) --mark end of level list
			end
		end
		file:write(0) --mark end of world list
		file:close()
	else
		result = false
	end
	dbg.print("g_saveData ... done")
    SaveToScoreLoop()

	return result
end

--[[function string:split(sep)
        local sep, fields = sep or ":", {}
        local pattern = string.format("([^%s]+)", sep)
        self:gsub(pattern, function(c) fields[#fields+1] = c end)
        return fields
end]]

function Split(str, delim, maxNb)
    -- Eliminate bad cases...
    if string.find(str, delim) == nil then
        return { str }
    end
    if maxNb == nil or maxNb < 1 then
        maxNb = 0    -- No limit
    end
    local result = {}
    local pat = "(.-)" .. delim .. "()"
    local nb = 0
    local lastPos
    for part, pos in string.gfind(str, pat) do
        nb = nb + 1
        result[nb] = part
        lastPos = pos
        if nb == maxNb then break end
    end
    -- Handle the last field
    if nb ~= maxNb then
        result[nb + 1] = string.sub(str, lastPos)
    end
    return result
end

function MergeScoreLoopScores()

    if not ScoreLoopEnabled then
        return
    end

    local save = false

    dbg.print("Merging scores from ScoreLoop")
    if ScoreLoopSync then
        local unlock_next_level = false
        for i,v in ipairs(worldInfos) do
            dbg.print("Merging world "..i)
            local stars = ScoreLoop:getString("World"..i.."Stars")
            dbg.print("World"..i.."Stars: "..stars)
            if stars ~= "" then
                stars = Split(stars, ',')

                local world_locked = v.isLocked

                for j,k in ipairs(stars) do
                    if v.levels[j] ~= nil then
                        local value = tonumber(k)

                        if unlock_next_level then
                            v.levels[j].isLocked = false
                            world_locked = false
                            unlock_next_level = false
                        end

                        if value > v.levels[j].numStars then
                            v.levels[j].numStars = value
                            v.levels[j].isLocked = false
                            world_locked = false
                            save = true
                            unlock_next_level = true
                        end

                    end
                end

                v.isLocked = world_locked
            end
        end
    end

    dbg.print("Merge complete")
    
    if save then
        g_saveData()
    end
end

function SaveToScoreLoop()

    if not ScoreLoopEnabled then
        return
    end

    dbg.print("Saving to ScoreLoop")
    if ScoreLoopSync then
        
        for i,v in ipairs(worldInfos) do
            dbg.print("Saving world "..i)

            local stars = {}

            for j,k in ipairs(v.levels) do
                stars[j] = k.numStars
            end

            stars = table.concat(stars, ",")

            ScoreLoop:setString("World"..i.."Stars", stars)
            dbg.print("World"..i.."Stars: "..stars)
        end

        ScoreLoop:saveContextToCloud()
    end
end

--function g_loaddata(hintsPurchaseManager)
function g_loaddata()
    -- Load the options file first
    LoadOptions()

	file = readIntsFromFile("save/save.bin")
	if file==nil then
        ScoresLoaded = true
        MergeScoreLoopScores()
		return
	end

	local version = file:read() -- TODO Wat?!
	local hintsCountString = tostring(file:read())
	dbg.print("loaded hintsCountString " ..hintsCountString)
	local fullGameUnlockedStr = tostring(file:read())
	dbg.print("loaded fullGameUnlockedStr " ..fullGameUnlockedStr)
	while true do
		local i = file:read()
		if i==0 then break end
	
		while true do
			local j = file:read()
			if j==0 then break end

			worldInfos[i].isLocked = false

			local numStars = file:read()
			local hintUsed = (file:read()==1)
			worldInfos[i].levels[j].isLocked = false
			worldInfos[i].levels[j].numStars = numStars
			worldInfos[i].levels[j].hintUsed = hintUsed

		end
	end
	file:close()	
	g_PurchasableProductsDataLoaded(tonumber(fullGameUnlockedStr) , tonumber(hintsCountString))
	dbg.print("*******************************")
	dbg.print("******************************=======================")
	dbg.print("game unlock status " ..tostring(g_isFullGameUnlocked()))
	dbg.print("hints count " ..tostring(g_getHintsCount()))
	dbg.print("******************************=======================")
	dbg.print("*******************************")
	--hintsPurchaseManager:dataLoaded(hintsCountString, fullGameUnlockedStr)
    ScoresLoaded = true
    MergeScoreLoopScores()
end

-- Called whenever a level is finished
function completeLevel(event)
	if event.numStars > currentWorldInfo.levels[currentLevelId].numStars then
		currentWorldInfo.levels[currentLevelId].numStars = event.numStars
	end
	if currentLevelId == #currentWorldInfo.levels then
		for i = 1,#worldInfos do
			if worldInfos[i] == currentWorldInfo then
				if i < #worldInfos then
					worldInfos[i + 1].isLocked = false
					worldInfos[i + 1].levels[1].isLocked = false
				end
			end
		end
	else
		currentWorldInfo.levels[currentLevelId + 1].isLocked = false
	end
	g_saveData()
end

function goToNextLevel(event)	
	dbg.print("main : goToNextLevel ")
	local nextLevelAvailable = true
	currentLevelId = currentLevelId + 1
	
	-- If we've reached the end of a chapter...
	if (currentLevelId > table.getn(currentWorldInfo.levels)) then
		currentLevelId = 1
		if(currentWorldInfo.id < table.getn(worldInfos)) then
			currentWorldInfo = worldInfos[currentWorldInfo.id + 1]
			levelSelection.worldInfo = currentWorldInfo
--			dbg.print("main : goToNextLevel - going to world " .. currentWorldInfo.id .. ", level 1")
		else
			nextLevelAvailable = false
		end
		
		-- If the next chapter exists...
		if(nextLevelAvailable == true) then
		
			-- If we've bought the full game...
			--if(hintsPurchaseManager:isFullGameUnlocked())then
			if(g_isFullGameUnlocked()) then
				-- Go to the first level of the next chapter
--				dbg.print("main : goToNextLevel - full game is unlocked, going to next level")
				game:restart(currentLevelId, levelSelection.worldInfo)
			else
				-- Otherwise, take the player to the upsell screen
--				dbg.print("main : goToNextLevel - full game is locked, going to upsell screen")
				game:backToUpsellScreen()
			end
			
		-- If we've reached the end of the game, play the outro cutscene
		else
--			dbg.print("main : goToNextLevel - game completed, playing intro cutscene")
			game:startShowOutroSequence()
		end
	
	-- Otherwise, just go to the next level
	else
--		dbg.print("main : goToNextLevel - going to world " .. currentWorldInfo.id .. ", level 1")
		game:restart(currentLevelId, levelSelection.worldInfo)
	end
end

-- PJC testing Ad suport
--[[quick.QAds:init("myaddidhere")
quick.QAds:setNewAdInterval(30)
quick.QAds:setAdProvider("MillennialMedia")
quick.QAds:setNumAdsVisible(1)
--quick.QAds:requestNewAd("MillennialMedia")
quick.QAds:start()]]
-- PJC testing end

-- This function loads all assets that are common to both the front end and in-game
function LoadCommonAssets()

    dbg.print("Loading common assets")

    -- Load the game font and st it to be the default font
    GAME_FONT = director:createFont("fonts/BlueTreatment.fnt")
	GAME_FONT_TITLE = director:createFont("fonts/GreenTreatment.fnt")

    GAME_FONT_COLOR = {r=255,g=255,b=255}
	GAME_FONT_COLOR_GREEN = {r=128,g=255,b=128}

    -- Paul's little hack to make this the default font
    director.defaultFont = GAME_FONT

    -- These are the SFX common to both the front end and in-game
    audio:loadSound("Assets/Audio/SFX_Back.raw")
    audio:loadSound("Assets/Audio/SFX_Confirm.raw")

    bFrontEndAssetsLoaded = false
end

function LoadFrontEndAssets()
	dbg.print("LoadFrontEndAssets: Entry")
    if not bFrontEndAssetsLoaded then
		bFrontEndAssetsLoaded = true

        dbg.print("LoadFrontEndAssets: Loading")

        local old_scene = director:getCurrentScene()
        director:setCurrentScene(menu.scene)

        -- Pre-cache SFX
        audio:loadSound("Assets/Audio/SFX_Swipe.raw")
        menu:LoadAssets()
        worldSelection:LoadAssets()
        levelSelection:LoadAssets()

        director:setCurrentScene(old_scene)
    end
	dbg.print("LoadFrontEndAssets: exit")
end

function UnloadFrontEndAssets()

    if bFrontEndAssetsLoaded then
        bFrontEndAssetsLoaded = false

        dbg.print("Unloading front end assets")
		--dbg.print(""..dbg.getProcessedCallstack())
        -- Remove Pre-cached SFX
        audio:unloadSound("Assets/Audio/SFX_Swipe.raw")
		
        menu.scene:releaseResources()
        menu:UnloadAssets()
        worldSelection:UnloadAssets()
        levelSelection:UnloadAssets()

        -- Perform GC so we can dump that lot properly
        collectgarbage("collect")
        director:cleanupTextures()

--        dbg.printTable( QEvent.statics)
    end

end

preLoaderComplete = false


function preLoadByPhase()

    -- Preload on the global scene
    local old_scene = director:getCurrentScene()
    director:setCurrentScene(nil)
    local ret_val = game.gameGrid:preLoad()
    director:setCurrentScene(old_scene)

    preLoaderComplete = ret_val

    return ret_val
end

function preLoaderTick(event)
    local ret_val = preLoadByPhase()
    if ret_val == true then
        -- Trick the timer into terminating
        event.timer.iterations = 1
        event.timer.doneIts = 1
    end
end

function startPreLoader()
    preLoadTimer = system:addTimer(preLoaderTick, 0.2, 0)
end

function stopPreLoader()
    if not preLoaderComplete then
        preLoadTimer:cancel()
        local ret_val
        repeat
            ret_val = preLoadByPhase()
        until ret_val == true
    end

end
